% Aggregation:  Estimates intraspecific (J) and interspecific aggregation (C)coefficients and the realtive strength
%   of intraspecific aggreagtion versus interspecific aggreation (A).
%
%   Usage: [J,Jpr,C,Cpr,A,Apr] = Aggreagtion(X,iter,model)
%
%       X = [sites x species] abundance matrix
%       iter = {optional}, specifies how many iterations to perform {default = 1000}
%       model = optional, specifies which null model to use
%           1 = Sample size is equiprobable and species abundance is equiprobable 
%           2 = Sample size is equiprobable and species abundance is fixed 
%           3 = Sample size is fixed & species abundance is equiprobable 
%           4 = Sample size is proportional & species abundance is fixed
%           5 = Sample size is fixed & species occurrence is proportional 
%           6 = Sample size is proportional & species abundance is equiprobable
%           7 = Sample size is equiprobable & species abundance is proportional
%           8 = Sample size is proportional & species abundance is proportional
%           9 = Sample size is fixed & species abundance is fixed
%       ---------------------------------------------------------------------------------------------------
%       J = [1 x species] intraspecifc aggregation coefficients
%       Jpr = [1 x species] associated p-values
%       C = [species x species -- upper triangular] interspecific aggregation coefficients
%       Cpr = [spcies x species] assocaited p-values (lower triangular = 2 --- not of interest!!!!)
%       A = [species x species -- upper triangular] realtive strength of intraspecific interaction
%       Apr = [spcies x species] assocaited p-values (lower triangular = 2 --- not of interest!!!!)
%
%   Chris L. Higgins 5/1/2008
%
%   Based on:
%       Ives, A.R. 1988. Aggregation and the coexistence of competitors. Annales Zoologici Fennici 25: 75-88.
%       Sevenster, J.G. 1996. Aggreagtion and coexistence. I. Theory and analysis. Journal of Animal Ecology 65: 297-307
%

function [J,Jpr,C,Cpr,A,Apr] = Aggregation(X,iter,model)

    if(nargin < 2)
        iter = [];
    end;
    
    if(isempty(iter))
        iter = 1000;
    end;

    [r,c] = size(X);                                                % Determine number of sites and species
 
% Estimate intraspecifc aggregation
    J = zeros(1,c);
    for k = 1:c
        m1 = mean(X(:,k));                                          % mean number of individuals per site
        v1 = var(X(:,k));                                           % variance in number of individuals per site
        J(k) = ((v1/m1)-1) / m1;                                    % equation 2 in Ives 1988
    end;
    
% Estimate interspecific aggreagtion
    C = zeros(c,c);
    for k = 1:c-1
        for i = k+1:c
            m1 = mean(X(:,k));                                      % mean number of individuals of species 1 per site
            m2 = mean(X(:,i));                                      % mean number of individuals of species 2 per site
            cv = cov(X(:,[k i]));                                   % covariation in number of individuals per site
            C(k,i) = cv(2,1)/(m1*m2);                               % equation 4 in Ives 1988
        end;
    end;

% Relative strength of intraspecific aggregation
    A = zeros(c,c);
    for k = 1:c-1
        for i = k+1:c
            A(k,i) = ((1 + J(k)) * (1 + J(i))) / (1 + C(k,i))^2;    % equation 25 in Sevenster 1996
        end;
    end;

% Everything beneath this points invovles randomization routines
    if(iter > 0)
        Jnull = zeros(1,c,iter);                                                % Allocate output variables
        Cnull = zeros(c,c,iter);
        Anull = zeros(c,c,iter);
        
        for j = 1:iter
            Null = randomnull(X,model);                                         % create null matrix based on specific model
        
        % Estimate intraspecifc aggregation   
            for k = 1:c
                m1 = mean(Null(:,k));                                           % mean number of individuals per site
                v1 = var(Null(:,k));                                            % variance in number of individuals per site
                Jnull(1,k,j) = ((v1/m1)-1) / m1;                                % equation 2 in Ives 1988
            end;
    
        % Estimate interspecific aggreagtion
            for k = 1:c-1
                for i = k+1:c
                    m1 = mean(Null(:,k));                                       % mean number of individuals of species 1 per site
                    m2 = mean(Null(:,i));                                       % mean number of individuals of species 2 per site
                    cv = cov(Null(:,[k i]));                                    % covariation in number of individuals per site
                    Cnull(k,i,j) = cv(2,1)/(m1*m2);                             % equation 4 in Ives 1988
                end;
            end;

        % Relative strength of intraspecific aggregation
            for k = 1:c-1
                for i = k+1:c
                    Anull(k,i,j) = ((1+Jnull(1,k,j))*(1+Jnull(1,i,j)))/(1+Cnull(k,i,j))^2;  % equation 25 in Sevenster 1996
                end;
            end;
        end;
    end;

% Assess signicance for each variable
    Jpr = ones(1,c)*2;
    for i = 1:c
        Jpr(i) = pvalue(J(i),Jnull(1,i,:),3);
    end;

    Cpr = ones(c,c)*2;
    Apr = ones(c,c)*2;
    for k = 1:c-1
        for i = k+1:c
            Cpr(k,i) = pvalue(C(k,i),Cnull(k,i,:),3);
            Apr(k,i) = pvalue(A(k,i),Anull(k,i,:),3);
        end;
    end;
    
return;