% DFA:	Discriminat function analysis 
%
%	USAGE:	[load,pervar,scores,float,ciload,cipervar] = dfa(x,grps,ndf,loadtype,fx,iter,conf) 
%			
%
%				load = 		a [p x ndf] column vector containing the vector corelations or
%								the eigenvalues
%				per_var = 	a [p x 1] column vector containing the percent variance 
%								accounted for by the corresponding loadings
%				scores =			the discriminant scores of the original data				
%				float =		the discriminat scores for the floated data
%				ci_load = 	a [p^p x 2] matrix containing the confidence	limits of the loadings
%									column 1 = the lower limit
%									column 2 = the upper limit
%				ci_per = 	a [p x 2] matrix containing the confidence limits for the per_var
%									column 1 = the lower limit
%									column 2 = the upper limit
%				----------------------------------------------------------
%				x  = 			a [n x p] data matrix
%				grps = 		a [n x 1] grouping vector
%				ndf = 		optional, the number of discrininant functions {default = n-1}
%				loadtype = 	optional, the scaling for the loadings:
%									0 = vector correlations
%				iter = 		the number of iterations {default = 0}
%				fx = 			a [m x p] matrix of floating data {default = []}
%
%	Chris Higgins 4/03/2000
%

function [load,per_var,scores,float,ci_load,ci_per]...
   = dfa(x,grps,ndf,loadtype,fx,iter,conf)

	[r,c] = size(x);
   len_grps = length(grps);
   len_uni = length(unique(grps));
   
   if(nargin < 3)
      ndf = [];
   end;
   if(nargin < 4)
      loadtype = [];
   end;
   if(nargin < 5)
      fx = [];
   end;
   if(nargin < 6)
      iter = [];
   end;
   if(nargin < 7)
      conf = [];
   end;
   
   if(isempty(iter))
      iter = 0;
   end;
   if(isempty(ndf))
      ndf = len_uni - 1;
      if(ndf == 1)
          ndf = 2;
      end;
   end;
   if(isempty(loadtype))
      loadtype = 0;
   end;
   if(isempty(conf))
      conf = 0.95;
   end;
   
   G = design(grps); 										% Generate design matrix
   grp_means = (G' * G) \ (G' * x);    				% Calculate group means
   grp_resid = x - (G * grp_means); 					% Calculate group residuals
   W = grp_resid' * grp_resid;							% Calculate Within groups matrix
   
	tot_resid = x - (ones(len_grps,1)* mean(x)); 
  	T = tot_resid' * tot_resid;  
   B = T - W; 													% Calculate Between groups matrix
     
   [eigvects,eigvals] = eiganalysis([],[],B,W);		% Perform eigenvector analysis
   
   per_var = (eigvals / sum(eigvals)) * 100;			% Calculate percent variance
   per_var = per_var(1:ndf,:);
   
   scores = x * eigvects(:,1:ndf);						% Generate discriminant scores
   z = zscores(scores);   
   
   if(~isempty(fx))         
      float = fx * eigvects(:,1:ndf);
      float = zscores(float);
   else
   	float = [];
  	end;
     
	if(loadtype == 0)
      load = corr(x,z);
   else
      load = eigvects(:,1:ndf);
   end;

	for h = 1:ndf            
    if (sum(load(:,h)) < 0)
      load(:,h) = -load(:,h);
      z(:,h) = -z(:,h);
      if (~isempty(float))
        float(:,h) = -float(:,h);
      end;
    end;
  end;

 return;

	
