% Entropy:  This function estimates the effective number of species derived from traditional species richness and Shannon diversity values.
%           This function is based on work by Lou Jost; in fact, the formulas were directly obtained from Entropy and diversity published in Oikos.
%
%   Usage: [S,H,E,D,B] = Entropy(X,standard)
%
%   X = [sites x species] matrix of species abundances
%   standard = optional, if passed specifies whether to report traditional measures of diversity
%   -------------------------------------------------------------------------------------------------------
%   S = [sites x 1] vector of entropy values for species richness
%   H = [sites x 1] vector of entropy values for the Shannon diversity index
%   E = [sites x 1] vectore of entropy values for Camargo evenness (equals camargo Diversity)
%   D = [sites x 1] vector of entropy values for the Simpson diversity index
%   B = [sites x 1] vector of entropy values for the Berger-Parker dominance index
%
%   Chris Higgins 3/19/2009
%
%   Jost, L.  2006.  Entropy and diversity.  Oikos 113: 363-375.
%      

function [S,H,E,D,B] = Entropy(X,standard)

    if(nargin < 2)
        standard = [];
    end;
    
    if(isempty(standard))
        standard = 0;
    end;

    [r,c] = size(X);                                            % Determine the number of sites
    if(r > 1 & c == 1)                                          % Convert column vector to row vector
        X = X';
        [r,c] = size(X);
    end;
    
    S = zeros(r,1);                                             % Allocate column vector for diversity indices
    H = zeros(r,1);                                             
    D = zeros(r,1);
    B = zeros(r,1);

    for i = 1:r                                                 % Cycle through each site to estimate entropy values
        ind = find(X(i,:) > 0);                                 % Find all non-zero entries
        if(isempty(ind))    
            S(i) = 0;
            H(i) = 0;
            D(i) = 0;
            B(i) = 0;
        else;
            p = X(i,ind)./sum(X(i,ind));                        % Convert absolute abundance distribution into a relative distribution
            D1=[];
            D2=[];
            D3=[];
            e = 0;
            for j = 1:length(ind)
                D1(j) = p(j)^0;
                D2(j) = p(j)*log(p(j));
                D3(j) = p(j)^2;
                for k = j+1:length(ind)
                    e = e + (abs(p(j) - p(k))/length(ind));
                end;
            end;
            if(standard == 0)
                S(i) = sum(D1);                                     % Estimate species richness
                H(i) = exp(-sum(D2));                               % Estimate the entropy value for the Shannon diversity index
                E(i) = (1-e)*length(ind);                           % Estimated the entropy value for Camargo evenness (Camargo Diversity)
                D(i) = 1/sum(D3);                                   % Estimate the entropy value for the Simpson diversity index
                B(i) = 1/max(p);                                    % Estimate the entropy value for the Berger-Parker index
            else
                S(i) = sum(D1);                                     % Estimate species richness
                H(i) = (-sum(D2));                                  % Estimate the traditional Shannon diversity index
                E(i) = 1-e;                                         % Estimated the traditional Camargo evenness
                D(i) = sum(D3);                                     % Estimate the traditional Simpson diversity index
                B(i) = 1/max(p);                                    % Estimate the traditional Berger-Parker index 
            end;
        end;
    end;
  
return;
    