% ExpPopGrowthFit:  This function estimates the mean and variance of the intrinsic rate of increase for exponential population growth.
%
%   Usage: N = ExpPopGrowthFit(Obs,sto,doplot)
%
%   Obs = [1 x t] vector of population sizes over time 
%   sto = optional, specified whether or not to produce deterministic {default = 0} or stochastic (~=0) model
%   doplot = optional, specifies whether or not to produce figure of observed (red) and expected (blue) values {default = 0}
%   ---------------------------------------------------------------------------------------------------------------------------
%   param = [1 x 2] column 1 = mean r & column 2 = variance of r
%   Exp = [1 x t] vector of predicted population sizes at time t
%
%   Chris Higgins 2/1/2008
%

function [param,Exp] = ExpPopGrowthFit(Obs,sto,doplot)

    if(nargin < 2)                                                          % Allocate input arguments
        sto = [];
    end;
    if(nargin < 3)                                                         
        doplot = [];
    end;
    
    if(isempty(sto))
        sto = 0;
    end;
    if(isempty(doplot))
        doplot = 0;
    end;

    t = length(Obs);                                                        % Determine the length of time involved with the observed abundances
    N0 = Obs(1);                                                            % Set the initial population size to the first obeserved value

    if(sto ~= 0)
        param(1) = 0;                                                       % Set initial mean of r to zero
        param(2) = 0;                                                       % Set initial variance of r to zero 
        param = fminsearch('ExpPopGrowthFitf',param,[],Obs);                % Optimization routine to estimate mean and variance of r
    else;
        param = 0;
        param = fminsearch('ExpPopGrowthFitf',param,[],Obs);                % Optimization routine to estimate mean of r
        param(2) = 0;
    end;
    
    for i = 1:t                                                             % Cycle through each time step
        rBar = normrnd(param(1),param(2));                                  % Sample a random number from a Normal dist. with a mean of 'r' and a variance of 'rVar'           
        Exp(i) = N0*exp(rBar*i);                                            % Estimate abundance for each unit of time
    end;
    
    if(doplot ~= 0)
        plot([1:t],Obs,'r');                                                % Plot Observed values in red
        hold on;
        plot([1:t],Exp,'b');                                                % Plot Expexted values in blue
    end;
    
return; 