% MIDSAGPTS:    Given a set of coordinates and a matrix of boundary points, MIDSAGPTS finds the major axis
%               of the coordinates and the point of intersection between the boundary points and the
%               major axis.  This function was designed to find midsagital pseudolandmarks for 
%               studies of bilateral asymmetry.
%
%       Usage:  new_crds = midsagpts(crds,bndpts)
%
%               crds = a [nx2] matrix of point coordinates
%               bndpts = a [mx2] matrix of boundary point indices to be intersected
%                   column 1 = index for right point : column 2 = index for left point
%               --------------------------------------------------------------------------------------
%               new_crds = a [mx2] matrix of midsagital point coordinates
%
%   Chris Higgins   10/17/2001

%   Requires: 

function [new_crds, crds] = midsagpts(crds,pt)

  
    if(nargin < 2)
        error('MIDSAGPTS: Must supply a matrix specifying which set of crds to intersect with the major axis.');
    end;
    
    [rx,cx] = size(crds);                               % Determine size of input arguments
    [rpt,cpt] = size(pt);

    fin = isfinite(crds(:,1));                           % eliminate nan for regression
    ind = find(fin == 1);
    C = crds(ind,:);
    
    new_crds = zeros(rpt,2);                            % Allocate new coordinates matrix
    
    b = majaxis(C(:,1),C(:,2));                         % Perform major axis regression

    max1 = 0;                                           % Calculate intercept points on regression line
    max2 = -b(1,2) / b(1,1);
    may1 = b(1,2);
    may2 = 0;

    l = [may1 - may2, max2 - max1, (max1 * may2) - (max2 * may1)];
       
    for i = 1:rpt           
      x1 = crds(pt(i,1),1);
      x2 = crds(pt(i,2),1);
      y1 = crds(pt(i,1),2);
      y2 = crds(pt(i,2),2);

      m = [y1 - y2, x2 - x1, (x1 * y2) - (x2 * y1)];
      d = (l(1) * m(2)) - (l(2) * m(1));

      new_crds(i,1) = (l(2) * m(3) - l(3) * m(2)) / d;
      new_crds(i,2) = (l(3) * m(1) - l(1) * m(3)) / d;
    end;