% NestD:  This function estimates nestedness discrepancy (Brualdi and Sanderson 1999), which is a count of thh number of 
%       absencdes or presences that must be erased to produce a perfectly nested matrix.  The smaller the discrepancy, 
%       the stronger the pattern of nestedness.
%
%   Usage: [D,PD,PvD,zD,MD,SD] = NestD(X,ord,model,iter,ax)
%
%       X = [sites x species] binary data matrix
%       ord = {optional}, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals {default}
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature)
%           2 = Species richness per site is equiprobable and species occurrence is fixed 
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0)--- {default}
%           4 = Species richness per site is proportional & species occurrence is fixed
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1) 
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       iter = optional, specifies how many iterations to run (deafult = 0}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       --------------------------------------------------------------------------------------------------------------
%       D = the number of discrepancies
%       PD = standardized version of d in percent nestedness (ranges from 0 to 100)
%       PvD = significance of d based on z distribution
%       zD = standardized z-score
%       MD = mean of randomized values
%       SD = mean of randomized values
%
%   Chris L. Higgins 3/27/2008
%
%   Brualdi, R.A. and J.G. Sanderson.  1999.  Nested species subsets, gaps, and discrepancy. Oecologia, 119: 256-264.
%

function [D,PD,PvD,zD,MD,SD] = NestD(X,ord,model,iter,ax)


    if(nargin < 2)                                      % Allocate input arguments
        ord = [];
    end;
    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 3;
    end;
    if(isempty(model))
        model = 3;
    end;
    if(isempty(iter))
        iter = 0;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    
    X = makebinary(X);                                  % Covert to binary matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            Y = nestsort(X);                            % Ordinate based on species richness and occurence totals 
    end;
    
    [r,c] = size (Y);
    Richness = rowsum(Y);
        
    P = zeros(r,c);
    for i = 1:r
        P(i,[1:Richness(i)]) = 1;
    end;
    ind = find(P == 1 & Y == 0);
    D = length(ind);    
    
    for j = 1:iter
        null = randomnull(X,model);
        
        switch(ord)                                         % Allow various kinds of ordination of data matrix
            case 0
                EY = null;                                  % Do not ordinate the data matrix
            case 1
                EX = allzeros(null);                        % Remove any rows or columns that contain all zeros
                EY = ordinate(EX,ax);                       % Ordinate the data matrix by reciprical averaging
            case 2
                EY = maxpack(null);                         % Ordinate the data matrix by maximumally packing it (nested calculator)
            case 3
                EY = nestsort(null);                        % Ordinate based on species richness and occurence totals 
        end;
        
        [r,c] = size(EY);
        ERichness = rowsum(EY);
        
        EP = zeros(r,c);
        for i = 1:r
            EP(i,[1:ERichness(i)]) = 1;
        end;
        Eind = find(EP == 1 & EY == 0);
        Ed = length(Eind);
        
        d_null(j) = Ed;
    end;

    if(iter > 0)
        MD = mean(d_null);
        SD = std(d_null);
        
        zD = (D + 0.5 - MD) / SD;
        [H,PvD] = ztest(zD,0,1,.05,-1);
        
        % Calculates the Percent of discrepancies (Pd)- standardization method
        dPerf = 0;
        PD = 100 * ((D - MD) / (dPerf - MD));
        
        Na = (D - MD)/SD;
    end;
    
return;