% NestU:  This function calculates the number of unexpected absences, unexpected presence, and total unexpecetdness, 
%   their standardized versions, corresponding significance values, and z-scores.
%
%   Usage: [Ua,Up,Ut,PUa,PUp,PUt,PvUa,PvUp,PvUt,zUa,zUp,zUt] = NestU(X,ord,model,iter,ax)
%
%       X = [sites x species] binary data matrix
%       ord = {optional}, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals {default}
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature)
%           2 = Species richness per site is equiprobable and species occurrence is fixed 
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0)
%           4 = Species richness per site is proportional & species occurrence is fixed
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1) --- {default}
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       iter = optional, specifies how many iterations to run (deafult = 0}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       --------------------------------------------------------------------------------------------------------------
%       Ua = the number of unexpected absences
%       Up = the number of unexpected presences
%       Ut = total unexpectedness
%       PUa = standardized version of Ua in percent nestedness (ranges from 0 to 100)
%       PUp = standardized version of Up in percent nestedness (ranges from 0 to 100)
%       PUt = standardized version of Ut in percent nestedness (ranges from 0 to 100)
%       PvUa = significance of Ua based on z distribution
%       PvUp = significance of Up based on z distribution
%       PvUt = significance of Ut based on z distribution
%       zUa = standardized z-score
%       zUp = standardized z-score
%       zUt = standardized z-score
%
%   Chris L. Higgins 4/2/2008
%
%   Cutler, A.  1991.  Nested faunas and extinction in fragmented habitats.  Conservation Biology, 5(4): 496-505.
%

function [Ua,Up,Ut,PUa,PUp,PUt,PvUa,PvUp,PvUt,zUa,zUp,zUt,MUa,MUp,MUt,SUa,SUp,SUt] = NestU(X,ord,model,iter,ax)

    if(nargin < 2)                                      % Allocate input arguments
        ord = [];
    end;
    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 3;
    end;
    if(isempty(model))
        model = 5;
    end;
    if(isempty(iter))
        iter = 0;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    
    X = makebinary(X);                                  % Covert to binary matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            Y = nestsort(X);                            % Ordinate based on species richness and occurence totals 
    end;

    [r,c] = size(Y);
    Richness = rowsum(Y);
    
    P = zeros(r,c);
    Ua = zeros(1,c);
    Up = zeros(1,c);
    for i = 1:c
        ind1 = find(Y(:,i) == 1);
        ind0 = find(Y(:,i) == 0);
        if(~isempty(ind1) & ~isempty(ind0))
            depsite = ind1(length(ind1));
            indRich = find(Richness > Richness(depsite));
            N0 = length(find(Y(indRich,i) == 0));
                
            indRich = find(Richness < Richness(ind0(1)));
            N1 = length(find(Y(indRich,i) == 1));
        else;
            N0 = 0;
            N1 = 0;
        end;
        
        if(N0 < N1)
            Ua(i) = N0;
        end;
        if(N1 < N0)
            Up(i) = N1;
        end;
        if(N0 == N1)
            Ua(i) = N0/2;
            Up(i) = N1/2;
        end;
    end;
    
    Ua = sum(Ua);
    Up = sum(Up);
    Ut = Ua + Up;

    if(iter > 0)
        Ua_null = zeros(iter,1);
        Up_null = zeros(iter,1);
        Ut_null = zeros(iter,1);
        for j = 1:iter
            null = randomnull(X,model);
        
            switch(ord)                                         % Allow various kinds of ordination of data matrix
                case 0
                    EY = null;                                  % Do not ordinate the data matrix
                case 1
                    EX = allzeros(null);                        % Remove any rows or columns that contain all zeros
                    EY = ordinate(EX,ax);                       % Ordinate the data matrix by reciprical averaging
                case 2
                    EY = maxpack(null);                         % Ordinate the data matrix by maximumally packing it (nested calculator)
                case 3
                    EY = nestsort(null);                        % Ordinate based on species richness and occurence totals 
            end;
        
            [r,c] = size(EY);
            ERichness = rowsum(EY);
    
            EUa = zeros(1,c);
            EUp = zeros(1,c);
            for i = 1:c
                Eind1 = find(EY(:,i) == 1);
                Eind0 = find(EY(:,i) == 0);              
                if(~isempty(Eind1) & ~isempty(Eind0))
                    Edepsite = Eind1(length(Eind1));
                    EindRich = find(ERichness > ERichness(Edepsite));
                    EN0 = length(find(EY(EindRich,i) == 0));
            
                    EindRich = find(ERichness < ERichness(Eind0(1)));
                    EN1 = length(find(EY(EindRich,i) == 1));    
            
                    if(EN0 < EN1)
                        EUa(i) = EN0;
                    end;
                    if(EN1 < EN0)
                        EUp(i) = EN1;
                    end;
                    if(EN0 == EN1)
                        EUa(i) = EN0/2;
                        EUp(i) = EN1/2;
                    end;
                end;
            end;

            Ua_null(j) = sum(EUa);
            Up_null(j) = sum(EUp);
            Ut_null(j) = Ua_null(j) + Up_null(j); 
        end;

        MUa = mean(Ua_null);
        SUa = std(Ua_null);
        MUp = mean(Up_null);
        SUp = std(Up_null);
        MUt = mean(Ut_null);
        SUt = std(Ut_null);
        
        zUa = (Ua + 0.5 - MUa) / SUa;
        [H,PvUa] = ztest(zUa,0,1,.05,-1);
        zUp = (Up + 0.5 - MUp) / SUp;
        [H,PvUp] = ztest(zUp,0,1,.05,-1);
        zUt = (Ut + 0.5 - MUt) / SUt;
        [H,PvUt] = ztest(zUt,0,1,.05,-1);
        
        % Calculates the Percent of holes (PN0)- standardization method
        UaPerf = 0;
        PUa = 100 * ((Ua - MUa) / (UaPerf - MUa));
        UpPerf = 0;
        PUp = 100 * ((Up - MUp) / (UpPerf - MUp));
        UtPerf = 0;
        PUt = 100 * ((Ut - MUt) / (UtPerf - MUt));
    end;
    
return;