% NicheOverlap: This function calculates the amount of niche overlap between species within a community.  Two measures of niche overlap can be used,
%               including Pianka and Czechanowski.  Various null models can be used as well. All results are consistent with those produceed by Ecosim.
%
% usage: [MeanStat,VarStat,MeanNullMean,VarNullMean,pMean,MeanNullVar,VarNullVar,pVar,PairStat] = NicheOverlap(X,index,iter,model)
%
%   X = a [r x c] data matrix with rows corresponding to species and columns to niche categories
%   index = optional, specifies whether to use Pianka {1 - default} or Czechabowski (2)
%   iter = optional, specifies how many iterations to perform in randomization
%   model = optional, specifies which null model or algortihm to use
%       These models break natural autocorrelation -- such as temporal autocorrelation
%           1 = niche breadth relaxed and zero states reshuffled
%           2 = niche breadth relaxed and zero states retained
%           3 = niche breadth retained and zero states reshuffled {default}
%           4 = niche breadth retained and zero states retained
%       This model maintains natural autocorrelation -- like Rosario (Ivan's program)
%           5 = shifts entire distribution
% ---------------------------------------------------------------------------------------------------------------------------------------------------------
%   MeanStat = the mean value for niche overlap among all pairwise combinations of species in the community
%   VarStat = the variance of niche overlap values among all possible combinations of species in the community
%   MeanNullMean = the mean of the distribution of mean values of overlap among all combinations of species under in the null community
%   VarNullMean  = the variance of the distribution of mean values of overlap among all combinations of species under in the null community
%   pMean = signficance of  the MeanStat compared to the null distribution
%   MeanNullVar = the mean of the distribution of variance values of overlap among all combinations of species under in the null community
%   VarNullVar  = the variance of the distribution of variance values of overlap among all combinations of species under in the null community
%   pVar = signficance of the VarStat compared to the null distribution
%   PairStat = a [r x r] matrix of all possible pairwise niche overlap values
%
%   Chris Higgings 7/11/2007
%

function [MeanStat,VarStat,MeanNullMean,VarNullMean,pMean,MeanNullVar,VarNullVar,pVar,PairStat] = NicheOverlap(X,index,iter,model)

% Set default values for input arguments

    if(nargin < 2)
        index = [];
    end;
    if(nargin < 3)
        iter = [];
    end;
    if(nargin < 4)
        model = [];
    end;
    
    if(isempty(index))
        index = 1;
    end;
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(model))
        model = 3;
    end;

% This part of the function is used to estimate observed values

    [MeanStat,VarStat,PairStat] = NicheOverlapf(X,index);
 
% This part of the function is for the null model and randomization aspect
    
    for k = 1:iter
        Null = NicheRand(X,model);
        [M(k),V(k)] = NicheOverlapf(Null,index);
    end;                              
    
    MeanNullMean = mean(M);
    VarNullMean = var(M);
    pMean = pvalue(MeanStat,M,3);
    
    MeanNullVar = mean(V);
    VarNullVar = var(V);
    pVar = pvalue(VarStat,V,3);
    
 return;