% NicheOverlapf: This function calculates the amount of niche overlap between species within a community.  Two measures of niche overlap can be used,
%               including Pianka and Czechanowski.  All results are consistent with those produceed by Ecosim.
%
%   usage: [MeanStat,VarStat,PairStat] = NicheOverlapf(X,index)
%
%   X = a [r x c] data matrix with rows corresponding to species and columns to niche categories
%   index = optional, specifies whether to use Pianka {1 - default} or Czechabowski (2)
%   ---------------------------------------------------------------------------------------------------------------------------------------------------
%   MeanStat = the mean value for niche overlap among all pairwise combinations of species in the community
%   VarStat = the variance of niche overlap values among all possible combinations of species in the community
%   PairStat = a [r x r] matrix of all possible pairwise niche overlap values
%
%   Chris L. Higgins    7/11/2007
%

function [MeanStat,VarStat,PairStat] = NicheOverlapf(X,index)

    [r,c] = size(X);                                            % Determine number of species and niche categories
    
    Stat = [];                                                  % Allocate vector for all values
    PairStat = zeros(r,c);                                      % Allocate matrix for pairwise values
    
    for i = 1:r-1
        for j = i+1:r
            S1 = X(i,:) / sum(X(i,:));
            S2 = X(j,:) / sum(X(j,:));
            if(index == 1)                                      % Compute Pianka's index
                P = sum(S1.*S2);
                P2 = sqrt(sum(S1.^2) .* sum(S2.^2));
                Pianka = P/P2;
                Stat = [Stat Pianka];
                PairStat(i,j) = Pianka;
            end;
            if(index == 2)                                      % Compute Czechanowski's index
                Diff = abs(S1 - S2);
                Czechanowski = 1 - (0.5*sum(Diff));
                %%%%%%C = 1 - (0.5*(sum(Diff)) - sum(min(S1,S2)))
                Stat = [Stat Czechanowski];
                PairStat(i,j) = Czechanowski;
            end;
        end;
    end;
                        
    MeanStat = mean(Stat);                                      % Estimate average degree of overlap among species
    VarStat = var(Stat);                                        % Estimate variance in degree of overlap among species 
    
return;