% NicheRand:  This function radnomizes a matrix for use in calucalting significance of niche overlap.  Based on Winemiller and Pianka (1990),
%           there are only 4 possibilities of null models (all 4 are in Ecosim).  Niche breadth can be relaxed or retained, and zeros states
%           can be reshuffled or retained.  If niche breadth is relaxed and zero states are reshuffled, every cell in the null matrix is
%           a uniform random number between 0 and 1 (RA1 of Ecosim and Winemiller and Pianka).  If niche breadth is relaxed and zero states
%           are retained, every cell but the zeros in the original matrix is a uniform number between 0 and 1 (RA2).  If niche breadth is 
%           retained and zero states reshuffled, then each column within a row is randomly shuffled keeping its original value (RA3).  If niche
%           breadth is retained and zero states are retained, then every column except those containing a zero within a row is randomly shuffled.
%
%   usage:  Null = NicheRand(X,model)
%   
%   X = a [r x c] data matrix with rows corresponding to species and columns to niche categories
%   model = optional, species which null model or algortihm to use
%       These models break natural autocorrelation -- such as temporal autocorrelation
%           1 = niche breadth relaxed and zero states reshuffled
%           2 = niche breadth relaxed and zero states retained
%           3 = niche breadth retained and zero states reshuffled
%           4 = niche breadth retained and zero states retained
%       This model maintains natural autocorrelation -- like Rosario (Ivan's program)
%           5 = shifts entire distribution
%
%   ----------------------------------------------------------------------------------------------------------------------------------------------------------
%   Null = a [r x c] null model
%
%   Chris L. Higgins 7/11/2007
%

function Null = NicheRand(X,model)

    [r,c] = size(X);
    Null = zeros(r,c);
    
    switch(model)
        case 1
                Null = rand(r,c);       
        case 2
            for i = 1:r
                Ind = find(X == 0);
                Null = rand(r,c);
                Null(Ind) = 0;
            end;           
        case 3
            for i = 1:r
                Ind = randperm(c);
                Null(i,:) = X(i,Ind);
            end;
        case 4
            for i = 1:r
                Ind1 = find(X(i,:) ~= 0);
                Ind2 = randperm(length(Ind1));            
                Null(i,Ind1) = X(i,Ind1(Ind2));
            end;
        case 5
            for i = 1:r
                Ind = ceil(rand*c);
                Null(i,Ind:c) = X(i,1:c-(Ind-1));
                Null(i,1:Ind-1) = X(i,length(Ind:c)+1:c);
            end;
    end;
    
        
    
return;