% Partition: This function partitions species diversity within and among samples (alpha and beta diversity) from hierarchically
%   scaled studies to assess the proportion of total diversity (gamma) found in different habitats, landscapes, or regions.  This function
%   uses additive partitioning to decompose species richness, Shannon diversity, Camargo evenness, Berger-Parker dominance, and rarity values.
%
%   Usage: [alpha,CIalpha,beta,CIbeta] = Partition(data,levels,weights,HO,iter,loc)
%
%   data = [sites x species] matrix of observed species abundances
%   levels = [sites x levels] matrix of hierchrical sample design
%       ---This function will automatically add a column for the smallest spatial scale (each site is a sample) and a column for the largest
%       spatial scale (all sites are a sample).  If no arguement is passed, calcualtions proceed assuming only two hierarchical levels.
%   weights = optional, specifies whether to use unweighted (0 - default} or weighted averages (1)
%   H0 = optional, specifies which null model to use
%       1 = individual based model {default} ---- this null model is the only one that will work if no arguement is passed for 'levels'
%       2 = sample based model ---- this null model is meaningless if only two spatial scales are used (i.e., no arguement is passed for 'levels')
%   iter = optional, specifies number of iterations to use in randomization {default = 0}
%   loc = optional, if any value is passed, global rarity is used instead of local rarity {default = []}
%   ------------------------------------------------------------------------------------------------------------------------------
%   alpha = [5 x levels+2] matrix of alpha diversity
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Camargo evenness
%       row 4 = Berger-parker dominance
%       row 5 = rarity
%       column 1 = alpha diversity for each site separaetly (smallest possible scale)
%       column levels+2 = alpha diversity for all sites combined (largest possible scale -- equals gamma diversity)
%   CIalpha = [10 x levels+2] associated significance values (last column in meaningless for current null models)
%       row 1 = lower confidence limit for species richness
%       row 2 = upper confidence limit for species richness
%       row 3 = lower confidence limit for Shannon diversity
%       row 4 = upper confidence limit for Shannon diversity
%       row 5 = lower confidence limit for Camargo evenness
%       row 6 = upper confidence limit for Camargo evenness
%       row 7 = lower confidence limit for Berger-parker dominanc
%       row 8 = upper confidence limit for Berger-parker dominanc
%       row 9 = lower confidence limit for rarity
%       row 10 = upper confidence limit for rarity
%   beta = [5 x levels+1] matrix of beta diversity
%   CIbeta = [10 x levels+1] associated signifaince values for beta diversity
%       row 1 = lower confidence limit for species richness
%       row 2 = upper confidence limit for species richness
%       row 3 = lower confidence limit for Shannon diversity
%       row 4 = upper confidence limit for Shannon diversity
%       row 5 = lower confidence limit for Camargo evenness
%       row 6 = upper confidence limit for Camargo evenness
%       row 7 = lower confidence limit for Berger-parker dominanc
%       row 8 = upper confidence limit for Berger-parker dominanc
%       row 9 = lower confidence limit for rarity
%       row 10 = upper confidence limit for rarity
%
%   Chris Higgins 10/25/2007
%
%   Crist, T.O., J.A. Veech, J.C. Gering, and K.S. Summerville.  2003.
%       Partitioning species diversity across landscapes and regions: a
%       hierarchical analysis of alpha, beta, and gamma diversity.  The
%       American Naturalist 162(6): 734-743.
%

function [alpha,CIalpha,beta,CIbeta] = Partition(data,levels,weights,H0,iter,loc)

    if(nargin < 2)                                                              % Allocate input arguements
        levels = [];
    end;
    if(nargin < 3)                                                          
        weights = [];
    end;
    if(nargin < 4)
        H0 = [];
    end;
    if(nargin < 5)
        iter = [];
    end;
    if(nargin < 6)
        loc = [];
    end;
    
    if(isempty(weights))                                                        % Set default values for input arguements
        weights = 0;
    end;
    if(isempty(H0))
        H0 = 1;
    end;
    if(isempty(iter))
        iter = 0;
    end;
    if(isempty(loc))
        TotSAD = [];
    else;
        TotSAD = sum(data);
    end;
    
    RowTot = rowsum(data);                                                      % Remove any sites with 0 individuals
    IR = find(RowTot == 0);                             
    data = remove(data,IR,1);
    if(~isempty(levels))
        levels = remove(levels,IR,1);
    end;
    
    ColTot = sum(data);                                                         % Remove any species with 0 individuals
    IC = find(ColTot == 0);
    data = remove(data,IC,2);
    
    [r,c] = size(data);                                                         % Determine the number of sites and species in data matrix
    GrpAll = ones(r,1);                                                         % Create grouping vector for all sites
    GrpEach = [1:r]';                                                           % Create grouping vector for each site
    Hierarchy = [GrpEach levels GrpAll];                                        % Combine levels of analysis into one matrix
    [rH,cH] = size(Hierarchy);                                                  % Determine size of levels matrix
    
    alpha = zeros(5,cH);                                                        % Allocate output arguements
    beta = zeros(5,cH-1);
    
% Estiamte alpha diversity
    for i = 1:cH
        [u,f] = unique1(Hierarchy(:,i));                                        % Determine number of samples within each level
        S = zeros(1,length(u));                                                 % Allocate vectors for each diversity component separately
        H = zeros(1,length(u));
        E = zeros(1,length(u));
        D = zeros(1,length(u));
        R = zeros(1,length(u));
        N = zeros(length(u),1);
        for j = 1:length(u)
            ind = find(Hierarchy(:,i) == u(j));                                 % Find the corresponding sites within each sample
            if(length(ind) == 1)                                                % Sum the individuals of each species for each sample
                X = data(ind,:);
            else
                X = sum(data(ind,:));
            end;
            [S(j),H(j),E(j),D(j),R(j)] = diversity(X,TotSAD);                   % Estimate various diversity indices for each sample
            N(j) = sum(X);                                                      % Calculate total number of individuals in each sample
        end;
        if(weights ~= 0)                     
            q = N / sum(N);                                                     % Partitioning conducted with sample-size weighting
            alpha(1,i) = sum(S*q);                                              % Estimate alpha diversity for species richness
            alpha(2,i) = sum(H*q);                                              % Estimate alpha diversity for Shannon diversity
            alpha(3,i) = sum(E*q);                                              % Estiamte alpha diversity for Camargo evenness
            alpha(4,i) = sum(D*q);                                              % Estimate alpha diversity for Berger-Parker dominance
            alpha(5,i) = sum(R*q);                                              % Estimate alpha diversity for rarity
        else;     
            q = ones(length(u),1) / length(u);                                  % Partitioning conducted without sample-size weighting                  
            alpha(1,i) = sum(S*q);                                              % Estimate alpha diversity for species richness
            alpha(2,i) = sum(H*q);                                              % Estimate alpha diversity for Shannon diversity
            alpha(3,i) = sum(E*q);                                              % Estiamte alpha diversity for Camargo evenness
            alpha(4,i) = sum(D*q);                                              % Estimate alpha diversity for Berger-Parker dominance
            alpha(5,i) = sum(R*q);                                              % Estimate alpha diversity for rarity
        end;
    end;
    
% Estimate beta diversity    
    beta = alpha(:,2:end)-alpha(:,1:end-1);

%%%%%%%%%%%%%%   EVERYTHING BENEATH THIS POINT REFERS TO THE RANDOMIZATION PART OF THE FUNCTION!!!!

    CIalpha=[];                                                                 % Allocate matrices for Pvalues
    CIbeta=[];
    
    if(iter ~= 0)
        alphanull = zeros(5,cH,iter);
        betanull = zeros(5,cH-1,iter);
        gammanull = zeros(5,1,iter);        
        
        for k = 1:iter
            
% Randomization based on individuals (Hypothesis 1 of Crist et al.)
            if(H0 == 1)                                         
                null = randind(data);                                           % Randomly allocate individuals, without replacement, to each site

            % Estiamte alpha diversity
                for i = 1:cH
                    [u,f] = unique1(Hierarchy(:,i));                            % Determine number of samples within each level
                    S = zeros(1,length(u));                                     % Allocate vectors for each diversity component separately
                    H = zeros(1,length(u));
                    E = zeros(1,length(u));
                    D = zeros(1,length(u));
                    R = zeros(1,length(u));
                    N = zeros(length(u),1);
                    for j = 1:length(u)
                        ind = find(Hierarchy(:,i) == u(j));                     % The the corresponding sites within each sample
                        if(length(ind) == 1)                                    % Sum the individuals of each species for each sample
                            X = null(ind,:);
                        else
                            X = sum(null(ind,:));
                        end;
                        [S(j),H(j),E(j),D(j),R(j)] = diversity(X,TotSAD);       % Estimate various diversity indices for each sample
                        N(j) = sum(X);                                          % Calculate total number of individuals in each sample
                    end;
                    if(weights ~= 0)                     
                        q = N / sum(N);                                         % Partitioning conducted with sample-size weighting
                        alphanull(1,i,k) = sum(S*q);                            % Estimate alpha diversity for species richness
                        alphanull(2,i,k) = sum(H*q);                            % Estimate alpha diversity for Shannon diversity
                        alphanull(3,i,k) = sum(E*q);                            % Estiamte alpha diversity for Camargo evenness
                        alphanull(4,i,k) = sum(D*q);                            % Estimate alpha diversity for Berger-Parker dominance
                        alphanull(5,i,k) = sum(R*q);                            % Estimate alpha diversity for rarity
                    else;     
                        q = ones(length(u),1) / length(u);                      % Partitioning conducted without sample-size weighting                  
                        alphanull(1,i,k) = sum(S*q);                            % Estimate alpha diversity for species richness
                        alphanull(2,i,k) = sum(H*q);                            % Estimate alpha diversity for Shannon diversity
                        alphanull(3,i,k) = sum(E*q);                            % Estiamte alpha diversity for Camargo evenness
                        alphanull(4,i,k) = sum(D*q);                            % Estimate alpha diversity for Berger-Parker dominance
                        alphanull(5,i,k) = sum(R*q);                            % Estimate alpha diversity for rarity
                    end;
                end;
       
            % Estimate beta diversity    
                betanull(:,:,k) = alphanull(:,2:end,k)-alphanull(:,1:end-1,k);   
            end;  
            
% Randomization based on samples (Hypothesis 2 of Crist et al.)            
            if(H0 == 2)                                 
                null = randsamp(data,levels);

            % Estiamte alpha diversity
                for i = 1:cH
                    [u,f] = unique1(Hierarchy(:,i));                            % Determine number of samples within each level
                    S = zeros(1,length(u));                                     % Allocate vectors for each diversity component separately
                    H = zeros(1,length(u));
                    E = zeros(1,length(u));
                    D = zeros(1,length(u));
                    R = zeros(1,length(u));
                    N = zeros(length(u),1);
                    for j = 1:length(u)
                        ind = find(Hierarchy(:,i) == u(j));                     % The the corresponding sites within each sample
                        if(length(ind) == 1)                                    % Sum the individuals of each species for each sample
                            X = null(ind,:,i);
                        else
                            X = sum(null(ind,:,i));
                        end;
                        [S(j),H(j),E(j),D(j),R(j)] = diversity(X,TotSAD);       % Estimate various diversity indices for each sample
                        N(j) = sum(X);                                          % Calculate total number of individuals in each sample
                    end;
                    if(weights ~= 0)                     
                        q = N / sum(N);                                         % Partitioning conducted with sample-size weighting
                        alphanull(1,i,k) = sum(S*q);                            % Estimate alpha diversity for species richness
                        alphanull(2,i,k) = sum(H*q);                            % Estimate alpha diversity for Shannon diversity
                        alphanull(3,i,k) = sum(E*q);                            % Estiamte alpha diversity for Camargo evenness
                        alphanull(4,i,k) = sum(D*q);                            % Estimate alpha diversity for Berger-Parker dominance
                        alphanull(5,i,k) = sum(R*q);                            % Estimate alpha diversity for rarity
                    else;     
                        q = ones(length(u),1) / length(u);                      % Partitioning conducted without sample-size weighting                  
                        alphanull(1,i,k) = sum(S*q);                            % Estimate alpha diversity for species richness
                        alphanull(2,i,k) = sum(H*q);                            % Estimate alpha diversity for Shannon diversity
                        alphanull(3,i,k) = sum(E*q);                            % Estiamte alpha diversity for Camargo evenness
                        alphanull(4,i,k) = sum(D*q);                            % Estimate alpha diversity for Berger-Parker dominance
                        alphanull(5,i,k) = sum(R*q);                            % Estimate alpha diversity for rarity
                    end;
                end;

            % Estimate beta diversity    
                betanull(:,:,k) = alphanull(:,2:end,k)-alphanull(:,1:end-1,k); 
            end;
         end;
         
%%%%%%%%%%%%%% This section uses the null distirbutions to determine 95% confidence intervals

% Create confidence intervals for estimates of alpha diversity at each hierarchial level
        for i = 1:cH                                               
            AS(:,1) = alphanull(1,i,:);
            AH(:,1) = alphanull(2,i,:);
            AE(:,1) = alphanull(3,i,:);
            AD(:,1) = alphanull(4,i,:);
            AR(:,1) = alphanull(5,i,:);
            CIAS(:,i) = bootci(AS);
            CIAH(:,i) = bootci(AH);
            CIAE(:,i) = bootci(AE);
            CIAD(:,i) = bootci(AD);
            CIAR(:,i) = bootci(AR);
        end;
        CIalpha = [CIAS;CIAH;CIAE;CIAD;CIAR];
        
% Create confidence intervals for estimates of beta diversity at each hierarchial level     
        for i = 1:cH-1
            BS(:,1) = betanull(1,i,:);
            BH(:,1) = betanull(2,i,:);
            BE(:,1) = betanull(3,i,:);
            BD(:,1) = betanull(4,i,:);
            BR(:,1) = betanull(5,i,:);
            CIBS(:,i) = bootci(BS);
            CIBH(:,i) = bootci(BH);
            CIBE(:,i) = bootci(BE);
            CIBD(:,i) = bootci(BD);
            CIBR(:,i) = bootci(BR);
        end;
        CIbeta = [CIBS;CIBH;CIBE;CIBD;CIBR];

    end;

return;