% PartitionEntropy: This function partitions total biodiversity (species richness, Shannon diversity, Simpson diversity, and Berger-Parker
%   dominance -- Entropy values from Jost 2006 and 2007)into within and among componenets (alpha and beta diversity) based on
%   multiplicative partitioning.
%
%   Usage: [alpha,CIalpha,beta,CIbeta] = PartitionEntropy(data,levels,weights,HO,iter)
%
%   data = [sites x species] matrix of observed species abundances
%   levels = [sites x levels] matrix of hierchrical sample design
%       --- This function will automatically add a column for the smallest spatial scale (each site is a sample) and a column for the largest
%       spatial scale (all sites are a sample).  If no arguement is passed, calcualtions proceed assuming only two hierarchical levels!
%   weights = optional, specifies whether to use unweighted (0 - default} or weighted averages (1)
%   H0 = optional, specifies which null model to use
%       1 = randomly samples individuals, with replacement, from each site constraining the total number of individuals 
%       2 = randomly sample individuals, with replacement, from a pooled abundance distrbition for each intermedaite level
%           ---- this null model only works with 3 spatial scales (otherwise use the 1 and 3 null models)
%       3 = randomly samples individuals, with replacement, from a pooled abundance distribution across the entire sampling region
%   iter = optional, specifies number of iterations to use in randomization {default = 0}
%   ------------------------------------------------------------------------------------------------------------------------------------------------
%   alpha = [2 x levels+2] matrix of alpha diversity
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Camargo diversity
%       row 4 = Simpson diversity
%       row 5 = Berger-Parker dominance
%       column 1 = alpha diversity for each site separaetly (smallest possible scale)
%       column levels+2 = alpha diversity for all sites combined (largest possible scale -- equals gamma diversity)
%   CIalpha = [4 x levels+2] associated confidence intervals for alpha and gamma diversity
%       row 1 = lower confidence limit for species richness
%       row 2 = upper confidence limit for species richness
%       row 3 = lower confidence limit for Shannon diversity
%       row 4 = upper confidence limit for Shannon diversity
%       row 5 = lower confidence limit for Camargo diversity
%       row 6 = upper confidence limit for Camargo diversity
%       row 7 = lower confidence limit for Simpson diversity
%       row 8 = upper confidence limit for Simpson diversity
%       row 9 = lower confidence limit for Berger-Parker dominance
%       row 10 = upper confidence limit for Berger-parker dominance
%   beta = [2 x levels+1] matrix of beta diversity
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Camargho diversity
%       row 4 = Simpson diversity
%       row 5 = Berger-Parker dominance
%       column 1 = beta diversity among sites (smallest possible scale)
%       column levels+1 = beta diversity among regions
%   CIbeta = [4 x levels+1] associated confidence intervals for beta diversity
%       row 1 = lower confidence limit for species richness
%       row 2 = upper confidence limit for species richness
%       row 3 = lower confidence limit for Shannon diversity
%       row 4 = upper confidence limit for Shannon diversity
%       row 5 = lower confidence limit for Camargo diversity
%       row 6 = upper confidence limit for Camargo diversity
%       row 7 = lower confidence limit for Simpson diversity
%       row 8 = upper confidence limit for Simpson diversity
%       row 9 = lower confidence limit for Berger-Parker dominance
%       row 10 = upper confidence limit for Berger-parker dominance
%
%   Chris Higgins 4/1/2009
%
%   Jost, L. 2006.  Entropy and diversity.  Oikos 113: 363-375.
%   Jost, L. 2007.  Partitioning diversity into independent alpha and beta components.  Ecology 88: 2427-2439.
%

function [alpha,CIalpha,beta,CIbeta] = PartitionEntropy(data,levels,weights,H0,iter)

    if(nargin < 2)                                                          % Allocate input arguements
        levels = [];
    end;
    if(nargin < 3)                                                          
        weights = [];
    end;
    if(nargin < 4)
        H0 = [];
    end;
    if(nargin < 5)
        iter = [];
    end;
    
    if(isempty(weights))                                                    % Set default values for input arguements
        weights = 0;
    end;
    if(isempty(H0))
        H0 = 1;
    end;
    if(isempty(iter))
        iter = 0;
    end;
    
    if(isempty(levels) & H0 == 2)
        error('This null model does not work when only two spatial scales are used -- use option 1 or 3.');
    end;
    
    RowTot = rowsum(data);                                                  % Remove any sites with 0 individuals
    IR = find(RowTot == 0);                             
    data = remove(data,IR,1);
    if(~isempty(levels))
        levels = remove(levels,IR,1);
    end;
    
    ColTot = sum(data);                                                     % Remove any species with 0 individuals
    IC = find(ColTot == 0);
    data = remove(data,IC,2);
    
    [r,c] = size(data);
    GrpAll = ones(r,1);                                                     % Create grouping vector for all sites
    GrpEach = [1:r]';
    Hierarchy = [GrpEach levels GrpAll];                                    % Combine levels and GrpA
    [rH,cH] = size(Hierarchy);                                              % Determine size of levels matrix   
    
    alpha = zeros(5,cH);                                                    % Allocate output arguements
    beta = zeros(5,cH-1);
    
% Estiamte alpha diversity
    for i = 1:cH
        [u,f] = unique1(Hierarchy(:,i));                                    % Determine number of samples within each level
        S = zeros(1,length(u));                                             % Allocate vectors for each diversity component separately
        H = zeros(1,length(u));
        E = zeros(1,length(u));
        D = zeros(1,length(u));
        B = zeros(1,length(u));
        N = zeros(length(u),1);
        for j = 1:length(u)
            ind = find(Hierarchy(:,i) == u(j));                             % The the corresponding sites within each sample
            if(length(ind) == 1)                                            % Sum the individuals of each species for each sample
                if(weights ~= 0)
                    X = data(ind,:);                                        % Summing absolute abundances is equivalent to weighting by sample size
                else
                    X = MakeRelative(data(ind,:));                          % Summing relative abundances does not weight by sample size
                end;
            else
                if(weights ~= 0)
                    X = sum(data(ind,:));                                   % Summing absolute abundances is equivalent to weighting by sample size
                else;
                    RelData = MakeRelative(data(ind,:));           
                    X = mean(RelData);                                      % Summing relative abundances does not weight by sample size
                end;
            end;
            [S(j),H(j),E(j),D(j),B(j)] = Entropy(X,1);                      % Estimate various diversity indices for each sample
            N(j) = sum(X);                                                  % Calculate total number of individuals in each sample
        end;
        if(weights ~= 0)                     
            q = N / sum(N);                                                 % Partitioning conducted with sample-size weighting
            alpha(1,i) = sum(S*q);                                          % Estimate alpha diversity for species richness
            alpha(2,i) = exp(sum(H*q));                                     % Estimate alpha diversity for Shannon diversity
            alpha(3,i) = sum(E*q)*alpha(1,i);
            alpha(4,i) = 1/sum(D*q);                                        % Estimate alpha diversity for Simpson diversity
            alpha(5,i) = sum(B*q);                                          % Estimate alpha diversity for Berger-Parker domninance
        else;     
            q = ones(length(u),1) / length(u);                              % Partitioning conducted without sample-size weighting                  
            alpha(1,i) = sum(S*q);                                          % Estimate alpha diversity for species richness
            alpha(2,i) = exp(sum(H*q));                                     % Estimate alpha diversity for Shannon diversity
            alpha(3,i) = sum(E*q)*alpha(1,i);
            alpha(4,i) = 1/sum(D*q);                                        % Estimate alpha diversity for Simpson diversity
            alpha(5,i) = sum(B*q);                                          % Estimate alpha diversity for Berger-Parker domninance
        end;

    end;
    
% Estimate beta diversity    
    beta = alpha(:,2:end)./alpha(:,1:end-1);


    
%%%%%%%%%%%%%%   EVERYTHING BENEATH THIS POINT REFERS TO THE RANDOMIZATION PART OF THE FUNCTION!!!!

    CIalpha=[];                                                              % Allocate matrices for confidence intervals
    CIbeta=[];

    [rN,cN] = size(data);
    
    if(iter ~= 0)
        alphanull = zeros(5,cH,iter);
        betanull = zeros(5,cH-1,iter);
        SampSize = rowsum(data);                                            % Calculate sample size for each sit
                         
        for k = 1:iter
            null = zeros(rN,cN);                                            % Allocate space for null matrix

% Randomly samples individuals, with replacement, from each site constraining the total number of individuals to equal observed values
            if(H0 == 1)             
                for i = 1:rN
                    Z = makegrps([1:cN],data(i,:))';                        % Create a vector of observed counts
                    Xsamp = samplewr(Z');                                   % Sample with replacement from vector of abundances
                    [u,f] = uniquef(Xsamp);                                 % Find unique species in bootstrapped vector
                    null(i,u) = f;                                          % Place species abundances in null matrix
                end;
            end;
            
% Randomly sample individuals, with replacement, from a pooled abundance distrbition for each intermedaite level
            if(H0 == 2)
                [u,f] = unique1(levels);                                    % Determine number of regions in intermediate level
                for i = 1:length(u)                                         % Cycle through each region at intermediate level
                    ind = find(levels == u(i));                             % Pull out sites that are found in each region
                    SppAbu = sum(data(ind,:));                              % Calculate regional species abundance distribution                   
                    N = sum(SppAbu);                                        % Calculate total number of individuals in regional SAD
                    G = makegrps(1:cN,SppAbu);                              % Create vector of individuals for entire region
                    for j = 1:length(ind)                                   % Cycle through each site within a region
                        Xsamp = samplewr(G,SampSize(ind(j)));               % Sample with repacement from vector
                        [uS,fS] = unique1(Xsamp);                           % Find unique species and abundances in bootstrapped vector
                        null(ind(j),uS) = fS';                              % Place species abundances in null matrix
                    end;
                end;
            end;
            
% Randomly samples individuals, with replacement, from a pooled abundance distribution across the entire sampling region
            if(H0 == 3)
                Z = makegrps([1:cN],sum(data));                             % Create total abundance distribution for largest spatial scale
                for i = 1:rN                                                % Cycle through each site
                    Xsamp = samplewr(Z,SampSize(i));                        % Randomly sample individuals from total distrbution 
                    [u,f] = Uniquef(Xsamp);                                 % Determine which species are present and in what abundance
                    null(i,u) = f;                                          % Put species abundances into null matrix
                end;
            end;
                
% Estiamte alpha diversity
            for i = 1:cH
                [u,f] = unique1(Hierarchy(:,i));                            % Determine number of samples within each level
                S = zeros(1,length(u));                                     % Allocate vectors for each diversity component separately
                H = zeros(1,length(u));
                E = zeros(1,length(u));
                D = zeros(1,length(u));
                B = zeros(1,length(u));
                N = zeros(length(u),1);
                for j = 1:length(u)
                    ind = find(Hierarchy(:,i) == u(j));                     % The the corresponding sites within each sample
                    if(length(ind) == 1)                                    % Sum the individuals of each species for each sample
                        if(weights ~= 0)
                            X = null(ind,:);                                % Summing absolute abundances is equivalent to weighting by sample size
                        else
                            X = MakeRelative(null(ind,:));                  % Summing relative abundances does not weight by sample size
                        end;
                    else
                        if(weights ~= 0)
                            X = sum(null(ind,:));                           % Summing absolute abundances is equivalent to weighting by sample size
                        else;
                            RelData = MakeRelative(null(ind,:));           
                            X = mean(RelData);                              % Summing relative abundances does not weight by sample size
                        end;
                    end;
                    [S(j),H(j),E(j),D(j),B(j)] = Entropy(X,1);              % Estimate various diversity indices for each sample
                    N(j) = sum(X);                                          % Calculate total number of individuals in each sample
                end;
                if(weights ~= 0)                     
                    q = N / sum(N);                                         % Partitioning conducted with sample-size weighting
                    alphanull(1,i,k) = sum(S*q);                            % Estimate alpha diversity for species richness
                    alphanull(2,i,k) = exp(sum(H*q));                       % Estimate alpha diversity for Shannon diversity
                    alphanull(3,i,k) = sum(E*q)*alphanull(1,i,k);                            % Estimate alpha diversity for Camargo
                    alphanull(4,i,k) = 1/(sum(D*q));                        % Estimate alpha diversity for Simpson diversity
                    alphanull(5,i,k) = sum(B*q);                            % Estimate alpha diversity for Berger-Parker dominance
                else;     
                    q = ones(length(u),1) / length(u);                      % Partitioning conducted without sample-size weighting                  
                    alphanull(1,i,k) = sum(S*q);                            % Estimate alpha diversity for species richness
                    alphanull(2,i,k) = exp(sum(H*q));                       % Estimate alpha diversity for Shannon diversity
                    alphanull(3,i,k) = sum(E*q)*alphanull(1,i,k);
                    alphanull(4,i,k) = 1/(sum(D*q));                        % Estimate alpha diversity for Simpson diversity
                    alphanull(5,i,k) = sum(B*q);                            % Estimate alpha diversity for Berger-Parker dominance
                end;
            end;

% Estimate beta diversity 
            betanull(:,:,k) = alphanull(:,2:end,k)./alphanull(:,1:end-1,k);     
            
        end;  
         
% This section uses the null distirbutions to determine confidence interval around estimates of alpha and beta diversities
        for i = 1:cH                                               
            AS(:,1) = alphanull(1,i,:);
            AH(:,1) = alphanull(2,i,:);
            AE(:,1) = alphanull(3,i,:);
            AD(:,1) = alphanull(4,i,:);
            AB(:,1) = alphanull(5,i,:);
            CIAS(:,i) = Bootci1(AS,[],[],0.95);
            CIAH(:,i) = Bootci1(AH,[],[],0.95);
            CIAE(:,i) = Bootci1(AE,[],[],0.95);
            CIAD(:,i) = Bootci1(AD,[],[],0.95);
            CIAB(:,i) = Bootci1(AB,[],[],0.95);
        end;
        CIalpha = [CIAS;CIAH;CIAE;CIAD;CIAB];

        for i = 1:cH-1
            BS(:,1) = betanull(1,i,:);
            BH(:,1) = betanull(2,i,:);
            BE(:,1) = betanull(3,i,:);
            BD(:,1) = betanull(4,i,:);
            BB(:,1) = betanull(5,i,:);
            CIBS(:,i) = Bootci1(BS,[],[],0.95);
            CIBH(:,i) = Bootci1(BH,[],[],0.95);
            CIBE(:,i) = Bootci1(BE,[],[],0.95);
            CIBD(:,i) = Bootci1(BD,[],[],0.95);
            CIBB(:,i) = Bootci1(BB,[],[],0.95);
        end;
        CIbeta = [CIBS;CIBH;CIBE;CIBD;CIBB];

    end;

return;