% PartitionEntropyDiff:  This function compares the additive partitioning of two systems to determine if they are statistically
%   different from one another.  It does so by combining individuals from both systems and randomly allocating them back to
%   the original systems.
%
%   USAGE: [alphaX,alphaY,Palpha,betaX,betaY,Pbeta] = PartitionEntropyDiff(X,Y,levels,weights,model,iter)
%
%   X = [sites x species] matrix of species abundances for first metacommunity
%   Y = [sites x species] matrix of species abundances for second metacommunity
%   levels = [sites x levels] matrix of hierchrical sample design
%       ---This function will automatically add a column for the smallest spatial scale (each site is a sample)and a column for the
%       largest spatial scale (all sites are a sample).  If no arguement is passed, calcualtions proceed assuming only two hierarchical levels!
%   weights = optional, specifies whether to use unweighted (0 - default} or weighted averages (1)
%   model = optional, specifies which null model to use:
%       1 = data for the two metacommunities are pooled, then randomly allocates individuals from the pooled data set to communities
%           constraining the original sample sizes
%       2 = data for each metacommunity are maintained separately, but individuals are randomly placed within communities constraining
%           the original sample sizes {default}
%   iter = optional, specifis how many iterations to perform {default = 1000}
%   ---------------------------------------------------------------------------------------------------------------------------------------
%   alphaX = [4 x levels+2] matrix of alpha diversity for the first data matrix
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Simpson diversity
%       row 4 = Berger-Parker dominance
%       column 1 = alpha diversity for each site separaetly (smallest possible scale)
%       column levels+2 = alpha diversity for all sites combined (largest possible scale -- equals gamma diversity)
%   alphaY = [4 x levels+2] matrix of alpha diversity for the second data matrix
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Simpson diversity
%       row 4 = Berger-Parker dominance
%       column 1 = alpha diversity for each site separaetly (smallest possible scale)
%       column levels+2 = alpha diversity for all sites combined (largest possible scale -- equals gamma diversity)
%   Palpha = [4 x levels+2] matrix of signifiance values for difference between two data matrices for alpha diversities
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Simpson diversity
%       row 4 = Berger-Parker dominance
%   betaX = [4 x levels+1] matrix of beta diversity for the first data matrix
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Simpson diversity
%       row 4 = Berger-Parker dominance
%       column 1 = beta diversity among sites (smallest possible scale)
%       column levels+2 = beta diversity among regions)
%   betaY = [4 x levels+1] matrix of alpha diversity for the second data matrix
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Simpson diversity
%       row 4 = Berger-Parker dominance
%       column 1 = beta diversity among sites (smallest possible scale)
%       column levels+2 = beta diversity among regions
%   Pbeta = [4 x levels+1] matrix of signifiance values for difference between two data matrices for beta diversities
%       row 1 = species richness
%       row 2 = Shannon diversity
%       row 3 = Simpson diversity
%       row 4 = Berger-Parker dominance
%
%   Chris Higgins 5/26/2009
%

function [alphaX,alphaY,Palpha,betaX,betaY,Pbeta] = PartitionEntropyDiff(X,Y,levels,weights,model,iter)

    if(nargin < 3)                                                                                          % Allocate input arguments
        levels = [];
    end;
    if(nargin < 4)                                                                                          
        weights = [];
    end;
    if(nargin < 5)
        model = [];
    end;
    if(nargin < 6)
        iter = [];
    end;
    
    if(isempty(weights))
        weights = 0;
    end;
    if(isempty(model))
        model = 2;
    end;
    if(isempty(iter))
        iter = 1000;
    end;

    [rX,cX] = size(X);                                                                                      % Determine size of X data matrix
    [rY,cY] = size(Y);                                                                                      % Determine size of Y data matrix
    [rL,cL] = size(levels);                                                                                 % Determine number of hierarchical levels in experimental design
    
    if(rX >= rY)                                                                                            % Determine which data matrix has more sites
        r = rX;
    else;
        r = rY;
    end;
    
    if(cX >= cY)                                                                                            % Determine which data matrix has more species
        c = cX;
    else;
        c = cY;
    end;                                      
    
    SampSizeX = rowsum(X);                                                                                  % Determine number of individuals at each site for X data set
    SampSizeY = rowsum(Y);                                                                                  % Determine number of individuals at each site for Y data set
    
    TotX = sum(X);                                                                                          % Create total species abundance distribution for X data set
    TotY = sum(Y);                                                                                          % create total species abundance distribution for Y data set
    
    IndX = makegrps([1:cX],TotX);                                                                           % Create vector of individuals for X data matrix
    IndY = makegrps([1:cY],TotY);                                                                           % Create vector of individuals for Y data matrix
    IndZ = [IndX;IndY];                                                                                     % Combine individuals from both data sets into one vector
    
    NTot = length(IndZ);                                                                                    % Calculate total number of individuals from both data sets
    
    [alphaX,CIalphaX,betaX,CIbetaX] = PartitionEntropy(X,levels,weights,[],0);                              % Partition diversity for radnomized X data set
    [alphaY,CIalphaY,betaY,CIbetaY] = PartitionEntropy(Y,levels,weights,[],0);                              % Partition diversity for randomized Y data set
    ObsAlphaDiff = alphaX - alphaY;                                                                         % calculate observed difference in alpha diversities
    ObsBetaDiff = betaX - betaY;                                                                            % Calculate observed difference in beta diversities
    
    NullX = zeros(rX,c);                                                                                    % Allocate matrix for randomized X data matrix
    NullY = zeros(rY,c);                                                                                    % Allocate matrix for randomized y data matrix
    ExpAlphaDiff = zeros(4,(cL+2),iter);                                                                    % Allocate matrix for differences in alpha values between X and y data sets
    ExpBetaDiff = zeros(4,(cL+1),iter);                                                                     % Allocate matrix for differences in beta values between X and y data sets
    ExpGammaDiff = zeros(4,1,iter);                                                                         % Allocate matrix for differences in gamma values between X and y data sets
       
    for i = 1:iter
        
        if(model == 1)
            RandIndZ = IndZ(randperm(NTot));                                                                % Randomly permute individuals in pooled data set (Z)
            
            cntX = 0;
            for j = 1:rX                                                                                    % Cycle through each site
                RandX = RandIndZ((cntX+1):(cntX+SampSizeX(j)));                                             % Pull out samples of individuals that match original sample sizes
                if(~isempty(RandX))
                    [uX,fX] = unique1(RandX);                                                               % Determine which species are present in the sample and how many
                    NullX(j,uX) = fX';                                                                      % Place individuals in null matrix
                    cntX = cntX + SampSizeX(j);                                                             % Increase index to keep track of already placed individuals
                end;
            end;
        
            cntY = cntX;
            for k = 1:rY                                                                                    % Cycle through each site
                RandY = RandIndZ((cntY+1):(cntY+SampSizeY(k)));                                             % Pull out samples of individuals that match original sample sizes
                if(~isempty(RandY))
                    [uY,fY] = unique1(RandY);                                                               % Determine which species are present in the sample and how many
                    NullY(k,uY) = fY';                                                                      % Place individuals in null matrix
                    cntY = cntY + SampSizeY(k);                                                             % Increase index to keep track of already placed individuals
                end;
            end;
        end;
        
        if(model == 2)
            RandIndX = IndX(randperm(length(IndX)));                                                        % Randomly permute individuals in data set X
            RandIndY = IndY(randperm(length(IndY)));                                                        % Randomly permute individuals in data set Y
            
            cntX = 0;
            for j = 1:rX                                                                                    % Cycle through each site
                RandX = RandIndX((cntX+1):(cntX+SampSizeX(j)));                                             % Pull out samples of individuals that match original sample sizes
                if(~isempty(RandX))
                    [uX,fX] = unique1(RandX);                                                               % Determine which species are present in the sample and how many
                    NullX(j,uX) = fX';                                                                      % Place individuals in null matrix
                    cntX = cntX + SampSizeX(j);                                                             % Increase index to keep track of already placed individuals
                end;
            end;
        
            cntY = 0;
            for k = 1:rY                                                                                    % Cycle through each site
                RandY = RandIndY((cntY+1):(cntY+SampSizeY(k)));                                             % Pull out samples of individuals that match original sample sizes
                if(~isempty(RandY))
                    [uY,fY] = unique1(RandY);                                                               % Determine which species are present in the sample and how many
                    NullY(k,uY) = fY';                                                                      % Place individuals in null matrix
                    cntY = cntY + SampSizeY(k);                                                             % Increase index to keep track of already placed individuals
                end;
            end;
        end;
        
        [RalphaX,RCIalphaX,RbetaX,RCIbetaX] = PartitionEntropy(NullX,levels,weights,[],0);                  % Partition diversity for radnomized X data set
        [RalphaY,RCIalphaY,RbetaY,RCIbetaY] = PartitionEntropy(NullY,levels,weights,[],0);                  % Partition diversity for randomized Y data set
        
        ExpAlphaDiff(:,:,i) = RalphaX - RalphaY;                                                            % Calculate difference in alpha between X and Y data sets
        ExpBetaDiff(:,:,i) = RbetaX - RbetaY;                                                               % Calculate difference in beta between X and Y data sets
    end;

% This section uses the null distirbutions to determine signifiance values
    Palpha=[];                                                                                              % Allocate matrices for Pvalues
    Pbeta=[];
    
    for i = 1:cL+2                                               
        Palpha(1,i) = pvalue(ObsAlphaDiff(1,i),ExpAlphaDiff(1,i,:),2);                                      % Calculate two-tail probabilities
        Palpha(2,i) = pvalue(ObsAlphaDiff(2,i),ExpAlphaDiff(2,i,:),2);
        Palpha(3,i) = pvalue(ObsAlphaDiff(3,i),ExpAlphaDiff(3,i,:),2);
        Palpha(4,i) = pvalue(ObsAlphaDiff(4,i),ExpAlphaDiff(4,i,:),2);
    end;
    for i = 1:cL+1
        Pbeta(1,i) = pvalue(ObsBetaDiff(1,i),ExpBetaDiff(1,i,:),2);                                         % Calculate two-tail probabilities
        Pbeta(2,i) = pvalue(ObsBetaDiff(2,i),ExpBetaDiff(2,i,:),2);
        Pbeta(3,i) = pvalue(ObsBetaDiff(3,i),ExpBetaDiff(3,i,:),2);
        Pbeta(4,i) = pvalue(ObsBetaDiff(4,i),ExpBetaDiff(4,i,:),2);
    end;

return;