%PATHDATA: 	Generates a data set by simulating all of the theoretical search tactics.
%				For each iteration, all tactics are fit to the same set of point coordinates.
%
%	USAGE:	[data,grps] = 	pathdata(crds,start,keep)
%
%				data =		a [n x 15] matrix of multivariate path statistics:
%     		   	   		1) total path length
%                 		   2) minimum step length
%                				3) maximum step length
%                       	4) mean step length
%                       	5) std of step length
%                       	6) autocorrelation of step length
%                       	7) number of path crossings
%                       	8) std of path distances between crossings
%                      		9) angular vector autocorrelation
%                   			10) minimum absolute angular deviation
%                   			11) maximum absolute angular deviation
%                   			12) mean of the absolute angular deviations
%									13) std of the absolute angular deviations
%                   			14) mean of the signed angular deviations
%                   			15) std of the signed angular deviations
%				grps = 		a grouping vector
%				-------------------------------------------------------
%				crds = 		a matrix of point coordinates, the number of iterations is 
%								is determined by the number of sets of point coordinates
%				start = 		optional, the starting point for the search tactics {default = random}
%				keep = 		optional, saves the path statistics
%									0 = no {default}
%									1 = yes			
%
%	Chris Higgins 5/22/2000
	
%	REQUIRES: 	distmatrix, pathrand, pathstat, pathmax, pathmin, pathnear, 
%					pathreci, pathinve, pathtraj, pathpear, pathlevy, makegroups	
%
%	Modified: 6/17/2001 - to add Gaussian random walk 

function [data,grps] = pathdata(crds,start,keep)

	[x,y,z] = size(crds);							% Determine number of sets of points
   
   if(nargin < 1)													
   	error('PATHDATASAME: A matrix of point coordinates must be supplied.');
   end;
   if(nargin < 2)
      start = [];
   end;
   if(nargin < 3)
      keep = [];
   end;

   if(isempty(start))
      start = ceil(rand(z) * x);					% Generate random starting point
   end;
	if(isempty(keep))									% Allocate input arguments
      keep = 0;
   end;
   

   thresh = 0;											% Set threshold value for 'pathstat'
   
   obs = 0;												% Allocate matrices and scalars
   data = zeros(7 * z,15);			
   g = [];
      
    for i = 1:z
        pts = (crds(:,:,i));							% Pull one set of crds from matrix
		dist = distmatrix(pts);						% Calculate distance matrix

        
    
   % Cycle thru all models for the same set of points and distance matrix
   
   	    obs = obs + 1								% Display current iteration number    
      
        minpath = pathmin(dist,start(i));
      	    data(0 * z + i,:) = (pathstat(pts(minpath,:),thresh))';
       	
        nearpath = pathnear(pts,start(i));
     	    data(1 * z + i,:) = (pathstat(pts(nearpath,:),thresh))';
       	
        recipath = pathreci1(dist,start(i));
 	        data(2 * z + i,:) = (pathstat(pts(recipath,:),thresh))';
         
        invepath = pathinve1(dist,start(i));
   	        data(3 * z + i,:) = (pathstat(pts(invepath,:),thresh))';
       
        trajpath = pathtraj(pts);
            data(4 * z + i,:) = (pathstat(pts(trajpath,:),thresh))';
     
%     pearsonpath = pathpear(pts);
%   	   data(5 * z + i,:) = (pathstat(pts(pearsonpath,:),thresh))';
      
%      levypath = pathlevy(pts);
%	  		data(6 * z + i,:) = (pathstat(pts(levypath,:),thresh))';
      
%      gauspath = pathgaus(pts);
%	  		data(7 * z + i,:) = (pathstat(pts(gauspath,:),thresh))';  
    
        randpath = pathrand(pts,start(i));					
            data(5 * z + i,:) = (pathstat(pts(randpath,:),thresh))';
       
        maxpath = pathmax(dist,start(i));
            data(6 * z + i,:) = (pathstat(pts(maxpath,:),thresh))'; 
            
        if(keep == 1)
      	    save('c:/matlab/data/data.mat');   
        end;
    end;
   
    p = [1:7];												% Create grouping vector for data set
    g = ones(1,7) * z;
    grps = makegroups(p,g); 
   
   if(keep == 1)
    	save('c:/matlab/grps/grps.mat');   
   end;

return; 
 
