%PATHMASTER: 	From this function, you can run all of the search path
%					models. All you need is the number of food item, the 
%					starting point index number, the corresponding
%					model number. The points are random and are not the same for each tactic.
%
%	USAGE:		stat = pathmaster(p,dist,n,c,sigma,size,close,arena,iter,keep)
%
%					stat =  a [15 x 1] column vector of descriptive statistics:
%               	        	1) total path length
%                 	      2) minimum step length
%                   			3) maximum step length
%                       	4) mean step length
%                       	5) std of step length
%                       	6) autocorrelation of step length
%                       	7) number of path crossings
%                       	8) std of path distances between crossings
%                       	9) angular vector autocorrelation
%                      		10) minimum absolute angular deviation
%                      		11) maximum absolute angular deviation
%                      		12) mean of the absolute angular deviations
%                      		13) mean of the signed angular deviations
%                      		14) std of the signed angular deviations
%             				   15) std of the signed angular deviations
%					-------------------------------------------------------
%					p = 		a number corresponding to the appropriate model
%									1 = pathmin - minimum distance path
%									2 = pathnear - nearest neighbor path
%									3 = pathrecip - reciprocal distance path
%									4 = pathinvsq - inverse square distance path
%									5 = pathtraj - trajectory directed path
%									6 = pathpearson - Pearson random walk
%									7 = pathlevy - Levy random walk
%									8 = pathrand - random path
%									9 = pathmax - maximum distance path
%					dist =	optional, the type of food distribution
%									0 = random distribution {default}
%									1 = clumped distribution
%					n = 		optional, the number of food items to be consumed {default = 15}
%					c = 		optional, the number of clusters {default = 3}
%					sigma = 	optional, the standard deviation of the bivariate normal
%									distribution for each of the clusters {default = 3}
%					size =	optional, the size of the arena {default = 50 cm}
%					close = 	optional, determine how close an individual must be in order to
%									"find" the food items {default = 5%}
%					arena = 	optional, determines the shape of the arena
%									0 = square {default}
%									1 = circular
%					iter = 	optional, the number of iterations {default = 1}
%					keep = 	optional, saves the path statistics
%									0 = no {default}
%									1 = yes			
%
%	Chris Higgins 2/9/2000
%
%	REQUIRES: 	randpts, pathstat, distmatrix, pathrand, pathmax, pathmin, pathnear,
%					pathrecip, pathinvsq, paththeta, pathtraj, pathpearson, pathlevy
%

function stat = pathmaster(p,distrib,n,c,sigma,size,close,arena,iter,keep)
	if(nargin < 1)
      error('PATHMASTER: A search path must be specified.');
   end;
   if(nargin < 2)
      distrib = [];
   end;
   if(nargin < 3)
      n = [];
   end;
   if(nargin < 4)
      c = [];
   end;
   if(nargin < 5)
      sigma = [];
   end;
   if(nargin < 6)
      size = [];
   end;
   if(nargin < 7)
      close = [];
   end;
   if(nargin < 8)
      arena = [];
   end;
	if(nargin < 9)
      iter = [];
   end;
   if(nargin < 10)
      keep = [];
   end;


   if(isempty(distrib))
      distrib = 0;
   end;
 	if(isempty(n))
      n = 15;
   end;
   if(isempty(c))
      c = 3;
   end;
   if(isempty(sigma))
      sigma = 3;
   end;
   if(isempty(size))
      size = 50;
   end;
   if(isempty(close))
      close = 0.05 * size;
   end;
   if(isempty(arena))
      arena = 0;
   end;
   if(isempty(iter))
      iter = 1;
   end;
   if(isempty(keep))
      keep = 0;
   end;
   
   switch p
     	case 1
         stat = zeros(iter,15);
         for i = 1:iter
  				pts = randpts(n,distrib,c,sigma,arena,size);           
            dist = distmatrix(pts);
            start = ceil(rand * n);
            minpath = pathmin(dist,start);
        		stat(i,:) = (pathstat(pts(minpath,:),0))';
        end;
        if(keep == 1)
           minstat = stat;
           save('d:/matlab/data/minstat.mat','minstat');
        end;
      case 2
         stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size);
            dist = distmatrix(pts);
	        	start = ceil(rand * n);
				nearpath = pathnear(dist,start);
         	stat(i,:) = (pathstat(pts(nearpath,:),0))';
         end;
         if(keep == 1)
            nearstat = stat;
            save('d:/matlab/data/nearstat.mat','nearstat');
         end;
 	  	case 3
         stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size); 
            dist = distmatrix(pts);
	        	start = ceil(rand * n);
				recippath = pathrecip(dist,start);
            stat(i,:) = (pathstat(pts(recippath,:),0))';
         end;
         if(keep == 1)
            recipstat = stat;
            save('d:/matlab/data/recipstat.mat','recipstat');
         end;
      case 4   
         stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size);
            dist = distmatrix(pts);
	        	start = ceil(rand * n);
				invsqpath = pathinvsq(dist,start);
         	stat(i,:) = (pathstat(pts(invsqpath,:),0))';
         end;
         if(keep == 1)
            invsqstat = stat;
            save('d:/matlab/data/invsqstat.mat','invsqstat');
         end;
		case 5
		   stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size);
	        	trajpath = pathtraj(pts);
         	stat(i,:) = (pathstat(pts(trajpath,:),0))';
         end;
         if(keep == 1)
            trajstat = stat;
            save('d:/matlab/data/trajstat.mat','trajstat');
         end;
		case 6
      	stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size);
	        	pearsonpath = pathpearson(pts,size,close);
         	stat(i,:) = (pathstat(pts(pearsonpath,:),0))';
         end;
         if(keep == 1)
            pearsonstat = stat;
            save('d:/matlab/data/pearsonstat.mat','pearsonstat');
         end;
      case 7
         stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size);
	        	levypath = pathlevy(pts,size,close);
         	stat(i,:) = (pathstat(pts(levypath,:),0))';
         end;
         if(keep == 1)
            levystat = stat;
            save('d:/matlab/data/levystat.mat','levystat');
         end;
      case 8
     		stat = zeros(iter,15);
         for i = 1:iter
				pts = randpts(n,distrib,c,sigma,arena,size);
            start = ceil(rand * n);
            randpath = pathrand(pts,start);
            stat(i,:) = (pathstat(pts(randpath,:),0))';
         end;
         if(keep == 1)
            randstat = stat;
            save('d:/matlab/data/randstat.mat','randstat');
         end;
       case 9
         stat = zeros(iter,15);
         for i = 1:iter
  				pts = randpts(n,distrib,c,sigma,arena,size);        
            dist = distmatrix(pts);
            start = ceil(rand * n);
            maxpath = pathmax(dist,start);
      		stat(i,:) = (pathstat(pts(maxpath,:),0))';
         end;
         if(keep == 1)
            maxstat = stat;
            save('d:/matlab/data/maxstat.mat','maxstat');
         end;
      end;
return;
      
   
   