%PATHRANDWALK: 	Generates a data set by simulating the random walk seach tactics. 
%                   (Pearson, Levy, and Gaussian random walks)
%				For each iteration, all tactics are fit to the same set of point coordinates.
%
%	USAGE:	[data,grps] = 	pathrandwalk(crds,size,close,step,r,keep,path)
%
%				data =		a [n x 15] matrix of multivariate path statistics:
%     		   	   		    1) total path length
%                 		    2) minimum step length
%                		    3) maximum step length
%                       	4) mean step length
%                       	5) std of step length
%                       	6) autocorrelation of step length
%                       	7) number of path crossings
%                       	8) std of path distances between crossings
%                      		9) angular vector autocorrelation
%                   		10) minimum absolute angular deviation
%                   		11) maximum absolute angular deviation
%                   		12) mean of the absolute angular deviations
%							13) std of the absolute angular deviations
%                  			14) mean of the signed angular deviations
%                  			15) std of the signed angular deviations
%				grps = 		a grouping vector
%				-------------------------------------------------------
%				crds = 		a matrix of point coordinates, the number of iterations is 
%								is determined by the number of sets of point coordinates
%				size = 	    optional, gives the size of the arena {default = 50 cm}
%				close = 	optional, determines how close to the animal must be to a 
%								food item for the animal to "find" it {default = 5% of "size"}
%               step =      optional, steplength {default = 5% of "size = close}
%               r =         optional, determines which type of walk
%                               0 = unbiased
%                               1 = biased
%               path =      optional, dtermines which path to analyze
%                               0 = path from pt to pt
%                               1 = total path traveled
%
%
%	Chris Higgins 2/28/2002
	
%	REQUIRES: 	pathstat, pathgaus, pathpear, pathlevy, makegroups	
%
%	Modified: 5/23/2002 - to add biased random walks

function [data,grps] = pathrandwalk(crds,size,close,step,r,keep,path)

	[x,y,z] = size(crds);							% Determine number of sets of points
   
   if(nargin < 1)													
   	error('PATHDATASAME: A matrix of point coordinates must be supplied.');
   end;
   if(nargin < 2)
       size = [];
   end;
   if(nargin < 3)
      close = [];
   end;
   if(nargin < 4)
       step = [];
   end;
   if(nargin < 5)
       r = [];
   end;
   if(nargin < 6)
       keep = [];
   end;

   if(isempty(size))
       size = 50;
   end;
   if(isempty(close))								% Allocate input arguments
      close = 0.05 * size;
   end;
   if(isempty(step))
       step = close;
   end;
   if(isempty(r))
       r = 0;
   end;
   if(isempty(keep))
       keep = 0;
   end;
   
   thresh = 0;										% Set threshold value for 'pathstat'
   
   obs = 0;											% Allocate matrices and scalars
   data = zeros(9 * z,15);			
   g = [];
      
   for i = 1:z
      pts = (crds(:,:,i));							% Pull one set of crds from matrix
      obs = obs + 1								% Display current iteration number   
      
 % Cycle thru all models for the same set of points and distance matrix
   
      [pearsonpath,total_walk] = pathpear(pts,size,close,step,r);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearsonpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [levypath,total_walk] = pathlevy(pts,size,close,step,r);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearsonpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [gauspath,total_walk] = pathgaus(pts,size,close,step,r);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearsonpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [pearsonpath,total_walk] = pathpear(pts,size,close,step,1);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearsonpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [levypath,total_walk] = pathlevy(pts,size,close,step,1);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearsonpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [gauspath,total_walk] = pathgaus(pts,size,close,step,1);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [pearpath,total_walk] = pathpear(pts,size,close,step,2);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(pearpath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [levypath,total_walk] = pathlevy(pts,size,close,step,2);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(levypath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      [gauspath,total_walk] = pathgaus(pts,size,close,step,2);
            if(path == 0)
                data(0 * z + i,:) = (pathstat(pts(gauspath,:),thresh))';
            else;
                data(0 * z + i,:) = (pathstat(total_walk,thresh))';
            end;
      
      if(keep == 1)
        save('c:/data/random walks/randwalk.mat','data');   
      end;     
      
   end;
   
   p = [1:9];										% Create grouping vector for data set
   g = ones(1,9) * z;
   grps = makegroups(p,g); 
   
   if(keep == 1)
    	save('c:/data/random walks/randwalk.mat','data','grps');   
   end;

return; 
 
