% Shannon: Returns a value of the Shannon diversity index for a vector of counts.
%
%  USAGE: [S,H,J] = Shannon(X)
%
%         X = [Sites x Species] abundance matrix
%         --------------------------------------------------------
%         S = [Sites x 1] vector of species richnesses
%         H = [Sites x 1] vector of Shannon diversity indices
%         J = [Sites x 1] vector of Shannon evenness indices  
%           
% Chris Higgins 7/27/2004
%
%   Magurran, A.E. 2004.  Measuring Biological Diversity. Blackwell Publishing.
%       Malden, MA. 256 pps.

function [S,H,J] = Shannon(X)

    [r,c] = size(X);                                            % Determine the number of sites
    if(r > 1 & c == 1)                                          % Convert column vector to row vector
        X = X';
        [r,c] = size(X);
    end;
    
    S = zeros(r,1);
    H = zeros(r,1);                                             % Allocate column vector for diversity indices
    J = zeros(r,1);                                             % Allocate column vector for evenness indices
    for i = 1:r
        ind = find(X(i,:) > 0);                                 % Find all non-zero entries
        if(isempty(ind))    
            S(i) = 0;
            H(i) = 0;
            J(i) = 0;
        else;
            S(i) = length(ind);                                 % Calculate species richness
            N = sum(X(i,ind));                                  % Calculate total number of individuals
            H(i) = -sum((X(i,ind)./N) .* log(X(i,ind)./N));     % Calculate Shannon diversity index
            if(S(i) == 1)
                J(i) = 1;
            else;
                J(i) = H(i)/log(S(i));                          % Calculate Shannon evenness index
            end;
        end;
    end;
  
return; 