% ASSEMBLAGE: This function produces 95% confidence intervales for degree of fit between
%   various niche-based abundance models and empirical species abundance disributiuons.
%
%   Usage: [Diff,DiffCI] = assemblage1(data,iter,dev,doplot)
%
%   data = a [r x c] matrix of species abundances for "r" different sites
%   iter = optional, specifies the number of iterations to use when esimating the confidence intervals
%       {default = 1000}
%   dev = optional, specifies which type of deviations to use when assesing the degree of fit
%       Sum-of-absolute-difference = 0
%       Sum-of-squared-differences = 1 {default}
%       Sum-of-chi-squared0difference = 2
%   doplot = optional, specifies whether or not to plot the expected abudance {default = 0}
%   ---------------------------------------------------------------------------------------
%   Diff = a [r x 5] matrix of relative difference in degree of fit between model and data
%   DiffCI = a [r x 2] matrix of corresponding confidence intervals
%
%   Chris L. Higgins 5/5/2004
%

function [Diff,DiffCI] = assemblage(data,iter,dev,doplot)

    if(nargin < 2)                              % Allocate input arguements
        iter = [];
    end;
    if(nargin < 3)                              
        dev = [];
    end;
    if(nargin < 4)
        doplot = [];
    end;
    
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(dev))
        dev = 1;
    end;
    if(isempty(doplot))
        doplot = 0;
    end;

    [r,c] = size(data);                         % Determine the total number of sites and species

    Diff = [];
    for i = 1:r
        ind = find(data(i,:) > 0);              % Find the species that are present at each site
        Obs = data(i,ind);                      % Create a new vector of observed counts
        Obs = abs(sort(-Obs));                  % Rank the species abundances from high to low
        S = length(ind);                        % Calculate the species richness for each site
        N = sum(Obs);                           % Calculate the total # of individuals for each site
        SAD1 = zeros(iter,S);
        SAD2 = zeros(iter,S);
        SAD3 = zeros(iter,S);
        SAD4 = zeros(iter,S);
        SAD5 = zeros(iter,S);
        if(S ~= 0 & N ~= 0)
            for j = 1:iter
                SAD1(j,:) = domindecay(S,N);        % Model the site according to dominance decay
                SAD2(j,:) = brokenstick(S,N,2);     % Model the site according to brokenstick
                SAD3(j,:) = randfract(S,N);         % Model the site according to random fraction
                SAD4(j,:) = randassort(S,N);        % Model the site according to random assortment
                SAD5(j,:) = preemption(S,N);        % Model the site according to dominance premtion
            end;
        end;
        Exp1 = mean(SAD1);
        Exp2 = mean(SAD2);
        Exp3 = mean(SAD3);
        Exp4 = mean(SAD4);
        Exp5 = mean(SAD5);
        if(dev == 0)
            D1(i) = absdev(Obs,Exp1);           % Calculate the sum-of-absolute-differences b/w Obs & Exp
            D2(i) = absdev(Obs,Exp2);
            D3(i) = absdev(Obs,Exp3);
            D4(i) = absdev(Obs,Exp4);
            D5(i) = absdev(Obs,Exp5);
        end;
        if(dev == 1)
            D1(i) = sumsqdev(Obs,Exp1);         % Calculate the sum-of-squared-differences b/w Obs & Exp
            D2(i) = sumsqdev(Obs,Exp2);
            D3(i) = sumsqdev(Obs,Exp3);
            D4(i) = sumsqdev(Obs,Exp4);
            D5(i) = sumsqdev(Obs,Exp5);
        end;
        if(dev == 2)
            D1(i) = chi2(Obs,Exp1);             % Calculate the sum-of-chi-quared-differences b/w Obs & Exp
            D2(i) = chi2(Obs,Exp2);
            D3(i) = chi2(Obs,Exp3);
            D4(i) = chi2(Obs,Exp4);
            D5(i) = chi2(Obs,Exp5);
        end;
        D = [D1(i) D2(i) D3(i) D4(i) D5(i)];
        if(sum(D) ~= 0)
            D = D/sum(D);
        end;
        Diff = [Diff;D];
    end;
    
    DiffCI = [];
    for i = 1:r
        for k = 1:iter
            ind = find(data(i,:) > 0);          % Find the species that are present at each site
            if(~isempty(ind))
            Z = data(i,ind);
            Z1 = makegrps([1:length(Z)],Z)';
            Xsamp = samplewr(Z1');              % Create a bootstrapped vector of observed counts
            [u,f] = uniquef(Xsamp);
            Obs = abs(sort(-f));                % Rank the species abundances from high to low
            S = length(ind);                    % Calculate the species richness for each site
            N = sum(Obs);                       % Calculate the total # of individuals for each site
            SAD1 = zeros(iter,S);
            SAD2 = zeros(iter,S);
            SAD3 = zeros(iter,S);
            SAD4 = zeros(iter,S);
            SAD5 = zeros(iter,S);
            if(S ~= 0 & N ~= 0)
                for j = 1:iter
                    SAD1(j,:) = domindecay(S,N);    % Model the site according to dominance decay
                    SAD2(j,:) = brokenstick(S,N,2); % Model the site according to brokenstick
                    SAD3(j,:) = randfract(S,N);     % Model the site according to random fraction
                    SAD4(j,:) = randassort(S,N);    % Model the site according to random assortment
                    SAD5(j,:) = preemption(S,N);    % Model the site according to dominance premtion
                end;
            end;
            Exp1 = mean(SAD1);
            Exp2 = mean(SAD2);
            Exp3 = mean(SAD3);
            Exp4 = mean(SAD4);
            Exp5 = mean(SAD5);
            if(dev == 0)
                D1(i) = absdev(Obs,Exp1);       % Calculate the sum-of-absolute-differences b/w Obs & Exp
                D2(i) = absdev(Obs,Exp2);
                D3(i) = absdev(Obs,Exp3);
                D4(i) = absdev(Obs,Exp4);
                D5(i) = absdev(Obs,Exp5);
            end;
            if(dev == 1)
                D1(i) = sumsqdev(Obs,Exp1);     % Calculate the sum-of-squared-differences b/w Obs & Exp
                D2(i) = sumsqdev(Obs,Exp2);
                D3(i) = sumsqdev(Obs,Exp3);
                D4(i) = sumsqdev(Obs,Exp4);
                D5(i) = sumsqdev(Obs,Exp5);
            end;
            if(dev == 2)        
                D1(i) = chi2(Obs,Exp1);         % Calculate the sum-of-chi-quared-differences b/w Obs & Exp
                D2(i) = chi2(Obs,Exp2);
                D3(i) = chi2(Obs,Exp3);
                D4(i) = chi2(Obs,Exp4);
                D5(i) = chi2(Obs,Exp5);
            end;
        end;
            DR = [D1(i) D2(i) D3(i) D4(i) D5(i)];
            if(sum(DR) ~= 0)
                DR = DR/sum(DR);
            end;
            DiffR(i,:,k) = DR;
        end;

        D_CI = [];                              % Esimate 95% confidence intervals for differences b/w Obs & Exp
        for j = 1:5
            W(1,:) = DiffR(i,j,:);
            ci = bootci(W')';
            D_CI = [D_CI ci];        
        end;
        DiffCI = [DiffCI;D_CI];
    end;
    
    if(doplot > 0)                              % Plots expected species abundance distributions
        figure;
        plot(Exp1);
        hold on;
        plot(Exp2);
        plot(Exp3);
        plot(Exp4);
        plot(Exp5);
        plot(Obs,'r');
    end;
    
return;