% ASSEMBLAGEAIC: This function produces 95% confidence intervales for degree of fit between
%   various niche-based abundance models and empirical species abundance disributiuons.
%
%   Usage: [Diff,DiffCI] = assemblageAIC(data,iter,dev,doplot)
%
%   data = a [r x c] matrix of species abundances for "r" different sites
%   iter = optional, specifies the number of iterations to use when esimating the confidence intervals
%       {default = 1000}
%   dev = optional, specifies which type of deviations to use when assesing the degree of fit
%       Sum-of-absolute-difference = 0
%       Sum-of-squared-differences = 1 {default}
%       Sum-of-chi-squared0difference = 2
%   doplot = optional, specifies whether or not to plot the expected abudance {default = 0}
%   ---------------------------------------------------------------------------------------
%   Diff = a [r x 5] matrix of relative difference in degree of fit between model and data
%   DiffCI = a [r x 2] matrix of corresponding confidence intervals
%
%   Chris L. Higgins 5/5/2004
%

function [Best,Diff] = assemblageAIC(data,iter,grps,rel,doplot)

    [r,c] = size(data);                         % Determine the total number of sites and species
    
    if(nargin < 2)                              % Allocate input arguements
        iter = [];
    end;
    if(nargin < 3)
        grps = [];
    end;
    if(nargin < 4)
        rel = [];
    end;
    if(nargin < 5)
        doplot = [];
    end;
    
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(grps))
        grps = [1:r];
    end;
    if(isempty(rel))
        rel = 0;
    end;
    if(isempty(doplot))
        doplot = 0;
    end;

    [u,f] = unique1(grps);
    lenu = length(u);
    X = zeros(lenu,c);
    
    for i = 1:lenu
        ind = find(grps == u(i));
        if(length(ind) > 1)
            X(i,:) = sum(data(ind,:));
        else;
            X(i,:) = data(ind,:);
        end;
    end;
    
    data = X;
    [r,c] = size(data);                         % Determine the total number of sites and species
       
    Diff = zeros(r,5);
    Best = zeros(r,1);
    
    for i = 1:r
        ind = find(data(i,:) > 0);              % Find the species that are present at each site
        Obs = data(i,ind);                      % Create a new vector of observed counts
        Obs = abs(sort(-Obs));                  % Rank the species abundances from high to low
        if(rel ~= 0)
            Obs = Obs / sum(Obs);
        end;
        S = length(ind);                        % Calculate the species richness for each site
        N = sum(Obs);                           % Calculate the total # of individuals for each site
        SAD1 = zeros(iter,S);
        SAD2 = zeros(iter,S);
        SAD3 = zeros(iter,S);
        SAD4 = zeros(iter,S);
        SAD5 = zeros(iter,S);
        
        if(S ~= 0 & N ~= 0)
            for j = 1:iter
                SAD1(j,:) = domindecay(S,N,[],rel);            % Model the site according to dominance decay
                SAD2(j,:) = brokenstick(S,N,2,rel);         % Model the site according to brokenstick
                SAD3(j,:) = randfract(S,N,[],rel);             % Model the site according to random fraction
                SAD4(j,:) = randassort(S,N,[],rel);            % Model the site according to random assortment
                SAD5(j,:) = preemption(S,N,[],rel);            % Model the site according to dominance premtion
                
                D1(j) = AIC(Obs,SAD1(j,:),2);              % Calculate the AIC criterion b/w Obs & Exp
                D2(j) = AIC(Obs,SAD2(j,:),2);
                D3(j) = AIC(Obs,SAD3(j,:),3);
                D4(j) = AIC(Obs,SAD4(j,:),2);
                D5(j) = AIC(Obs,SAD5(j,:),2);
            end;  
            Diff(i,1) = mean(D1);                              % Estimate mean AIC values
            Diff(i,2) = mean(D2);
            Diff(i,3) = mean(D3);
            Diff(i,4) = mean(D4);
            Diff(i,5) = mean(D5);
            Diff(i,:) = Diff(i,:) - min(Diff(i,:));
            Best(i) = find(Diff(i,:) == min(Diff(i,:)));
            
            Exp1 = mean(SAD1);                          % Estimate average abundance distribution
            Exp2 = mean(SAD2);
            Exp3 = mean(SAD3);
            Exp4 = mean(SAD4);
            Exp5 = mean(SAD5);
        end;

    end;
    
    if(doplot > 0)                                      % Plots expected species abundance distributions
        figure;
        plot(Exp1);
        hold on;
        plot(Exp2);
        plot(Exp3);
        plot(Exp4);
        plot(Exp5);
        plot(Obs,'r');
    end;
    
return;