% BROKENSTICK: This function provides abundance data that follows MacArthur's Broken Stick Model
%   according to simultaneuos breakage, the formula provided in Magurran (1988), or Tokeshi's
%   sequential breakage (1990).
%
%   Uasge:  Abundance = brokenstick(S,N,model,rel)
%
%   S = the total number of species for each community
%   N = the total number of individuals in each community
%   model = optional, specifes whether to use a randomized version (0) or the equation verison (1) 
%   from Magurran (1988) {default = 1}
%   rel = optional, specifies whether or not to make the abundances relative {default = 0}
%   ----------------------------------------------------------------------------------------------
%   Abundance = a [1 x S] vector of species abundances
%
%   Chris L. Higgins 3/5/2003
%
%   Magurran, A.E. 1988. Ecological Diversity and Its Measurement. Princeton University
%       Press. Princeton, NJ. 179 pps.
%
%   Tokeshi, M. 1990. Niche apportionment or random assortment: species abundance patters revisited.
%       Journal of Animal Ecology, 59: 1129-1146. 
%

function Abundance = brokenstick(S,N,model,rel)

    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        rel = [];
    end;
    if(isempty(model))
        model = 1;
    end;
    if(isempty(rel))
        rel = 0;
    end;

    Abundance = zeros(1,S);
    
    if(model == 0)                              % Based on simultaneous breakage
        r = rand(1,S-1) * N;
        r = sort(r);
        x = [0 r N]; 
        for i = 1:S
            Abundance(i) = (x(i+1) - x(i));
        end;
        Abundance = abs(sort(-Abundance));
    end;

    if(model == 1)                              % Based on Magurran
        for i = 1:S
            n = 0;
            for j = i:S
                n = n + (1 / j);
            end;
            Abundance(i) = (N/S) * sum(n);
        end;
    end;
    
    if(model == 2)                              % Based on sequential breakage (probability)
        x = [0 1];
        for i = 1:S-1
            in = rand;
            ind = find(x >= in);
            D = x(ind(1)) - x(ind(1)-1);
            r = rand * D;
            s = x(ind(1)-1) + r;
            x = [x s];
            x = sort(x);
        end;
        for j = 1:S
            Ab(j) = (x(j+1) - x(j)) * N;
        end;
        Abundance = abs(sort(-Ab));
    end;
    
    if(rel ~=0)
        Abundance = Abundance / N;
    end;
    
return;