% CLUMPING: This function calculates the Morisita Community Index for a presence-absence data set as
%           described in Leibold and Mikkelson 2002.  It compares it to an expected values and uses
%           a ChiSquare test to assess significance.
%
%   USAGE: [M,Mpr,ComBnd] = clumping(X,ord,Comm,ax)
%
%       X = [sites x species] incidence matrix
%       ord = optional, specifies how to ordinate the data matrix
%           0 = no ordination is performed
%           1 = reciprical averaging {default}
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals
%       Comm = optional, specifies whether to look at the Community (1) or Range (0) perspective {default = 0}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       -------------------------------------------------------------------------------------------------------
%       M = Morisita Community index value
%       Mpr = pvalue for Morisita index
%           if M < 1, performs a left-tailed test ---- less clumped than expected
%           if M > 1, performd a right-tailed test ---- more clumped than expected
%       ComBnd = vector of boundaries
%
%   Chris L. Higgins 3/14/2005
%
%   Leibold,M.A. and Mikkelson,G.M.  2002.  Coherence, species turnover, and boundary clumping: elements 
%       of meta-community structure.  Oikos 97: 237-250.
%
%   REQUIRES: makebinary, allzeros, ordinate
%


function [M,Mpr,ComBnd] = clumping(X,ord,Comm,ax)

    if(nargin < 2)                                      % Set default input arguements
        ord = [];
    end;
    if(nargin < 3)
        Comm = [];
    end;
    if(nargin < 4)
        ax = [];
    end;

    if(isempty(ord))
        ord = 1;
    end;
    if(isempty(Comm))
        Comm = 0;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    
    if(Comm == 0)                                       % If range is considered, transpose the matrix
        X = X';
    end;
    
    X = makebinary(X);                                  % Make sure the data matrix is a presence-absence matrix
        
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = nestsort(X);                            % Ordinate data matrix by species richness and occurence values
    end;

    [r,c] = size(Y);
    ComBnd = zeros(c,1);
    M = 0;
    ComBndChi = 0;
    
    for i = 1:r
        ind1 = find(Y(i,:) == 1);
        First = ind1(1);
        Last = ind1(length(ind1));
        for j = 1:c
            if(First == j)
                ComBnd(j) = ComBnd(j) + 1;
            end;
            if(Last == j)
                ComBnd(j) = ComBnd(j) + 1;
            end;
        end;
    end;
    TotComBnd = (r*2) - ComBnd(1)-ComBnd(c);
    ExpComBnd = TotComBnd / (c-2);
    df = -1;

    for i = 2:c-1
        M = M + ((ComBnd(i)/TotComBnd)*((ComBnd(i)-1)/(TotComBnd-1)));
        ComBndChi = ComBndChi + ((ComBnd(i) - ExpComBnd)^2/ExpComBnd);
        df = df + 1;
    end;
    M = M * (c-2);

% Calculate statistical significance based on chi-square distirbution
    if(M < 1)
        Mpr = chi2cdf(ComBndChi,df);                    % If observed is less than expected (1) perform a left-tailed test
    else;
        Mpr = 1-chi2cdf(ComBndChi,df);                  % If observed is greater than expected (1) perform a right-tailed test
    end;
    
return;