% CLUMPINGSim: This function calculates the Morisita Community Index for a presence-absence data set as
%           described in Leibold and Mikkelson 2002.  It compares it to an expected values and uses
%           a ChiSquare test to assess significance.
%
%   USAGE: [M,Mpr,ComBnd] = clumpingSim(X,ord,Comm,iter,ax)
%
%       X = [sites x species] incidence matrix
%       ord = optional, specifies how to ordinate the data matrix
%           0 = no ordination is performed
%           1 = reciprical averaging {default}
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals
%       Comm = optional, specifies whether to look at the Community (1) or Range (0) perspective {default = 0}
%       iter = optional, specifies whether to use Chi-square test (0) or how many iterations in randomization procedures (1)
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       -------------------------------------------------------------------------------------------------------
%       M = Morisita Community index value
%       Mpr = pvalue for Morisita index
%           if M < 1, performs a left-tailed test ---- less clumped than expected
%           if M > 1, performd a right-tailed test ---- more clumped than expected
%       ComBnd = vector of boundaries
%
%   Chris L. Higgins 3/14/2005
%
%   Leibold,M.A. and Mikkelson,G.M.  2002.  Coherence, species turnover, and boundary clumping: elements 
%       of meta-community structure.  Oikos 97: 237-250.
%
%   REQUIRES: makebinary, allzeros, ordinate
%


function [M,Mpr,ComBnd,MM,SM] = clumpingSim(X,ord,Comm,iter,ax)

    if(nargin < 2)                                      % Set default input arguements
        ord = [];
    end;
    if(nargin < 3)
        Comm = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;

    if(isempty(ord))
        ord = 1;
    end;
    if(isempty(Comm))
        Comm = 0;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    if(isempty(iter))
        iter = 0;
    end;
    
    if(Comm == 0)                                       % If range is considered, transpose the matrix
        X = X';
    end;
    
    X = allzeros(X);                                    % Remove any rows or columns that contain all zeros
    X = makebinary(X);                                  % Make sure the data matrix is a presence-absence matrix
        
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = nestsort(X);                            % Ordinate data matrix by species richness and occurence values
    end;
 
    [M,ComBndChi,df,ComBnd] = clumpingf(Y);
    
% Calculate statistical significance based on chi-square distirbution
    if(iter == 0)
        MM=[];
        SM=[];

        if(M < 1)
            Mpr = chi2cdf(ComBndChi,df);                    % If observed is less than expected (1) perform a left-tailed test
        else
            Mpr = 1-chi2cdf(ComBndChi,df);                  % If observed is greater than expected (1) perform a right-tailed test
        end;
    end;
    
% Calculate significance based on randomization procedures
    Mexp = [];
    if(iter ~=0)
        for i = 1:iter
            
% This null model uses a wrap-around approach            
            Mfill = boundfill(Y,Comm);
            Null = NicheRand(Mfill',5)';

% This null model uses the same model as used to assess turnover
 
                %Null = replacenull(Y,0);            

% This null model is used to assess turnover
            %Null = randomnull(Y,3);
            
            Mexp(i) = clumpingf(Null);                  % Calculates the Morista's index for each null matrix
        end;
        MM = mean(Mexp);                         
        SM = std(Mexp);
        [H,Mpr] = ztest(M,MM,SM,.05,0);                % Perform two-tailed z-test comparing null distrbution to observed
    end;
    
return;