% COHERENCE:  This function assses the coherence of a incidence matrix by counting the number of
%   embedded absences, which is an interruption in a range or community.
%
%   USAGE: [Abs,Apr,MA,SA] = coherence(X,ord,model,iter,ax)
%
%       X = [sites x species] incidence matrix
%       ord = optional, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging {default}
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature) 
%           2 = Species richness per site is equiprobable and species occurrence is fixed 
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0)
%           4 = Species richness per site is proportional & species occurrence is fixed
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1)  {default}
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       iter = optional, specifies how many iterations to perform {default = 200}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       -------------------------------------------------------------------------------------------------------------------------------------
%       Abs = the number of embedded absences in a given ordicated matrix
%       Apr = significance value for a two-tailed test --- either less coherent (checkerboard) or more coherent than expected by chance
%       MA = mean number of rembedded absences base on null models
%       SA = standard deviation of number of embedded absences based on null models
%   
%   Chris L. Higgins 3/15/2005
%
%   Leibold,M.A. and Mikkelson,G.M.  2002.  Coherence, species turnover, and boundary clumping: elements 
%       of meta-community structure.  Oikos 97: 237-250.
%
%   REQUIRES: makebinary, allzeros, ordinate, embedabs, nullmatrix, random4
%


function [Abs,Apr,MA,SA] = coherence(X,ord,model,iter,ax)

    if(nargin < 2)                                      % Set default input arguments
        ord = [];
    end;
    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 1;
    end;
    if(isempty(model))
        model = 5;
    end;
    if(isempty(iter))
        iter = 200;
    end;
    if(isempty(ax))
        ax = 1;
    end;

    X = makebinary(X);                                  % Make sure the data matrix is a presence-absence matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            Y = nestsort(X);                            % Ordinate data matrix according to species richness and occurence values
    end;

    Abs = embedabs(Y);                                  % Count the number of embedded absences with a range or community
    
    if(iter > 0)
        for i = 1:iter                                  % Cycle through the iterations to create a null distribution
            Null = randomnull(X,model);

            switch(ord)                                 % Allow various kinds of ordination of data matrix
                case 0
                    EY = Null;                          % Do not ordinate the data matrix
                case 1
                    EX = allzeros(Null);                % Remove any rows or columns that contain all zeros
                    EY = ordinate(EX,ax);               % Ordinate the data matrix by reciprical averaging
                case 2
                    EY = maxpack(Null);                 % Ordinate the data matrix by maximumally packing it (nested calculator)
                case 3
                    EY = nestsort(Null);                % Ordinate data matrix according to species richness and occurence values
            end;

            EAbs(i) = embedabs(EY);                     % Count embedded absences
        end;
  
        MA = mean(EAbs);                                % Estimate mean emebedded absences from null matrices
        SA = std(EAbs);                                 % Estimate standard devaition of emebedded absences from null matrices
        [H,Apr] = ztest(Abs,MA,SA,.05,0);               % Use z-test to estimate significance values (two-tailed)
    end;
    
return;