% DISTRIBDIFF: This function compares two sample distributions to determine whether they come from the same
%              underlying population.
%
%   Usage:  [ObsDev,pr] = distribdiff(X,Y,stat,iter,repl,tail,doplot)
%
%   X = vector
%   Y = vector
%   stat = optional, specifies what test statistic to use
%           0 = sum of absolute deviations
%           1 = sum of squared deviations {default}
%           2 = chi-square devistions
%   iter = optional, specifies how many iterations to perform when randomizing {defaul = 1000}
%   repl = optional, specied whether or not to sampel with replacment
%           0 = sample without replacement {default}
%           1 = sample with replacement
%   tail = optional. secifies whether to one-tailed or two-tailed test
%           1 = left tail test
%           2 = two tailed test {default}
%           3 = right tailed test
%   doplot = optional, specifies whether to plot the expected distribution {default = 0}
%   ---------------------------------------------------------------------------------------------------------------------
%   ObsDev = the observed test statistic
%   pr = the corresponding p value
%
%   Chris L. Higgins    3/2/2006
%

function [ObsDev,pr] = distribdiff(X,Y,stat,iter,repl,tail,doplot)

    if(nargin < 3)                                      % allocate input arguements
        stat = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        repl = [];
    end;
    if(nargin < 6)
        tail = [];
    end;
    if(nargin < 7)
        doplot = [];
    end;
    
    if(isempty(stat))                                   % set default values for input arguements
        stat = 1;
    end;
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(repl))
        repl = 0;
    end;
    if(isempty(tail))
        tail = 2;
    end;
    if(isempty(doplot))
        doplot = 0;
    end;
    
    [rX,cX] = size(X);                                  % determine size of X vector
    [rY,cY] = size(Y);                                  % determine size of Y vector
    
    if(size(X,2) > 1)                                   % convert row vectors to column vectors
        X = X';
      end;
    if(size(Y,2) > 1)
        Y = Y';
    end;

    Z = [X;Y];                                          % combine vectors for randomization
    
    if(stat == 0)                                       % calcualte observed test statistic, whichever is desired
        ObsDev = absdev(X,Y);
    end;
    if(stat == 1)
        ObsDev = sumsqdev(X,Y);
    end;
    if(stat == 2)
        ObsDev = chi2(X,Y);
    end;
        
    for i = 1:iter
        if(repl == 0)                                   % Sample without replacement
            R = randperm(length(Z));                    % randomly permute indices for pooled vector
            Xr = Z(R(1:rX));                            % use first half of randomized indices for X vector
            Yr = Z(R(rX+1:rX+rY));                      % use second half of randomized indices for Y vector
        end;
        if(repl == 1)                                   % Sample with replacement
            Xr = samplewr(Z,rX);                        % randomly pull observations (length X) from pooled vector
            Yr = samplewr(Z,rY);                        % randomly pull observations (length Y) from pooled vector
        end;
        
        if(stat == 0)                                   % calcualte expected test statistic, whichever is desired
            ExpDev(i) = absdev(Xr,Yr);
        end;
        if(stat == 1)
            ExpDev(i) = sumsqdev(Xr,Yr);
        end;
        if(stat == 2)
            ExpDev(i) = chi2(Xr,Yr);
        end;
    end;

    pr = pvalue(ObsDev,ExpDev,tail);                    % calcaulte p-value from observed statistic and expected distribution

    if(doplot ~= 0)                                     % if desired, this will plot the expected distirbution and test statistic
        figure;
        hist(ExpDev)
        N = hist(ExpDev);
        hold on;
        plot(ObsDev,[0:max(N)],'r.');
    end;
    
return;