% mcvi (metacommunity visual inspection)
%
%   USAGE: [Y,Z,Bound,Spp] = mcvi(X,ord,Comm,ax,plotAbs)
%
%       X = [sites x species] incidence matrix
%       ord = optional, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging {default}
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals
%       Comm = optional, specifies whether to look at the Community (1) or Range (0) perspective {default = 0}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       plotAbs = optional, specifies whether or not to plot embedded absences
%           0 = does not plot embedded absences
%           1 = does plot emebeddes absences {default}
%       ---------------------------------------------------------------------------------------------------------
%       Y = [sites x species] ordinated matrix or original data if is specified
%       Z = [sites x species] matrix of emebedded absences
%       Bound = [sites x 2] if range perspective or [species x 2] if community perspective, first column is indices of
%           original data matrix for new ordinated matrox and second column is boundary values
%       Spp = [sites x 1] vector of species indices
%
%   Chris L. Higgins 2/23/2009
%

function [Y,Z,Bound,Spp] = mcvi(X,ord,Comm,ax,plotAbs)

    if(nargin < 2)                                      % Set default input arguments
        ord = [];
    end;
    if(nargin < 3)
        Comm = [];
    end;
    if(nargin < 4)
        ax = [];
    end;
    if(nargin < 5)
        plotAbs = [];
    end;
    
    if(isempty(ord))
        ord = 1;
    end;
    if(isempty(Comm))
        Comm = 0;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    if(isempty(plotAbs))
        plotAbs = 1;
    end;
    
    X = makebinary(X);                                  % Make sure the data matrix is a presence-absence matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            X = allzeros(X);
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            [Y,ir,ic] = ordinate(X,ax);                 % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            [Y,ir,ic] = nestsort(X);                    % Ordinate data matrix according to species richness and occurence values
    end;
    
    [sites,species] = size(Y);   

    [FirstSpecies,LastSpecies,FirstSite,LastSite] = boundfind(Y);
   
    Z = zeros(sites,species);
    for i = 1:sites                                                     % Cycle through sites
        for j = 1:species                                               % Cycle through species        
            if(Y(i,j) == 0)
                flag = 0;
                if(j > FirstSpecies(i) & j < LastSpecies(i))            % Find absences within sites
                    Z(i,j) = 1;
                    flag = 1;
                end;
                if(i > FirstSite(j) & i < LastSite(j) & flag == 0)      % Find absences within species
                    Z(i,j) = 1;
                end;
            end;
        end;
    end;
    
    if(ord == 0 | ord == 2)
        ir = 1:sites';
        ic = 1:species;
    end;
    
    [M,Mpr,ComBnd] = clumping(Y,0,Comm,ax);
    if(Comm == 0)
        Bound = [ir ComBnd];
        Spp = ic;
    else
        Bound = [ic' ComBnd];
    end;
    
   
%  This portion of the function produces the figure
    [rY,cY] = find(Y >= 1);
    [rZ,cZ] = find(Z >= 1);
    rY = (sites+1) - rY;
    rZ = (sites+1) - rZ;

    figure;
    plot(cY-.5,rY-.5,'+')

    ax1 = gca; 
    hold on;
    if(plotAbs ~= 0)
        plot(cZ-.5,rZ-.5,'ro');
    end;

    putx('Species');
    puty('Sites');

    grid on;
    set(0,'DefaultLineLineWidth','default') 
    
    set(ax1,'YTick',[1:sites]);
    set(ax1,'YLim',[0 sites]);
    set(ax1,'YTickLabel',flipud(Bound(:,1)),'YAxisLocation','left')
    set(ax1,'XTick',[1:species]); 
    set(ax1,'XTickLabel',ic);
    axis equal;
    axis([0 species 0 sites]);
     
    ax2 = axes('Position',get(ax1,'Position'),'Color','none'); 
  
    set(ax2,'YTick',[1:sites]);
    set(ax2,'YLim',[0 sites]);
    set(ax2,'YTickLabel',flipud(Bound(:,2)),'YAxisLocation','right')
    set(ax2,'XTick',[1:species],'XTickLabel',[]); 
    axis equal;
    axis([0 species 0 sites]);
    puty('Boundaries');
    
return;