% METACOMMUNITY2:  This function quantifies coherence, species turnover, and boundary clumping
%   within a site-by-species incidence matrix.  Based on these measures, one can indentify which
%   type of meta-community pattern (e.g., Checkerboards, Nested subsets, Clementsian, Gleasonian,
%   Evenly spaced gradients, or Random) is most prevalent.
%
%   USAGE: [Abs,Apr,MA,SA,Re,Rpr,MR,SR,M,Mpr] = metacommunity2(X,ord,model,Comm,iter,ax)
%
%       X = [sites x species] incidence matrix
%       ord = optional, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging {default}
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature) 
%           2 = Species richness per site is equiprobable and species occurrence is fixed 
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0) --- {default}
%           4 = Species richness per site is proportional & species occurrence is fixed
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1) 
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       Comm = optional, specifies whether to look at the Community (1) or Range (0) perspective {default = 0}
%       iter = optional, specifies how many iterations to perform {default = 200}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       ---------------------------------------------------------------------------------------------------------
%       Abs = the number of embedded absences in a given ordicated matrix
%       Apr = pvalue associated with embedded absences
%       MA = mean number of rembedded absences base on null models
%       SA = standard deviation of number of embedded absences based on null models
%       Re = number of replacemnets (checkerboard)
%       Rpr = pvalue associated with replacements
%       MR = mean number of replacements base on null models
%       SR = standard deviation of number of repalcements based on null models
%       M = Morisita Community index value
%       Mpr = pvalue associated with Morisita index
%
%   Leibold,M.A. and Mikkelson,G.M.  2002.  Coherence, species turnover, and boundary clumping:
%       elements of meta-community structure.  Oikos 97: 237-250.
%
%   CHRIS L. HIGGINS 4/27/2005
%

function [Abs,Apr,MA,SA,Re,Rpr,MR,SR,M,Mpr] = metacommunity2(X,ord,model,Comm,iter,ax)

    if(nargin < 2)                                              % Set default input arguements
        ord = [];
    end;
    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        Comm = [];
    end;
    if(nargin < 5)
        iter = [];
    end;
    if(nargin < 6)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 1;
    end;
    if(isempty(model))
        model = 3;
    end;
    if(isempty(Comm))
        Comm = 0;
    end;
    if(isempty(iter))
        iter = 200;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    
    [Abs,Apr,MA,SA] = coherence2(X,ord,model,iter,ax);           % Assess coherence
    [Re,Rpr,MR,SR] = turnover2(X,ord,Comm,iter,ax);              % Assess species turnover
    [M,Mpr] = clumping2(X,ord,Comm,ax);                          % Assess boundary clumping 
    
return;