% NESTBEZ: This function finds the Bezier curve that corresponds to the fill of a nested matrix.
%   The curve will be concave down for fills less than 50% and convex down for fills greater than
%   50% fill. 
%
%   Usage: [fill,curve] = nestedbezier(data)
%
%   data = [r x c] data matrix with rows representing "islands" and columns representing "species"
%   ------------------------------------------------------------------------------------------------
%   fill = the percentage of cells that contain "presences"
%   curve = a [n x 2] matrix of point coordinates that define the Bezier curve corresponding to fill
%
%   Chris L. Higgins 3/12/2003
%

function [fill,curve] = nestbez(data)
    
    [r,c] = size(data);                                 % Determine the size of the data matrix
    
    rfill = find(data > 0);                             % Find all "presence" cells
    pf = (length(rfill) / (r*c));                       % Calulcate fill as decimal
    fill = pf * 100; 

    p = 0;                                              % Optimization rountine to find the exact location 
    p = fminsearch('bezierarea',p,[],data,fill);        %   of the control points for the Bezier curve
    
    if(fill < 50)                                       % Create a matrix of point coordinates that
        Cpt1 = [0.5 (0.5+(p*(r-1)))];                   %   correspond to the optimized control pts
        Cpt2 = [((c-0.5)-(p*(c-1))) r-0.5];
        pts = [0.49 0.49;Cpt1;Cpt2;c-0.49 r-0.49];
    else;
        Cpt1 = [(0.5+(p*(c-1))) 0.5];
        Cpt2 = [c-0.5 ((r-0.5)-(p*(r-1)))];
        pts = [0.49 0.49;Cpt1;Cpt2;c-0.49 r-0.49];
    end;
    
    curve = bezier(pts,50,0);                           % Create Bezier curve
    
return;