% NESTC: This function calculates the nestedness metric (Nc) and its standardization (C)
%   proposed by Wright and Reeves (1992).  The metric "counts the number of times that a species'
%   presence at a site correctly predicts its presence at richer sites, and sums these counts across
%   species and sites."
%
%   usage: [Nc,C] = nestc(X)
%
%   X = a binary data matrix (rows = sites & columns = species)
%   ------------------------------------------------------------------------------------------------
%   Nc = the nestedness metric
%   C = the standardized nestedness matrix (independent of matrix size)
%
%   Chris L. Higgins 3/17/2004
%
%   Wright,D.H. and J.H. Reeves.  1992.  On the meaning and measuremment of nestedness of species
%       assemblages.  Oecologia, 92: 416-428.
%

function [N0,N1,Nc,C,PN0,PN1,PNc,PC] = nestc(X,H0,iter)
    
    if(nargin < 2)
        H0 = [];
    end;
    if(nargin < 3)
        iter = [];
    end;
    
    if(isempty(H0))
        H0 = 0;
    end;
    if(isempty(iter))
        iter = 0;
    end;

    packed = nestsort(X);
    %packed = X;
    
    [N0,N1,Nc,C,z,P] = nestcf(packed);

    if(H0 == 0)
        for i = 1:iter
            null = random0(packed);
            [N0_null(i),N1_null(i),Nc_null(i),C_null(i)] = nestcf(null);
        end;
    end;
    
    if(H0 == 1)
        for i = 1:iter
            null = random1(packed);
            [N0_null(i),N1_null(i),Nc_null(i),C_null(i)] = nestcf(null);
        end;
    end;
    
    if(iter == 0)
        PN0 = [];
        PN1 = [];
        PNc = [];
        PC = [];
    else;
        PN0 = pvalue(N0,N0_null,[]);
        PN1 = pvalue(N1,N1_null,[]);
        PNc = pvalue(Nc,Nc_null,[]);
        PC = pvalue(C,C_null,[]);
    end;
    
return;