% NESTCALC: Emulated the Nestedness Calculator of Atmar and Patterson.  That is, it calculates
%   the nestedness "temperature," percent fill, number of holes, number of outliers, and
%   statistical significance of the temperature base on randomization for a presence/absence
%   data set. The above metrics are based on deviations from the extinction threshold determined
%   by a Bezier curve, which is fit to the data matrix based on the fill.
%
%   Usage: [Temp,fill,p_Temp,Temp_null,m,SD,richness,occurence] = nestcalc(data,allblack,iter,con,model,dis,doplot)
%
%   data = [r x c] data matrix with rows representing "islands" and columns representing "species"
%   allblack = optional, specifices whether or not to remove redundant all-black rows and columns
%       0 = remove redundant all-black rows or columns {default}
%       1 = do not remove redundant all-black rows or columns 
%   iter = optional, specifies the number of iterations desired for significance testing
%   con = optional, specifies which type of null model to use in dewtermining significance
%       1 = neither row or column constraints {default} (corresponds to Atmar and Patterson)
%       2 = 
%       3 = 
%       4 = based on row and column probabilities
%   model = optional, specifies whether to randomly assign species or individuals
%       1 = species {default}
%       2 = individuals
%   doplot = optional, specifies whether or not to plot the packed matrix and extinction threshold
%   -----------------------------------------------------------------------------------------------------------------------
%   Temp = the nested temperature of a matrix
%   fill = the percentage of cells that contain "presences"
%   richness = a vector of species richness values corresponding to the packed matrix
%   occurence = a vector of species occurences corresponding to a packed matri
%   holes = the number of unexpected absences above the extinction line
%   outliers = the number of unexpected presences below the extinction line
%   p_Temp = the p-value for temperature
%
%   Chris L. Higgins 2/1/2003
%
%   Atmar,W. and B.D. Patterson. 1993. The measure of order and disorder in the distribution of species in fragmented habitat.
%       Oecologia 96:373-382.
%

function [Temp,fill,p_Temp,Temp_null,m,SD,richness,occurence] = nestcalc(data,iter,doplot)
    
    b = isbinary(data);
    if(b == 0)
        data = makebinary(data);
        warning('NESTCALC: The data martix was converted to a presence/absence matrix!');
    end;
    
    if(nargin < 2)
        iter = [];
    end;
    if(nargin < 3)
        doplot = [];
    end;
    if(isempty(iter))
        iter = 0;
    end;
    if(isempty(doplot))
        doplot = 0;
    end;

    [packed,richness,occurence] = maxpack(data);            % Produces a new matrix that is maximally packed
    [r,c] = size(packed);
    
    [fill,curve] = nestbez(packed);                         % Find the Bezier curve corresponding to the fill of the data matrix

    Temp = nesttemp(packed,curve);  
  
    if(iter > 0)                                            % If specified, calculate the level of significance
        Temp_null = zeros(1,iter);                      
        for i = 1:iter
           null = nestrand(r,c,sum(richness));
           packed_null = maxpack(null);
           Temp_null(i) = nesttemp(packed_null,curve);
        end;
        m = mean(Temp_null);                                % Calculate descriptive statistics of the null temperature distribtion
        SD = std(Temp_null);
        p_Temp = normcdf(Temp,m,SD);                        % Determine significance assuming a normal distribution
    end;
    
    if(doplot ~= 0)                                         % If specified, produce a figure
        [islands,species] = size(packed);
        [r,c] = find(packed >= 1);
        r = (islands+1) - r;
        figure;
        plot(c-.5,r-.5,'*');
        axis equal;
        axis([0 species 0 islands]);
        hold on;
        set(0,'DefaultLineLineWidth',1.5); 
        plot(curve(:,1),curve(:,2),'r');
        set(0,'DefaultLineLineWidth','default'); 
        putx('Species');
        puty('Islands');
        set(gcf,'DefaultTextColor','blue')
        p = sprintf(' %2.2f',Temp);
        q = sprintf(' %2.1f',fill);
        puttitle(['Temp:' p '    Fill:' q],18)
        puttick([1:species],[1:islands]);
        set(gca,'XTickLabel','')
        set(gca,'YTickLabel','')
        grid on;
    end;
    
return;