% NESTEDMAXPACK: This function, when given a presence-absence matrix, rearranges the rows and columns  
%   according to the code provided by Wirt Atmar. Thus, this program is completely analagous to the
%   Nestedness Temperature Calculator - it produces identical maximally packed matrices!!!!!
%
%   Usage: [packed,richness,occurence] = maxpack(data,allblack)
%
%   data = a [r x c] presence-absence data matrix with the rows corresponding to sites and columns corresponding to species
%   allblack = optional, specifices whether or not to remove redundant all-black rows and columns
%       0 = remove redundant all-black rows or columns {default}
%       1 = do not remove redundant all-black rows or columns 
%   -----------------------------------------------------------------------------------------------------------------------
%   packed = a presence-absence data matrix that is "maximally packed"
%   richness = a vector of species richness values corresponding to the packed matrix
%   occurence = a vector of species occurences corresponding to a packed matrix
%
%   Chris Higgins 3/7/2003
%
%   Atmar,W. and B.D. Patterson. 1993. The measure of order and disorder in the distribution of species in fragmented habitat.
%       Oecologia 96:373-382.
%

% 9/19/2003 Revised algorithm to allow for the option of not removing redundant all-black rows and columns

function [packed,richness,occurence] = nestedmaxpack(data,allblack)

    if(nargin < 2)
        allblack = [];
    end;
    
    if(isempty(allblack))
        allblack = 0;
    end;

    [islands,species] = size(data);                 % Determine the total number of islands and species
    
    if(species > islands)
        Rpacked = data;
        for fold = 1:8
            Cpacked = nestedsortcols(Rpacked);
            Rpacked = nestedsortrows(Cpacked);
        end;
        packed = Rpacked;
    else;
        Cpacked = data;
        for fold = 1:8
            Rpacked = nestedsortrows(Cpacked);
            Cpacked = nestedsortcols(Rpacked);
        end;
        packed = Cpacked;
    end;

    Binary = makebinary(packed);    %%%%%%%%%%%%%%%%%%%% ADDED TO CREATE SAME WITH ABUNDANCE DATA %%
    
    richness = rowsum(Binary);                      % Calculate the species richness for all sites
    occurence = sum(Binary);                        % Calculate the occurences for all site
    
    non0rows = find(richness ~= 0);                 % Remove any rows that contain all zeros
    packed = packed(non0rows,:);
    
    non0cols = find(occurence ~= 0);                % Remove any columns that contain all zeros
    packed = packed(:,non0cols);

    if(allblack == 0)
        [r,c] = size(packed);   
        all1rows = find(richness == c);             % Remove any redundant all black rows
        lenR = length(all1rows);
        if(lenR > 1)
            packed = packed([lenR:r],:);
        end;
        all1cols = find(occurence == r);            % Remove any redundant all black columns
        lenC = length(all1cols);
        if(lenC > 1)
            packed = packed(:,[lenC:c]);
        end;
    end;

    richness = rowsum(packed);                      % Calculate the species richness for all sites
    occurence = sum(packed);                        % Calculate the occurences for all site
    
return;