% NESTEDNESS: This function calculates the nestedness metric (Nc) and its standardization (C)
%   proposed by Wright and Reeves (1992).  The metric "counts the number of times that a species'
%   presence at a site correctly predicts its presence at richer sites, and sums these counts across
%   species and sites."
%
%   usage: [N0x,PN0x,N1x,PN1x,Ncx,PNcx] = nestedness(X,H0,iter)
%
%   X = a binary data matrix (rows = sites & columns = species)
%   ------------------------------------------------------------------------------------------------
%   Nc = the nestedness metric
%   C = the standardized nestedness matrix (independent of matrix size)
%
%   Chris L. Higgins 3/17/2004
%
%   Wright,D.H. and J.H. Reeves.  1992.  On the meaning and measuremment of nestedness of species
%       assemblages.  Oecologia, 92: 416-428.
%

function [N0x,PN0x,N1x,PN1x,Ncx,PNcx] = nestedness(X,model,iter,meth)
    
    b = isbinary(X);
    if(b == 0)
        X = makebinary(X);
    end;

    if(nargin < 2)
        model = [];
    end;
    if(nargin < 3)
        iter = [];
    end;
    if(nargin < 4)
        meth = [];
    end;
    
    if(isempty(model))
        model = 1;
    end;
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(meth))
        meth = 0;
    end;

    packed = nestsort(X);
    
    [N0,N1,Nc] = nestednessf(packed);

    for i = 1:iter
        null = randomnull(X,model)
        packed_null = nestsort(null);
        [N0_null(i,:),N1_null(i,:),Nc_null(i,:)] = nestednessf(packed_null,H0);
    end;
    
    if(meth == 0)
        MN0 = mean(N0_null(:,1));
        SN0 = std(N0_null(:,1));
        MPN0 = mean(N0_null(:,2));
        SPN0 = std(N0_null(:,2));
        MN1 = mean(N1_null(:,1));
        SN1 = std(N1_null(:,1));
        MPN1 = mean(N1_null(:,2));
        SPN1 = std(N1_null(:,2));
        MNc = mean(Nc_null(:,1));
        SNc = std(Nc_null(:,1));
        MPNc = mean(Nc_null(:,2));
        SPNc = std(Nc_null(:,2));
    
        zN0 = (N0(1) + 0.5 - MN0) / SN0;
        zPN0 = (N0(2) + 0.5 - MPN0) / SPN0;
        zN1 = (N1(1) + 0.5 - MN1) / SN1;
        zPN1 = (N1(2) + 0.5 - MPN1) / SPN1;
        zNc = (Nc(1) - 0.5 - MNc) / SNc;
        zPNc = (Nc(2) + 0.5 - MPNc) / SPNc;
    
        [H,PvN0] = ZTEST(zN0,0,1,.05,-1);
        [H,PvPN0] = ZTEST(zPN0,0,1,.05,1);
        [H,PvN1] = ZTEST(zN1,0,1,.05,-1);
        [H,PvPN1] = ZTEST(zPN1,0,1,.05,1);
        [H,PvNc] = ZTEST(zNc,0,1,.05,1);
        [H,PvPNc] = ZTEST(zPNc,0,1,.05,1);
    else;
        PvN0 = pvalue(N0(1),N0_null(:,1),1);
        PvPN0 = pvalue(N0(2),N0_null(:,2),3);
        PvN1 = pvalue(N1(1),N1_null(:,1),1);
        PvPN1 = pvalue(N1(2),N1_null(:,2),3);
        PvNc = pvalue(Nc(1),Nc_null(:,1),3);
        PvPNc = pvalue(Nc(2),Nc_null(:,2),3);
    end;    
    
    N0x = [N0(1) PvN0];
    PN0x = [N0(2) PvPN0];
    N1x = [N1(1) PvN1];
    PN1x = [N1(2) PvPN1];
    Ncx = [Nc(1) PvNc];
    PNcx = [Nc(2) PvPNc];

    
return;