% NESTEDNESSIND: Calculates the nestedness "temperature," percent fill, number of holes, number of
%   outliers, and statistical significance of the temperature base on randomization for a
%   a presence/absence data set. The above metrics are based on deviations from the extinction
%   threshold determined by a Bezier curve, which is fit to the data matrix based on the fill.
%
%   Usage: [Temp,fill,richness,occurence,holes,outliers,p-Temp] = nestedtemp(data,iter,moedl,doplot)
%
%   data = [r x c] data matrix with rows representing "islands" and columns representing "species"
%   iter = optional, specifies the number of iterations desired for significance testing
%   model = optional, specifies which type of null model to use in dewtermining significance
%       1 = neither row or column constraints {default} (corresponds to Atmar and Patterson)
%       4 = based on row and column probabilities
%   doplot = optional, specifies whether or not to plot the packed matrix and extinction threshold
%   -----------------------------------------------------------------------------------------------------------------------
%   Temp = the nested temperature of a matrix
%   fill = the percentage of cells that contain "presences"
%   richness = a vector of species richness values corresponding to the packed matrix
%   occurence = a vector of species occurences corresponding to a packed matri
%   holes = the number of unexpected absences above the extinction line
%   outliers = the number of unexpected presences below the extinction line
%   p_Temp = the p-value for temperature
%
%   Chris L. Higgins 2/1/2003
%
%   Atmar,W. and B.D. Patterson. 1993. The measure of order and disorder in the distribution of species in fragmented habitat.
%       Oecologia 96:373-382.
%

function [Temp,fill,p_Temp,Temp_null,richness,occurence] = nestednessind(data,iter,con,doplot)
    
    if(nargin < 2)
        iter = [];
    end;
    if(nargin < 3)
        con = [];
    end;
    if(nargin < 4)
        doplot = [];
    end;
    
    if(isempty(iter))
        iter = 1;
    end;
    if(isempty(con))
        con = 4;
    end;
    if(isempty(doplot))
        doplot = 0;
    end;
    
    [packed,richness,occurence] = maxpack(data);            % Produces a new matrix that is maximally packed
    
    [fill,curve] = nestedbezier(packed);                    % Find the Bezier curve corresponding to the fill of the data matrix
    
    Temp = nestedtemp(packed,curve);  
    
    if(iter > 1)                                            % If specified, calculate the level of significance
        Temp_null = zeros(1,iter);                      
        for i = 1:iter
            disp(i);
            null = nestednull(richness,occurence,con,2);
            packed_null = maxpack(null);              
            [fillp,curvep] = nestedbezier(packed_null);
            Temp_null(i) = nestedtemp(packed_null,curvep);
        end;
        m = mean(Temp_null);
        SD = std(Temp_null);
        p_Temp = normcdf(Temp,m,SD);
    end;
    
    if(doplot ~= 0)                                         % If specified, produce a figure
        [islands,species] = size(packed);
        [r,c] = find(packed == 1);
        r = (islands+1) - r;
        figure;
        plot(c-.5,r-.5,'*');
        axis equal;
        axis([0 species 0 islands]);
        hold on;
        set(0,'DefaultLineLineWidth',1.5); 
        plot(curve(:,1),curve(:,2),'r');
        set(0,'DefaultLineLineWidth','default'); 
        putx('Species');
        puty('Islands');
        set(gcf,'DefaultTextColor','blue')
        p = sprintf(' %2.2f',Temp);
        q = sprintf(' %2.1f',fill);
        puttitle(['Temp:' p '    Fill:' q],18)
        puttick([1:species],[1:islands]);
        set(gca,'XTickLabel','')
        set(gca,'YTickLabel','')
        grid on;
    end;
    
return;