% NESTEDSIG:  This function compares the nestedness temperatures between two abundance matrices
%   to determine whether they are significantly differrent from one another.
%
%   Usage: [X_temp,Y_temp,p_value,Diff] = nestedsig(X,Y,allblack,iter)
%
%   X = [r x c] data matrix of species abundances
%   Y = [r x c] data matrix of species abundances
%   allblack = specifies whether of not to include allblack rows and columns when packing the matrices
%   iter = specifies the number of iterations to conduct when determing statistcial significance
%   --------------------------------------------------------------------------------------------------
%   X_temp = nestedness temperature of the first data matrix
%   Y_temp = nestedness temperature of the second data matrix
%   p_value = the p value
%   Diff = the distribution of differences between the randomized temperatures
%
%   Chris L. Higgins 1/16/2004
%

function [X_temp,Y_temp,p_value,Diff] = nestedsig(X,Y,allblack,iter)

    X_temp = nestedness(X,allblack);            % Calulates the nestedness temperature for the first data matrix
    NX = sum(sum(X));                           % Determines the total number of individuals in the first data matrix
    Y_temp = nestedness(Y,allblack);            % Calulates the nestedness temperature for the second data matrix
    NY = sum(sum(Y));                           % Determines the total number of individuals in the second data matrix
    D = abs(X_temp - Y_temp);                   % Calculates the observed difference between nestedness temperatures
    
    %[rx,cx] = size(X)
    %[ry,cy] = size(Y)
    %if(rx ~= ry
    
    Pop = X+Y;                                  % Combines the two data matrices to produce an underlying population
    N = sum(sum(Pop));                          % Calculate the total number of individuals
    P = rowsum(Pop)*sum(Pop) / N^2;             % Create a probability matrix
      
    Diff = zeros(iter,1);                       % Allocate a vector to store randomized differences in temperatures
    for i = 1:iter
        i
        A_null = assignind(P,NX);               % Create the first null data matrix
        B_null = assignind(P,NY);               % Create the second null data matrix
        A_temp = nestedness(A_null,allblack);   % Calculate nestedness tempeature for first null matrix
        B_temp = nestedness(B_null,allblack);   % Calcualte nestedness temperatue for second null matrix
        Diff(i) = abs(A_temp - B_temp);         % Find the difference between tempoeratures and store it
    end;
    
    p_value = pvalue(D,Diff);                   % Calculate statistical significance based on a two-tailed test
    
return;