% NESTN0:  This function calculates the number of holes (unexpected absences), its standardized version, and the 
%   corresponding significance values.
%
%   Usage: [N0,PN0,PvNO,zN0] = nestn0(X,ord,model,iter,ax)
%
%       X = [sites x species] binary data matrix
%       ord = {optional}, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals {default}
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature) {default}
%           2 = Species richness per site is equiprobable and species occurrence is fixed 
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0)
%           4 = Species richness per site is proportional & species occurrence is fixed
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1)
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       iter = optional, specifies how many iterations to run (deafult = 0}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       --------------------------------------------------------------------------------------------------------------
%       N0 = the number of holes (i.e., unexpected absences)
%       PN0 = standardized version of N0 in percent nestedness (ranges from 0 to 100)
%       PvN0 = significance of N0 based on z distribution
%       zN0 = standardized z-score
%
%   Chris L. Higgins 3/17/2004
%
%

function [N0,PN0,PvN0,zN0] = nestn0(X,ord,model,iter,ax)

    if(nargin < 2)                                      % Allocate input arguments
        ord = [];
    end;
    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 3;
    end;
    if(isempty(model))
        model = 3;
    end;
    if(isempty(iter))
        iter = 0;
    end;
    if(isempty(ax))
        ax = 1;
    end;
    
    X = makebinary(X);                                  % Covert to binary matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            Y = nestsort(X);                            % Ordinate based on species richness and occurence totals 
    end;

    [K,S] = size(Y);

    N0 = 0;
    for i = 1:S
        ind0 = find(Y(:,i) == 0);
        ind1 = find(Y(:,i) == 1);
        len1 = length(ind1);
        if(~isempty(ind0) & ~isempty(ind1))
            if(ind0(1) < ind1(len1))
                A = find(ind0 < ind1(len1));
                N0 = N0 + length(A);
            end;
        end;
    end;
    
    for j = 1:iter
        null = randomnull(X,model);
        
        switch(ord)                                         % Allow various kinds of ordination of data matrix
            case 0
                EY = null;                                  % Do not ordinate the data matrix
            case 1
                EX = allzeros(null);                        % Remove any rows or columns that contain all zeros
                EY = ordinate(EX,ax);                       % Ordinate the data matrix by reciprical averaging
            case 2
                EY = maxpack(null);                         % Ordinate the data matrix by maximumally packing it (nested calculator)
            case 3
                EY = nestsort(null);                        % Ordinate based on species richness and occurence totals 
        end;
        
        [r,c] = size(EY);
        
        EN0 = 0;
        for i = 1:c
            ind0 = find(EY(:,i) == 0);
            ind1 = find(EY(:,i) == 1);
            len1 = length(ind1);
            if(~isempty(ind0) & ~isempty(ind1))
                if(ind0(1) < ind1(len1))
                    A = find(ind0 < ind1(len1));
                    EN0 = EN0 + length(A);
                end;
            end;
        end;
        N0_null(j) = EN0;
    end;

    if(iter > 0)
        MN0 = mean(N0_null);
        SN0 = std(N0_null);
        
        zN0 = (N0 + 0.5 - MN0) / SN0;
        [H,PvN0] = ztest(zN0,0,1,.05,-1);
        
        % Calculates the Percent of holes (PN0)- standardization method
        N0Perf = 0;
        PN0 = 100 * ((N0 - MN0) / (N0Perf - MN0));
    end;
    
return;