% NESTNC:  This function calculates the Nc nestedness measure, its standardized version, and the corresponding
%   signifiance values based on Wright and Reeves 1992.
%
%   Usage: [Nc,PNc,PvNc,zNc,PvC,NcExp,NcStd,Nc_null] = nestnc(X,ord,model,iter,ax)
%
%       X = [sites x species] binary data matrix
%       ord = {optional}, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals {default}
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature) {default}
%           2 = Species richness per site is equiprobable and species occurrence is fixed
%                   DOESNT ALLOW FOR ANY VARIATION IN RANDOMIZED MATRICES
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0)
%           4 = Species richness per site is proportional & species occurrence is fixed
%                   DOESNT ALLOW FOR ANY VARIATION IN RANDOMIZED MATRICES
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1)
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%                   DOESNT ALLOW FOR ANY VARIATION IN RANDOMIZED MATRICES
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       iter = optional, specifies how many iterations to run (deafult = 0}
%           if iter = 0, calcualtes the analytical solution
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       --------------------------------------------------------------------------------------------------------------
%       Nc = the nestedness metric
%       PNc = standardized version of Nc in percent nestedness (ranges from 0 to 100)
%       PvNc = significance of Nc based on z distribution
%       zNc = standardized z-score
%       PvC = significance of Nc based on Cochran's Q
%       NcMax = maximum value of Nc
%       NcExp = the expected value of Nc
%       NcStd = the standard deviation of Nc
%
%   Chris L. Higgins 3/17/2004
%
%   Wright,D.H. and J.H. Reeves.  1992.  On the meaning and measuremment of nestedness of species
%       assemblages.  Oecologia, 92: 416-428.
%

function [Nc,PNc,PvNc,zNc,PvC,NcMax,NcExp,NcStd] = nestnc(X,ord,model,iter,ax)

    if(nargin < 2)                                      % Allocate inmput arguments 
        ord = [];
    end;
    if(nargin < 3)
        model = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 3;
    end;
    if(isempty(model))
        model = 3;
    end;
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(ax))
        ax = 1;
    end;

    X = makebinary(X);                                  % Covert to binary matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            Y = nestsort(X);                            % Ordinate based on species richness and occurence totals    
    end;
    
    [K,S] = size(Y);                                    % Determine size of data matrix after ordination
    Richness = rowsum(Y);                               % Calculate species richness
    Incidence = sum(Y);                                 % Calculate species occurences

% Calculates the Nc metric
    Nc = 0;
    for i = 1:S
        Nc = Nc + (Incidence(i)*(Incidence(i)-1));
    end;
    Nc = 0.5 * Nc;
 
% Calculates the maximum Nc metric 
    NcMax = 0;
    for i = 1:K
        NcMax = NcMax + (i-1)*Richness(i);
    end; 

    if(iter == 0)

% Calculates the expected Nc metric -- ANALYTICAL SOLUTION
        NcExp = 0;
        for i = 1:K
            NcExp = NcExp + (Richness(i)^2);
        end;
        NcExp =  (sum(Richness)^2 - NcExp) / (2*S);
        
% Calculate the variance of Nc -- ANALYTICAL SOLUTION
        NcVar = 0;
        for i = 1:K-1
            for m = i+1:K
                NcVar = NcVar + (Richness(i)*Richness(m)*(S-Richness(i))*(S-Richness(m)));
            end;
        end;
        NcVar = NcVar / (S^2 * (S-1));
        NcStd = sqrt(NcVar);
    else;
        for k = 1:iter
            null = randomnull(X,model);

            switch(ord)                                 % Allow various kinds of ordination of data matrix
                case 0  
                    EY = null;                          % Do not ordinate the data matrix
                case 1
                    EX = allzeros(null);                % Remove any rows or columns that contain all zeros
                    EY = ordinate(EX,ax);               % Ordinate the data matrix by reciprical averaging
                case 2
                    EY = maxpack(null);                 % Ordinate the data matrix by maximumally packing it (nested calculator)
                case 3
                    EY = nestsort(null);                % Ordinate based on species richness and occurence totals    
            end;
            
            [r,c] = size(EY);
            Erichness = rowsum(EY);
            Eincidence = sum(EY);
            
            ENc = 0;
            for i = 1:c
                ENc = ENc + (Eincidence(i)*(Eincidence(i)-1));
            end;
            ENc = 0.5 * ENc;
          
            Nc_null(k) = ENc;
        end;
% Calculates the expected Nc metric  -- RANDOMIZED SOLUTION
        NcExp = mean(Nc_null);
        NcStd = std(Nc_null);
    end;

% Calculates the standardized Nc Metric (C)  
    C = (Nc-NcExp)/(NcMax-NcExp);
    
% Calculates the Percent of Nc
    PNc = C * 100;

% Calculate the z-statistic and its significance
    z = (Nc - 0.5 - NcExp) / NcStd;
    [H,PvNc,CI,zNc] = ztest(z,0,1,.05,1);
    
% Calculate Cochran's Q statistic and its significance
    A1 = S*sum(sum(X).^2);
    A2 = sum(sum(X)).^2;
    A3 = S*sum(sum(X));
    A4 = sum(rowsum(X).^2);
    Q = ((S-1)*(A1-A2))/(A3-A4);
    df = S-1;
    PvC = 1-chi2cdf(Q,S-1);
    
return;