% NESTTEMP: Calculates the nestedness "temperature," number of holes, and number of outliers
%   of a presence/absence data set. The above metrics are based on deviations from the extinction
%   threshold determined by a Bezier curve, which is fit to the data matrix based on the fill.
%
%   Usage: [Temp,holes,outliers] = nestedtemp(data,curve)
%
%   data = [r x c] data matrix with rows representing "islands" and columns representing "species"
%   curve = [n x 2] matrix of point corrdinates defining the Bezier curve used as the extinction threshold
%               column 1 = x coordinates
%               column 2 = y coordinates
%   -----------------------------------------------------------------------------------------------------------------------
%   Temp = the nested temperature of a matrix
%   holes = the number of unexpected absences above the extinction line
%   outliers = the number of unexpected presences below the extinction line
%
%   Chris L. Higgins 2/1/2003
%
%   Atmar,W. and B.D. Patterson. 1993. The measure of order and disorder in the distribution of species in fragmented habitat.
%       Oecologia 96:373-382.
%

function [Temp,holes,outliers] = nesttemp(data,curve)
    
    [rd,cd] = size(data);                                   % Determine the size of the data matrix
    r = ceil(curve(51,2));                                  % Determine the size of the matrix in curve
    c = ceil(curve(51,1));
    Zr = zeros((r-rd),cd);
    Zc = zeros(r,(c-cd));
    data = [data;Zr];
    data = [data Zc];

    polypts = [0 0;curve;c r;0 r;0 0];                      % Add points to create the upper-left polygon
    
    LineX = ptsline([0 0],[c 0]);                           % Calculate line for bottom axis
    LineY = ptsline([0 0],[0 r]);                           % Calculate line for left axis
    LineR = ptsline([0 r],[c r]);                           % Calculate line for top margin
    LineC = ptsline([c 0],[c r]);                           % Calculate line for right Margin
    
    LineUM = ptsline([0 r],[c 0]);                          % Calculate line for skew diagonal
    thetaUM = anglepts([c 0],[0 r],[0 0]);                  % Calculate angle between horizontal axis and skew diagonal
    
    Unexpected = [];                                        % Allocate "unexpected" vector
    holes = 0;                                              % Set the number of unexpected absences to zero
    outliers = 0;                                           % Set the number of unexpected presences to zero
    
    for a = 1:c
        for b = 1:r
            x = a - 0.5;                                    % Get X coordinate for center of cell
            y = (r + 0.5) - b;                              % Get Y coordinate for center of cell
            
% This section determines is the point is in the upper polygon formed by the Bezier curve
            indX = find(curve(:,1) >= x);       
            indX = indX(1);
            isin = 0;
            if(y >= curve(indX,2))
                isin = 1;
            end;
            if(y < curve(indX,2) & y >= curve(indX-1,2))
                ppts = [curve(indX-1,:);curve(indX,:);curve(indX-1,1) curve(indX,2)];
                isin = isinpoly([x y],ppts);
            end;
            y2 = ((-LineUM(1)*x)/LineUM(2)) - (LineUM(3)/LineUM(2));
            if(y <= y2)
                isin2 = 1;
            else;
                isin2 = 0;
            end;
% End of section

            if(isin == 1 & data(b,a) == 0)                  % Determine if point is an unexpected absence
                holes = holes + 1;                          % Add to number of unexpected absences
                indHole = find(curve(:,1) >= x);
                ind = indHole(1);
                ind2 = find(curve(:,2) >= y);
                stop = 0;                                   % Create stopping criteria for while loop
                while(stop == 0)                            % Find which line segment of curve to use as ET
                    if(x ~= curve(ind,1) & y ~= curve(ind,2))
                        theta = anglepts([x y],[curve(ind,1) curve(ind,2)],[c y]);
                        if(theta > thetaUM)
                            ind = ind + 1;
                        end;
                        if(theta <= thetaUM)
                            LineET = ptsline([curve(ind,1) curve(ind,2)],[curve(ind-1,1) curve(ind-1,2)]); % Find line for extinction threshold
                            LineIJ = paraline(LineUM,[x y]);        % Find line through unexpected absence parallel to skew diagonal
                            ptET = ptinter(LineET,LineIJ);          % Find point of intersection between extinction threshold and parallele line
                            dIJ = ptsdist([x y],ptET);              % Calculate distance from unexpected absenece to extinction threshold
                            if(isin2 == 1)
                                ptH = ptinter(LineIJ,LineX);        % Determine point of intersection between parallel line and X axis
                                ptV = ptinter(LineIJ,LineY);        % Determine point of intersection between parallel line and Y axis
                            else;
                                ptH = ptinter(LineIJ,LineR);
                                ptV = ptinter(LineIJ,LineC);
                            end;
                            DIJ = ptsdist(ptH,ptV);              % Calculate length of full line passing through unexpected absence
                            uIJ = (dIJ/DIJ)^2;                   % Calculate the degree of local unexpectedness
                            Unexpected = [Unexpected; uIJ];      % Creat vector of all local unexpectedness
                            stop = 1;
                        end;
                        if(ind >= ind2(1))
                            stop = 1;
                        end;
                    else;
                        stop = 1;
                    end;
                end;
            end;
            if(isin == 0 & data(b,a) > 0)           % == 1           % Determine if point is an unexpected presence
                outliers = outliers + 1;                        % Add to number of unexpected presences
                indOut = find(curve(:,2) > y);
                ind = indOut(1);
                ind3 = find(curve(:,2) <= x);
                stop = 0;
                while(stop == 0)                                % Find which line segment of curve to use as ET
                    theta = anglepts([x y],[curve(ind,1) curve(ind,2)],[0 y]);
                    if(theta < thetaUM)
                        ind = ind + 1;
                    end;
                    if(theta >= thetaUM)
                        LineET = ptsline([curve(ind,1) curve(ind,2)],[curve(ind-1,1) curve(ind-1,2)]); % Find line for extinction threshold
                        LineIJ = paraline(LineUM,[x y]);        % Find line through unexpected absence parallel to skew diagonal
                        ptET = ptinter(LineET,LineIJ);          % Find point of intersection between extinction threshold and parallele line
                        dIJ = ptsdist([x y],ptET);              % Calculate distance from unexpected absenece to extinction threshold
                        if(isin2 == 1)
                            ptH = ptinter(LineIJ,LineX);        % Determine point of intersection between parallel line and X axis
                            ptV = ptinter(LineIJ,LineY);        % Determine point of intersection between parallel line and Y axis
                        else;
                            ptH = ptinter(LineIJ,LineR);
                            ptV = ptinter(LineIJ,LineC);
                        end;
                        DIJ = ptsdist(ptH,ptV);                 % Calculate length of full line passing through unexpected absence
                        uIJ = (dIJ/DIJ)^2;                      % Calculate the degree of local unexpectedness
                        Unexpected = [Unexpected; uIJ];         % Creat vector of all local unexpectedness
                        if(ind >= ind3(1))
                            stop = 1;
                        end;
                        stop = 1;
                    end;
                end;
            end;
        end;
    end;

    Umax = 0.04145;                                             % Provide a maximum for total unexpectedness
    U = (sum(Unexpected)) / (r * c);                            % Calculate total unexpectedness
    Temp = (U / Umax) * 100;                                    % Calculate temperature 

return;