% phylodiv: Calculates the integrated multiple dimensions of biodiversity metric of Scheiner 2012, including
%   a single metric that combines taxonomic, functional, and phylogenetic biodiversity as well as
%   individual components for each dimension.
%
%   Usage: [DA, DP, DAP] = phylodiv(edges, edgematrix, community,q)
%
%       edges =         vector of edge lengths from a phylogenetic tree
%       edgematrix =    an edge x species matrix to associated particular
%                       edges with particular species
%       community =     vector of species abundances
%       q =             exponent that determines how relative proportions are weighed
%       --------------------------------------------------------------------------------------------------------------------------------
%       DA = abundance diversity of Scheiner 2012
%       DP = phylogenetic divergence of Scheiner 2012
%
%   Steven J. Presley 12/18/12
%

function [DA, DP, DAP] = phylodiv(edges,edgematrix,community,q)

%error checking
    [E,S] = size(edgematrix);               % determines # of edges(E) and species (S) in community

    if(nargin < 2)
        disp('ERROR: too few input arguments')
    end;
    
    if(nargin < 3)
        community = ones(S,1);
    end;
   
    if(nargin < 4)
        q = 0;
    end;

    
    s = length(community);                   % determines number of species in community
    r = length(edges);                      % determines number of edges in tree
       
    if(r ~= E)
        disp('ERROR: number of edges do not match in edges and edgematrix')
    end;
    
    if(s ~= S)
        disp('ERROR: number of species do not match in community and edgematrix')
    end;

    edges=edges';
    R = S;                                  % sets richness(R) to equal the number of species
    N = sum(S);                             % calculates total number of individuals
    
    % Calculated phylogenetic diversity of Scheiner
    
    edgelineage = sum(edgematrix');         % determines the number of lineages associated with each edge
    
    L = sum(edges);                         % determines the total lineage divergence
    
    proportionaledges = [];
    
    %divides the divergense of each edge equally among the lineages associated with it
    for i = 1:r
        proportionaledges(1,i)= edges(1,i)/edgelineage(1,i);
    end;
    
    lineagedivergence = zeros(E,S);
    
    for i = 1:r
        for j = 1:S
            lineagedivergence(i,j) = edgematrix(i,j)*proportionaledges(1,i);
        end;
    end;
    
    speciesLD = sum(lineagedivergence);     % sum of divergences for each species (Li of Scheiner)
    speciesPLD = speciesLD/L;               % calculates the proportional divergence of each lineage
    
    DPQ = [];
    
    if (q == 1)
        
        for i=1:S      
            DPQ(1,i) = speciesPLD(1,i)*log(speciesPLD(1,i)); % First part of calculation for DP when q = 1 (limit function)
        end;
        
        DP = exp(-sum(DPQ));                    % Calcuation of phylogenetic diversity (DP) for q = 1
        
    else;
        
        for i=1:S
            DPQ(1,i) = speciesPLD(1,i)^q;       % Raises proportional divergenge values from each lineage to the power of q
        end;
    
        DP = (sum(DPQ))^(1/(1-q));              % Calculates phylogenetic diversity (DP)
    end;
    
    
    % Calculation of abundance diversity of Scheiner
    
    N = sum(community);                     % Calculatese total number of individuals in a community
    propabund = community/N;                % Calculates proportional abundances
    
    speciesDAQ = [];
    
    if (q == 1)
        for i=1:S
            speciesDAQ(1,i) = propabund(1,i)*log(propabund(1,i));
        end;
        
        DA = exp(-sum(speciesDAQ));             % Calculates abundance diversity (DA) for q = 1
        
    else;
    
        for i=1:S
            speciesDAQ(1,i) = propabund(1,i)^q; % Raises proportional abundances to the power of q
        end;
        
        DA = (sum(speciesDAQ))^(1/(1-q));       % Calculates abundance diversity (DA) of Scheiner 2012
    end;
    
 
    %Calculation of D(AP) of Scheiner
    
    DAPdenom = [];
    
    for i=1:S
        DAPbase(1,i) = community(1,i)*speciesLD(1,i);  %calculates elements (niLi) for D(AP) of Scheiner
    end;
    
    DAPdenom = sum(DAPbase);                           %sums elements to get denominator for D(AP) of scheiner
    
    DAPbase = DAPbase/DAPdenom;
    
    for i=1:S
        DAPbase(1,i)=DAPbase(1,i)^q;
    end;
    
    
    
    if (q==1)
        for i=1:S
           DAPbase(1,i) = DAPbase(1,i)*log(DAPbase(1,i));        %Calculates D(AP) of Scheiner for q=1
        end;
       
        DAP = exp(-sum(DAPbase));
        
    else;
        DAPbase = sum(DAPbase);
        DAP = DAPbase^(1/(1-q));                %Calculates D(AP) of Scheiner for q <> 1
    end;
    
    
    
    
    
    
return;


