%POWERFRACT: This function provides the expected species abundances based on Tokeshi's power
%   fraction model (1996). In this model, the probability that a niche will be targeted by an invading
%   species is a function of its size when that size has been raised to the power K (0 - 1).   
%
%   Uasge:  Abundance = powerfract(S,N,K,rel)
%
%   S = the total number of species for each community
%   N = the total number of individuals in each community
%   K = the power to which niches are targeted
%           0 = random fraction; 1 = brokenstick
%   rel = optional, specifies whether or not to make the abundances relative {default = 0}
%   -------------------------------------------------------------------------------------------
%   Abundance = a [S x 1] vector of expected species abundances 
%
%   Chris L. Higgins 3/5/2003
%
%   Tokeshi, M. 1996. Power fraction: a new explanation for species abundance patterns in species-rich
%       assemblages.  Oikos, 75: 543-550.
%

function Abundance = powerfract(K,S,N,rel)

    if(nargin < 4)
        rel = [];
    end;
    if(isempty(rel))
        rel = 0;
    end;    

    Abundance = zeros(1,S);
    
    x = [0 1];
        
    for i = 1:S-1
        in = rand;
        if(K ~= 0)
            y = x.^K;
            z = y/y(i+1);
        else
            y = makegrps((1/i),i)';
            z = [0 cumsum(y)];
        end;
        ind = find(z >= in);
        D = x(ind(1)) - x(ind(1)-1);
        r = rand * D;
        s = x(ind(1)-1) + r;
        x = [x s];
        x = sort(x);
    end;
    
    for j = 1:S
        Ab(j) = (x(j+1) - x(j)) * N;
    end;
    
    Abundance = abs(sort(-Ab));

    if(rel ~=0)
        Abundance = Abundance / N;
    end;
    
return;