% RANDIND:  This function is used in the randomization part of PARTITION.
% It generates null distributions by assuming that the observed numbers of
% individuals of each species are randomly placed among samples at the
% lowest level (i=1), and samples containing ranomly placed individuials
% are then grouped togethr into progressively larger samples at each
% higher level i = 2,...
%
%   Usage:  null = randind(X)
%
%   X = [sites x species] matrix of observed species abundances
%   ----------------------------------------------------------------------
%   null = [sites x species] matrix of randomized abundances
%
%   Chris L Higgins 10/16/2007
%
%   Crist, T.O., J.A. Veech, J.C. Gering, and K.S. Summerville.  2003.
%       Partitioning species diversity across landscapes and regions: a
%       hierarchical analysis of alpha, beta, and gamma diversity.  The
%       American Naturalist 162(6): 734-743.
%

% Also based on personal communication with Joseph Veech via email

function null = randind(X)

    [r,c] = size(X);                            % Determine size of original data matrix
    SampSize = rowsum(X);                       % Calculate sample size for each site
    SppAbu = sum(X);                            % Calculate species abundances
    N = sum(sum(X));                            % Calculate total number of individuals in data matrix

    null = zeros(r,c);                          % Allocate output argument
    
    G = makegrps(1:c,SppAbu);                   % Create vector of individuals for entire matrix
    NullInd = G(randperm(N));                   % Randomize vector of individuals

    Y = 0;
    for i = 1:r                                 % Cycle through each site
        Z = NullInd((Y+1):(Y+SampSize(i)));     % Pull out samples of individuals that match original sample sizes
        if(~isempty(Z))
            [u,f] = unique1(Z);                 % Determine which species are present in the sample and how many
            null(i,u) = f';                     % Place individuals in null matrix
            Y=Y+SampSize(i);                    % Increase index to keep track of already placed individuals
        end;
    end;
    
return;       