% RANDOMNULL:  This function produces various null models of species co-occurence based on Gotelli 2000.
%   Gotelli used [species x sites] matrices, but I choose to use [sites x species] matrices.
%
%   Usage: null = randomnull(X,model)
%
%       X = [sites x species] incidence matrix
%       model = optional, specifies which null model to use
%           1 = Species richness per site is equiprobable and species occurrence is equiprobable (RANDOM00 - Temperature) {default}
%           2 = Species richness per site is equiprobable and species occurrence is fixed 
%           3 = Species richness per site is fixed & species occurrence is equiprobable (RANDOM0)
%           4 = Species richness per site is proportional & species occurrence is fixed
%           5 = Species richness per site is fixed & species occurrence is proportional (RANDOM1)
%           6 = Species richness per site is proportional & species occurrence is equiprobable
%           7 = Species richness per site is equiprobable & species occurrence is proportional
%           8 = Species richness per site is proportional & species occurrence is proportional
%           9 = Species richness per site is fixed & species occurrence is fixed
%           10 = RANDNEST null model (Jonsson 2001) that relaxes contraints of percent fill and fixed row totals
%       -----------------------------------------------------------------------------------------------------------------------------
%       null = [sites x species] null matrix
%
%   Chris L. Higgins 8/25/2006
%
%   Gotelli, N.J. 2000.  Null model analysis of species co-occurence patterns. Ecology, 81(9): 2606-2621.
%   Jonsson,B.G.  2001.  A null model for randomization tests of nestedness in species assemblages.  Oecologia, 127: 309-313.
%

function null = randomnull(X,model)

    if(nargin < 2)                                              % Set default value for "model"
        model = [];
    end;
    if(isempty(model))
        model = 1;
    end;
    
    [r,c] = size(X);                                            % Determine size of input matrix
    N = sum(sum(X));                                            % Calculate total number of presences
    Richness = rowsum(X);                                       % Calcualte species richness for each site
    Incidence = sum(X);                                         % Calculate incidence for each species
    
    FixRichness = ones(r,1);                                    % Create vector for fixed richness
    EqRichness = ones(r,1) * (1/r);                             % Create vector for equiprobable richness
    PropRichness = rowsum(X) / N;                               % Create vector for proporational richness
    FixIncidence = ones(1,c);                                   % Create vector for fixed incidence
    EqIncidence = ones(1,c) * (1/c);                            % Create vector for equiprobable incidence
    if(r == 1)                                                  % Create vector for proportional richness
        PropIncidence = X / N;          
    else;
        PropIncidence = sum(X) / N;
    end;
    
    null = zeros(r,c);
    if(isbinary(X))
        switch(model)
            case 1
                P = EqRichness * EqIncidence;                       % Species richness per site is equiprobable and species occurrence is equiprobable
                null = assignsp(P,N,[]);                            % Allocate species to probability matrix
            case 2
                P = EqRichness * FixIncidence;                      % Species richness per site is equiprobable and species occurrence is fixed
                for i = 1:c
                    null(:,i) = assignsp(P(:,i),Incidence(i),[]);
                end;
            case 3
                P = FixRichness * EqIncidence;                      % Species richness per site is fixed & species occurrence is equiprobable
                for i = 1:r
                    null(i,:) = assignsp(P(i,:),Richness(i),[]);
                end;
            case 4
                P = PropRichness * FixIncidence;                    % Species richness per site is proportional & species occurrence is fixed
                for i = 1:c
                    null(:,i) = assignsp(P(:,i),Incidence(i),[]);
                end;
            case 5
                P = FixRichness * PropIncidence;                    % Species richness per site is fixed & species occurrence is proportional
                for i = 1:r
                    null(i,:) = assignsp(P(i,:),Richness(i),[]);
                end;
            case 6
                P = PropRichness * EqIncidence;                     % Species richness per site is proportional & species occurrence is equiprobable
                null = assignsp(P,N,[]);                            % Allocate species to probability matrix
            case 7
                P = EqRichness * PropIncidence;                     % Species richness per site is equiprobable & species occurrence is proportional
                null = assignsp(P,N,[]);                            % Allocate species to probability matrix
            case 8
                P = PropRichness * PropIncidence;                   % Species richness per site is proportional & species occurrence is proportional
                null = assignsp(P,N,[]);                            % Allocate species to probability matrix
            case 9                                                  
                null = Swap(X);                                     % Species richness per site is fixed & species occurrence is fixed
            case 10
                null = randnest(X);                                 % Null matrix based on RANDNEST (Jonsson 2001) that relaxes 2 contraints -- percent fill and fixed row totals
        end;
    else;
         switch(model)
            case 1
                P = EqRichness * EqIncidence;                           % Species richness per site is equiprobable and species occurrence is equiprobable
                null = assignind(P,N,[]);                               % Allocate species to probability matrix
            case 2
                P = EqRichness * FixIncidence;                          % Species richness per site is equiprobable and species occurrence is fixed
                for i = 1:c
                    null(:,i) = assignind(P(:,i),Incidence(i),[]);
                end;
            case 3
                P = FixRichness * EqIncidence;                          % Species richness per site is fixed & species occurrence is equiprobable
                for i = 1:r
                    null(i,:) = assignind(P(i,:),Richness(i),[]);
                end;
            case 4
                P = PropRichness * FixIncidence;                        % Species richness per site is proportional & species occurrence is fixed
                for i = 1:c
                    null(:,i) = assignind(P(:,i),Incidence(i),[]);
                end;
            case 5
                P = FixRichness * PropIncidence;                        % Species richness per site is fixed & species occurrence is proportional
                for i = 1:r
                    null(i,:) = assignind(P(i,:),Richness(i),[]);
                end;
            case 6
                P = PropRichness * EqIncidence;                         % Species richness per site is proportional & species occurrence is equiprobable
                null = assignind(P,N,[]);                               % Allocate species to probability matrix
            case 7
                P = EqRichness * PropIncidence;                         % Species richness per site is equiprobable & species occurrence is proportional
                null = assignind(P,N,[]);                               % Allocate species to probability matrix
            case 8
                P = PropRichness * PropIncidence;                       % Species richness per site is proportional & species occurrence is proportional
                null = assignind(P,N,[]);                               % Allocate species to probability matrix
            case 9                                                  
                SampSize = rowsum(X);                                   % Calculate sample size for each site
                SppAbu = sum(X);                                        % Calculate species abundances
                G = makegrps(1:c,SppAbu);                               % Create vector of individuals for entire matrix
                NullInd = G(randperm(N));                               % Randomize vector of individuals
                Y = 0;
                for i = 1:r                                             % Cycle through each site
                    Z = NullInd((Y+1):(Y+SampSize(i)));                 % Pull out samples of individuals that match original sample sizes
                    [u,f] = unique1(Z);                                 % Determine which species are present in the sample and how many
                    null(i,u) = f;                                      % Place individuals in null matrix
                    Y=Y+SampSize(i);                                    % Increase index to keep track of already placed individuals
                end;                                     	
        end;
    end;
return;