% RAREFY:   Performs rarefaction analysis based on bootstrapped samples.  Calculates various measure of diversity, including
%           species richness, Shannon diversity, Camargo evenness, Berger-Parker dominance, and rarity.
%
%   USAGE: [S,H,E,D,R] = rarefy(X,N,iter,sing)
%
%       X = [1 x species] vector of species abundances or [sites x species] matrix of species abundances
%       N = {optional}, specifies the number of individuals to stop at {default = sum(x) if vector or min(rowsum(X)) if matrix)
%       iter = {optional}, species the number of iterations to perform {default = 1000)
%       sing = {optional}, specfies whether to rarefy only one number (1) or all (0) {default = 0}
%       ----------------------------------------------------------------------------------------------------
%       S = species richness [mean lowerCI upperCI]
%       H = Shannon diversity [mean lowerCI upperCI]
%       E = Camargo evenness [mean lowerCI upperCI]
%       D = Berger-parker dominance [mean lowerCI upperCI]
%       R = rarity [mean lowerCI upperCI]
%
%   Chris L. Higgins 3/12/2005
%


function [S,H,E,D,R] = rarefy(X,N,iter,sing)

    V = isvector(X);                                                % Determine if input matrix is a vector or matrix
    [r,c] = size(X);                                                % Determine size of input matrix

    if(nargin < 2)
        N = [];
    end;
    if(nargin < 3)
        iter = [];
    end;
    if(nargin < 4)
        sing = [];
    end;
    
    if(isempty(N) & V == 1)
        N = sum(X);
    end;
    if(isempty(N) & V == 0)
        N = min(rowsum(X));
    end;
    if(isempty(iter))
        iter = 1000;
    end;
    if(isempty(sing))
        sing = 0;
    end;
    
    for k = 1:r
        if(sing == 0)
            for i = 1:N                                                 % Cycle through each level of individuals (1,2,3,4,...,N)
                for j = 1:iter                                          % Cycle through iterations to create distributions
                    Y = bootsamp2(X(k,:),N);                                 % Create bootstrapped sample of N observations from X distribution
                    Y = abs(sort(-Y));                                  % Sort species abundance distribution
                    [ES(j),EH(j),EE(j),ED(j),ER(j)] = diversity(Y);     % Calculate diversity measures
                end;
                S(k,1,i) = mean(ES);                                      % Calculate descriptive statistics
                S(k,[2:3],i) = bootci(ES')';
                H(k,1,i) = mean(EH);
                H(k,[2:3],i) = bootci(EH')';
                E(k,1,i) = mean(EE);
                E(k,[2:3],i) = bootci(EE')';
                D(k,1,i) = mean(ED);
                D(k,[2:3],i) = bootci(ED')';
                R(k,1,i) = mean(ER);
                R(k,[2:3],i) = bootci(ER')';
            end;
        else;
            for j = 1:iter                                          % Cycle through iterations to build distribution    
                Y = bootsamp2(X(k,:),N);                                 % Create bootstrapped sample of N observations from X distribution
                Y = abs(sort(-Y));                                  % Sort species abundance distribution
                [ES(j),EH(j),EE(j),ED(j),ER(j)] = diversity(Y);     % Calculate diversity values
            end;
            EE';
            S(k,1) = mean(ES);                                        % Calculate descriptive statistics
            S(k,[2:3]) = bootci(ES')';
            H(k,1) = mean(EH);
            H(k,[2:3]) = bootci(EH')';
            E(k,1) = mean(EE);
            E(k,[2:3]) = bootci(EE')';
            D(k,1) = mean(ED);
            D(k,[2:3]) = bootci(ED')';
            R(k,1) = mean(ER);
            R(k,[2:3]) = bootci(ER')';
        end;
    end;
    
 return;