% RARITY:  This functions estimates rarity (i.e., the number of species with abundance < N/S) for a given community.
%   Can be used to assess local rarity or global rarity.
%
%   Usage: R = rarity(X,TotSAD)
%
%   X = [sites x species] data matrix of species abundances
%   TotSAD = optional, [1 x species] if passed, this vector of species abundances is used to assess global rarity; if nothing is 
%       passed, then local rarity is analyzed
%   ----------------------------------------------------------------------------------------------------------------------------
%   R = [sites x 1] vector of rarity values
%
% Chris L. Higgins 7/28/2008
%

function R = rarity(X,TotSAD)

    if(nargin < 2)                                          % Allocate input arguments
        TotSAD = [];
    end;

    [r,c] = size(X);                                        % Determine the number of sites
    if(r > 1 & c == 1)                                      % Convert column vector to row vector
        X = X';
        [r,c] = size(X);
    end;
    
    X = allzeros(X,2);                                      % Remove any column (species) that contains all zeros
    TotSAD = allzeros(TotSAD,2);
    
    R = zeros(r,1);                                         % Allocate column vector for diversity indices   

% Estimate local rarity 
    if(isempty(TotSAD))
        for i = 1:r
            ind = find(X(i,:) > 0);                             % Find all non-zero entries
            if(isempty(ind))
                R(i) = 0;
            else
                N = sum(X(i,ind));
                S = length(ind);
                Ind = find(X(i,ind) < N/S);
                R(i) = length(Ind);                                      
            end;
        end;
    end;

% Estimate global rarity
    if(~isempty(TotSAD))
        TotS = length(find(TotSAD > 0));
        TotN = sum(TotSAD);
        TotInd = find(TotSAD < TotN/TotS);
        for i = 1:r
            ind = find(X(i,:) > 0);                             % Find all non-zero entries
            if(isempty(ind))
                R(i) = 0;
            else
                for j = 1:length(ind)
                    ind2 = find(TotInd == ind(j));
                    R(i) = R(i) + length(ind2);
                end;
            end;        
        end;
    end;
    
return;