% SAMPBIAS:   This function uses standard abundance distributions to create a universe of individuals...
%
%   Usage: ObsSAD = sampbias(S,N,replace,sampsize,elusive,ElusiveType,bias)
%
%   'model' = specifies the abundance model -- MUST BE IN SINGLE QUOTES
%       'brokenstick' = Broken-stick
%       'logseries' = Logseries
%       'uniform' = Uniform
%       'geometric' = Geometric series
%       'preemption' = Tokeshi's dominance premption model
%       'domindecay' = Tokeshi's dominance decay model
%       'randfract' = Tokeshi's random fraction model
%       'randassort' = Tokeshi's random assortment model
%   S = number of species in universe
%   N = number of individuals in universe
%   replace = flag for determining sampling with or without replacement
%   sampsize = number of individuals in random sample
%   samples = specifies the number of iterations to perform
%   elusive = percent of species that are subject to sampling bias (entered in decimal form (e.g., 0.05 for five percent))
%   ElusiveType = specifies whether elusive species are randomly assiged or only rare species
%   bias = percent of individuals of the elusive species that typically avoid capture (decimal form)
%   -------------------------------------------------------------------------------------------------------------------------
%   ObsSAD = [S x samples] observed species abundance distribution
%   misses = [1 x samples] the number of individuals of elusive species that were not captured in sample
%
%   Chris Higgins 06/05/2006
%

%  CODE FOR STEVE PRESLEY AND MIKE WILLIG

function [ObsSAD,misses] = sampbias(model,S,N,replace,sampsize,samples,elusive,ElusiveType,bias)

    Abundance = feval(model,S,N);                       % Create a known universe based certain SAD model
    RndSAD = round(Abundance);                          % Round the known SAD so no fraction of individuals occurs

    NumEluSpp = ceil(elusive*S);                        % Calculate the number of elusive species
    if(ElusiveType == 1)                                % Determine which species are elusive and which are nonelusive
        EluSpp = [S-NumEluSpp+1:S];                     % Only the rarest species are elusive
        NonEluSpp = [1:S-NumEluSpp];
    else;
        rS = randperm(S);                               % Randomly determine which species are elusive
        EluSpp = rS(1:NumEluSpp);
        NonEluSpp = rS(NumEluSpp+1:S);
    end;
    
    Grp = makegroups(1:S,RndSAD);                       % Create vector of individuals
    
    ObsSAD = zeros(S,samples);                          % Allocate output vector for species abundances
    misses = zeros(1,samples);
    for i = 1:samples
        xsamp = [];
        itnum = 0;
        while(length(xsamp) < sampsize)                 % Cycle through and sample individuals
            r = ceil(rand*length(Grp));                 % Randomly sample individual with replacement
            if(isin(Grp(r),NonEluSpp))                  % Determine if sampled individual is elusive or nonelusive
                xsamp = [xsamp Grp(r)];                 % If individual is nonelusive, add to sampled distribution             
            else;
                if(rand > bias)                         % Determine if elusive individual is captured 
                    xsamp = [xsamp Grp(r)];             % If elusive individual if captured, add to sampled distribution
                end;
            end;
            itnum = itnum + 1;                          % Keep track of number of times through while loop
        end;
    
        [u,f] = unique1(xsamp);                         % Convert vector of individuals back to species abundance distbrution
        ObsSAD(u,i) = f;                                % Place abundance of each species in vector to keep track of elusive species     
    
        misses(i) = itnum - sampsize;                   % Determine the number of elusive individuals that were not captured
    end;
     
    
return;