% scheinerMDB: Calculates the integrated multiple dimensions of biodiversity metric of Scheiner 2012, including
%   a single metric that combines taxonomic, functional, and phylogenetic biodiversity as well as
%   individual components for each dimension.
%
%   Usage: [DA, DP, DF, DAP, DAF, DPF, DAPF,L,totvol] = scheinerMDB(edges, edgematrix, functions, community, fweights, q)
%
%       edges =         vector of edge lengths from a phylogenetic tree
%       edgematrix =    an edge x species matrix to associated particular
%                       edges with particular species
%       functions =     S x m matrix of functional values (S = number of
%                       species, m = number of functions)
%       community =     vector of species abundances (default = ones)
%       fweights =      1 x m vector of weights for functional data
%       q =             exponent that determines how relative proportions
%                       are weighed (default = 0)
%       --------------------------------------------------------------------------------------------------------------------------------
%       DA = abundance diversity of Scheiner 2012
%       DP = phylogenetic divergence of Scheiner 2012
%       DF = integrated functional metric of Scheiner 2012
%       DAP = integrated abundance-phylogenetic diverstiy metric of scheiner 2012
%       DAF = integrated abundance-functional diverstiy metric of scheiner 2012
%       DPF = integrated phylogenetic-functional diverstiy metric of scheiner 2012
%       DPF = integrated abundance-phylogenetic-functional diverstiy metric of scheiner 2012
%
%   Steven J. Presley 1/10/13
%

function [DA, DP, DF, DAP, DAF, DPF, DAPF,L,totvol] = scheinerMDB(edges,edgematrix,functions,community,fweights,q)

%error checking
    [E,S] = size(edgematrix);               % determines # of edges(E) and species (S) in community
    [e,m] = size(functions);                % dermines # species (e) and functions (m) in functional data
    if(nargin < 2)
        disp('ERROR: too few input arguments')
    end;
    
    if(nargin < 4)
        community = ones(S,1);
    end;
    
    if(nargin < 5)
        fweights = ones(1,S);
    end;
   
    if(nargin < 6)
        q = 0;
    end;
    
    [er,ec]=size(edges);
    [cr,cc]=size(community);
    
    if(er==1)
        edges=edges';
    end;
    
    if(cc==1)
        community=community';
    end;
    
    s = length(community);                  % determines number of species in community
    r = length(edges);                      % determines number of edges in tree
    M = length(fweights);                   % determines number of traits for which weights are provided
    
       
    if(r ~= E)
        disp('ERROR: number of edges do not match in edges and edgematrix')
    end;
    
    if(s ~= S)
        disp('ERROR: number of species do not match in community and edgematrix')
    end;
    
    if (s ~= e)
        disp('ERROR: number of species in functional data does not match number of species in community data')
    end;
    
    if (M ~= m)
        disp('ERROR: number of weights provided for functions does not match the number of functions')
    end;
    

    edges=edges';
    R = S;                                  % sets richness(R) to equal the number of species
    N = sum(S);                             % calculates total number of individuals
    
    % Calculated phylogenetic diversity of Scheiner
    
    edgelineage = sum(edgematrix');         % determines the number of lineages associated with each edge
    
    L = sum(edges);                         % determines the total lineage divergence
    
    proportionaledges = [];
    
    %divides the divergense of each edge equally among the lineages associated with it
    for i = 1:r
        proportionaledges(1,i)= edges(1,i)/edgelineage(1,i);
    end;
    
    lineagedivergence = zeros(E,S);
    
    for i = 1:r
        for j = 1:S
            lineagedivergence(i,j) = edgematrix(i,j)*proportionaledges(1,i);
        end;
    end;
    
    speciesLD = sum(lineagedivergence);     % sum of divergences for each species (Li of Scheiner)
    speciesPLD = speciesLD/L;               % calculates the proportional divergence of each lineage
    
    DPQ = [];
    
    if (q == 1)
        
        for i=1:S      
            DPQ(1,i) = speciesPLD(1,i)*log(speciesPLD(1,i)); % First part of calculation for DP when q = 1 (limit function)
        end;
        
        DP = exp(-sum(DPQ));                    % Calcuation of phylogenetic diversity (DP) for q = 1
        
    else
        
        for i=1:S
            DPQ(1,i) = speciesPLD(1,i)^q;       % Raises proportional divergenge values from each lineage to the power of q
        end;
    
        DP = (sum(DPQ))^(1/(1-q));              % Calculates phylogenetic diversity (DP)
    end;
    
    
    % Calculation of abundance diversity of Scheiner
    
    N = sum(community);                     % Calculatese total number of individuals in a community
    propabund = community/N;                % Calculates proportional abundances
    
    speciesDAQ = [];
    
    if (q == 1)
        for i=1:S
            speciesDAQ(1,i) = propabund(1,i)*log(propabund(1,i));
        end;
        
        DA = exp(-sum(speciesDAQ));             % Calculates abundance diversity (DA) for q = 1
        
    else
    
        for i=1:S
            speciesDAQ(1,i) = propabund(1,i)^q; % Raises proportional abundances to the power of q
        end;
        
        DA = (sum(speciesDAQ))^(1/(1-q));       % Calculates abundance diversity (DA) of Scheiner 2012
    end;
    
 
    %Calculation of D(AP) of Scheiner
    
    DAPdenom = [];
    
    for i=1:S
        DAPbase(1,i) = community(1,i)*speciesLD(1,i);  %calculates elements (niLi) for D(AP) of Scheiner
    end;
    
    DAPdenom = sum(DAPbase);                           %sums elements to get denominator for D(AP) of scheiner
    
    DAPbase = DAPbase/DAPdenom;
    
    for i=1:S
        DAPbase(1,i)=DAPbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DAPbase(1,i) = DAPbase(1,i)*log(DAPbase(1,i));   %Calculates D(AP) of Scheiner for q=1
        end;
       
        DAP = exp(-sum(DAPbase));
        
    else
        DAPbase = sum(DAPbase);
        DAP = DAPbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
    end;
    
    
    % Calculation of integrated functional diversity of Scheiner
    
    zfunc = zscore(functions);                          % z-score standardization
    
    for i = 1:S
        for j = 1:m
            zfunc(i,j) = zfunc(i,j)*fweights(1,j);       % effect standardization based on a priori functional weighting
        end;
    end;
    
    mindist = zeros(1,S);                               % creates empty vector of minimum functional distances
    
    for i = 1:S
        spfunc = zfunc(i,:);                            % extracts functional values of focal species
        tempfunc = zfunc;                               % creates copy of functional value matrix
        tempfunc(i,:) = [];                             % deletes functional values of focal species from copy of functional matrix
        spfuncdist=zeros(S-1,m);                        % creates matrix for pair-wise functional value differences
        
            for k = 1:m
                for j = 1:S-1
                    spfuncdist(j,k) = (spfunc(1,k)-tempfunc(j,k))^2;   % calculates and squares pair-wise functional differences                  
                end;
            end;
     
        if m == 1
            mindist(1,i) =  min(sqrt(spfuncdist));     
        else
            mindist(1,i) = min(sqrt(sum(spfuncdist')));     % calculates minimum functional distance from ith species to any other species (di of Scheiner)
        end;
    end;
    
    C = (pi^(m/2))/gamma(m/2+1);                        % calculates constant (Cm of Scheiner) based on number of functions
    
    funcvol = zeros(1,S);
    for i = 1:S
        funcvol(1,i) = C*(mindist(1,i)^m);             % calculates fuctional volume of each species
    end;
    funcvol;
    totvol = sum(funcvol);                             % calculates total functional volume of community
    totvol(totvol==0)=1;
    
    propvol = funcvol/totvol;                           % calculates proportional functional volumes
       
    DFQ = [];
    if (q == 1)
        for i=1:S
            DFQ(1,i) = propvol(1,i)*log(propvol(1,i));
        end;
        
        DFQ(isnan(DFQ))=0;
        DF = exp(-sum(DFQ));             % Calculates functional diversity (DF) for q = 1
        
    else
    
        for i=1:S
            DFQ(1,i) = propvol(1,i)^q; % Raises proportional abundances to the power of q
        end;
        
        DF = (sum(DFQ))^(1/(1-q));       % Calculates functional diversity (DF) of Scheiner 2012
        DF(isinf(DF))=1;
    end;
             
    %Calculation of D(AF) of Scheiner
    
    DAFdenom = [];
    
    for i=1:S
        DAFbase(1,i) = community(1,i)*funcvol(1,i);  %calculates elements (nivi) for D(AF) of Scheiner
    end;
    
    DAFdenom = sum(DAFbase);                           %sums elements to get denominator for D(AF) of scheiner
    DAFbase = DAFbase/DAFdenom;
    
    for i=1:S
        DAFbase(1,i)=DAFbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DAFbase(1,i) = DAFbase(1,i)*log(DAFbase(1,i));   %Calculates D(AF) of Scheiner for q=1
        end;
       
        DAFbase(isnan(DAFbase))=0;
        DAF = exp(-sum(DAFbase));
        
    else
        DAFbase = sum(DAFbase);
        DAF = DAFbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
    end;
    
     %Calculation of D(PF) of Scheiner
    
    DPFdenom = [];
    
    for i=1:S
        DPFbase(1,i) = speciesLD(1,i)*funcvol(1,i);  %calculates elements (Livi) for D(PF) of Scheiner
    end;
    
    DPFdenom = sum(DPFbase);                           %sums elements to get denominator for D(PF) of scheiner
    DPFbase = DPFbase/DPFdenom;
    
    for i=1:S
        DPFbase(1,i)=DPFbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DPFbase(1,i) = DPFbase(1,i)*log(DPFbase(1,i));   %Calculates D(AF) of Scheiner for q=1
        end;
       
        DPFbase(isnan(DPFbase))=0;
        DPF = exp(-sum(DPFbase));
        
    else
        DPFbase = sum(DPFbase);
        DPF = DPFbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
        %DPF(isnan(DPF))=1;
    end;
    
    
    %Calculation of D(APF) of Scheiner
    
    DAPFdenom = [];
    
    for i=1:S
        DAPFbase(1,i) = community(1,i)*speciesLD(1,i)*funcvol(1,i);  %calculates elements (niLivi) for D(APF) of Scheiner
    end;
    
    DAPFdenom = sum(DAPFbase);                           %sums elements to get denominator for D(APF) of scheiner
    DAPFbase = DAPFbase/DAPFdenom;
    
    for i=1:S
        DAPFbase(1,i)=DAPFbase(1,i)^q;
    end;
          
    if (q==1)
        for i=1:S
           DAPFbase(1,i) = DAPFbase(1,i)*log(DAPFbase(1,i));   %Calculates D(APF) of Scheiner for q=1
        end;
       
        DAPFbase(isnan(DAPFbase))=0;
        DAPF = exp(-sum(DAPFbase));
        
    else
        DAPFbase = sum(DAPFbase);
        DAPF = DAPFbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
    end;
    
    
return;


