% scheinerMDB: Calculates the integrated multiple dimensions of biodiversity metric of Scheiner 2012, including
%   a single metric that combines taxonomic, functional, and phylogenetic biodiversity as well as
%   individual components for each dimension.
%
%   Usage: [S, N, DA, DP, DFmean, DFmin, DAP, DAFmean, DAFmin, DPFmean, DPFmin, DAPFmean, DAPFmin, L,totvolmean, totvolmin] = scheinerMDBx(edges, edgematrix, traits, community, fweights, q)
%
%       edges =         vector of edge lengths from a phylogenetic tree
%       edgematrix =    an edge x species matrix to associated particular
%                       edges with particular species
%       traits =     S x m matrix of functional values (S = number of
%                       species, m = number of functions)
%       community =     vector of species abundances (default = ones)
%       fweights =      1 x m vector of weights for functional data
%       q =             exponent that determines how relative proportions
%                       are weighed (default = 1)
%       --------------------------------------------------------------------------------------------------------------------------------
%       S = species richness
%       N = total number of individuals
%       DA = abundance diversity of Scheiner 2012
%       DP = phylogenetic divergence of Scheiner 2012
%       DFmin = integrated functional metric of Scheiner 2012 based on
%               minimum functional distance
%       DFmean = integrated functional metric that is parallel to that of 
%               Scheiner 2012, but is based on mean functional distance
%       DAP = integrated abundance-phylogenetic diverstiy metric of 
%             scheiner 2012
%       DAFmin = integrated abundance-functional diverstiy metric of
%                scheiner 2012 based on minimum functional distance
%       DAFmean = integrated abundance-functional diverstiy metric that is
%                 parallel to that of scheiner 2012, but is based on
%                 mean functional distance
%       DPFmin = integrated phylogenetic-functional diverstiy metric of 
%                scheiner 2012
%       DPFmean = integrated phylogenetic-functional diverstiy metric that
%                 is parallel to that of scheiner 2012, but is based on
%                 mean functional distance
%       DAPFmin = integrated abundance-phylogenetic-functional diverstiy 
%                 metric of scheiner 2012
%       DAPFmean = integrated abundance-phylogenetic-functional diverstiy
%                  metric that is parallel to that of scheiner 2012, but 
%                  is based on mean functional distance
%       L = Faith's PD (= total branch length in tree)
%       totvolmin = total functional volume based on minimum functional
%                   distances
%       totvolmean = total functional volume based on mean functional
%                   distances
%
%   Steven J. Presley 10/11/13
%

function [S, N, DA, DP, DFmean, DFmin, DAP, DAFmean, DAFmin, DPFmean, DPFmin, DAPFmean, DAPFmin, L,totvolmean, totvolmin] = scheinerMDBx(edges,edgematrix,traits,community,fweights,q)

%error checking
    [E,S] = size(edgematrix);               % determines # of edges(E) and species (S) in community
    [e,m] = size(traits);                % dermines # species (e) and functions (m) in functional data
    if(nargin < 2)
        disp('ERROR: too few input arguments')
    end;
    
    if(nargin < 4)
        community = ones(S,1);
    end;
    
    if(nargin < 5)
        fweights = ones(1,m);
    end;
   
    if(nargin < 6)
        q = 1;
    end;
    
    [er,ec]=size(edges);
    [cr,cc]=size(community);
    
    if(er==1)
        edges=edges';
    end;
    
    if(cc==1)
        community=community';
    end;
    
    s = length(community);                  % determines number of species in community
    r = length(edges);                      % determines number of edges in tree
    M = length(fweights);                   % determines number of traits for which weights are provided
    
       
    if(r ~= E)
        disp('ERROR: number of edges do not match in edges and edgematrix')
    end;
    
    if(s ~= S)
        disp('ERROR: number of species do not match in community and edgematrix')
    end;
    
    if (s ~= e)
        disp('ERROR: number of species in functional data does not match number of species in community data')
    end;
    
    if (M ~= m)
        disp('ERROR: number of weights provided for functions does not match the number of functions')
    end;
    

    edges=edges';
    R = S;                                  % sets richness(R) to equal the number of species
    N = sum(community);                             % calculates total number of individuals
    
    % Calculated phylogenetic diversity of Scheiner
    
    edgelineage = sum(edgematrix');         % determines the number of lineages associated with each edge
    
    L = sum(edges);                         % determines the total lineage divergence
    
    proportionaledges = [];
    
    %divides the divergence of each edge equally among the lineages associated with it
    for i = 1:r
        proportionaledges(1,i)= edges(1,i)/edgelineage(1,i);
    end;
    
    lineagedivergence = zeros(E,S);
    
    for i = 1:r
        for j = 1:S
            lineagedivergence(i,j) = edgematrix(i,j)*proportionaledges(1,i);
        end;
    end;
    
    speciesLD = sum(lineagedivergence);     % sum of divergences for each species (Li of Scheiner)
    speciesPLD = speciesLD/L;               % calculates the proportional divergence of each lineage
    
    DPQ = [];
    
    if (q == 1)
        
        for i=1:S      
            DPQ(1,i) = speciesPLD(1,i)*log(speciesPLD(1,i)); % First part of calculation for DP when q = 1 (limit function)
        end;
        
        DP = exp(-sum(DPQ));                    % Calcuation of phylogenetic diversity (DP) for q = 1
        
    else
        
        for i=1:S
            DPQ(1,i) = speciesPLD(1,i)^q;       % Raises proportional divergenge values from each lineage to the power of q
        end;
    
        DP = (sum(DPQ))^(1/(1-q));              % Calculates phylogenetic diversity (DP)
    end;
    
    
    % Calculation of abundance diversity of Scheiner
    
    propabund = community/N;                % Calculates proportional abundances
    
    speciesDAQ = [];
    
    if (q == 1)
        for i=1:S
            speciesDAQ(1,i) = propabund(1,i)*log(propabund(1,i));
        end;
        
        DA = exp(-sum(speciesDAQ));             % Calculates abundance diversity (DA) for q = 1
        
    else
    
        for i=1:S
            speciesDAQ(1,i) = propabund(1,i)^q; % Raises proportional abundances to the power of q
        end;
        
        DA = (sum(speciesDAQ))^(1/(1-q));       % Calculates abundance diversity (DA) of Scheiner 2012
    end;
    
 
    %Calculation of D(AP) of Scheiner
    
    DAPdenom = [];
    
    for i=1:S
        DAPbase(1,i) = community(1,i)*speciesLD(1,i);  %calculates elements (niLi) for D(AP) of Scheiner
    end;
    
    DAPdenom = sum(DAPbase);                           %sums elements to get denominator for D(AP) of scheiner
    
    DAPbase = DAPbase/DAPdenom;
    
    for i=1:S
        DAPbase(1,i)=DAPbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DAPbase(1,i) = DAPbase(1,i)*log(DAPbase(1,i));   %Calculates D(AP) of Scheiner for q=1
        end;
       
        DAP = exp(-sum(DAPbase));
        
    else
        DAPbase = sum(DAPbase);
        DAP = DAPbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
    end;
    
    
    % Calculation of integrated functional diversity of Scheiner based on minimum distances
    
    zfunc = zscore(traits);                          % z-score standardization
    
    for i = 1:S
        for j = 1:m
            zfunc(i,j) = zfunc(i,j)*fweights(1,j);       % effect standardization based on a priori functional weighting
        end;
    end;
    
    mindist = zeros(1,S);                               % creates empty vector of minimum functional distances
    
    for i = 1:S
        spfunc = zfunc(i,:);                            % extracts functional values of focal species
        tempfunc = zfunc;                               % creates copy of functional value matrix
        tempfunc(i,:) = [];                             % deletes functional values of focal species from copy of functional matrix
        spfuncdist=zeros(S-1,m);                        % creates matrix for pair-wise functional value differences
        
            for k = 1:m
                for j = 1:S-1
                    spfuncdist(j,k) = (spfunc(1,k)-tempfunc(j,k))^2;   % calculates and squares pair-wise functional differences                  
                end;
            end;
     
        if m == 1
            mindist(1,i) =  min(sqrt(spfuncdist));     
        else
            mindist(1,i) = min(sqrt(sum(spfuncdist')));     % calculates minimum functional distance from ith species to any other species (di of Scheiner)
        end;
        
    end;
    
    
    C = (pi^(m/2))/gamma(m/2+1);                        % calculates constant (Cm of Scheiner) based on number of functions
    
    funcvolmin = zeros(1,S);
    for i = 1:S
        funcvolmin(1,i) = C*(mindist(1,i)^m);             % calculates fuctional volume of each species
    end;
    
    totvolmin = sum(funcvolmin);                             % calculates total functional volume of community
    totvolmin(totvolmin==0)=1;
    
    propvolmin = funcvolmin/totvolmin;                           % calculates proportional functional volumes
       
    DFQ = [];
    if (q == 1)
        for i=1:S
            DFQ(1,i) = propvolmin(1,i)*log(propvolmin(1,i));
        end;
        DFQ;
        DFQ(isnan(DFQ))=0;
        DFmin = exp(-sum(DFQ));             % Calculates functional diversity (DF) for q = 1
        
    else
    
        for i=1:S
            DFQ(1,i) = propvolmin(1,i)^q; % Raises proportional abundances to the power of q
        end;
        
        DFmin = (sum(DFQ))^(1/(1-q));       % Calculates functional diversity (DF) of Scheiner 2012
        DFmin(isinf(DFmin))=1;
    end;
    
    
    % Calculation of integrated functional diversity of Scheiner based on mean distances
     
    meandist = zeros(1,S);                               % creates empty vector of mean functional distances
    
    for i = 1:S
        spfunc = zfunc(i,:);                            % extracts functional values of focal species
        tempfunc = zfunc;                               % creates copy of functional value matrix
        tempfunc(i,:) = [];                             % deletes functional values of focal species from copy of functional matrix
        spfuncdist=zeros(S-1,m);                        % creates matrix for pair-wise functional value differences
        
            for k = 1:m
                for j = 1:S-1
                    spfuncdist(j,k) = (spfunc(1,k)-tempfunc(j,k))^2;   % calculates and squares pair-wise functional differences                  
                end;
            end;
     
        if m == 1
            meandist(1,i) =  mean(sqrt(spfuncdist));     
        else
            meandist(1,i) = mean(sqrt(sum(spfuncdist')));     % calculates minimum functional distance from ith species to any other species (di of Scheiner)
        end;
    end;
    meandist';
    C = (pi^(m/2))/gamma(m/2+1);                        % calculates constant (Cm of Scheiner) based on number of functions
    
    funcvolmean = zeros(1,S);
    for i = 1:S
        funcvolmean(1,i) = C*(meandist(1,i)^m);             % calculates fuctional volume of each species
    end;
    
    totvolmean = sum(funcvolmean);                             % calculates total functional volume of community
    totvolmean(totvolmean==0)=1;
    
    propvolmean = funcvolmean/totvolmean;                           % calculates proportional functional volumes
       
    DFQ = [];
    if (q == 1)
        for i=1:S
            DFQ(1,i) = propvolmean(1,i)*log(propvolmean(1,i));
        end;
        
        DFQ(isnan(DFQ))=0;
        DFmean = exp(-sum(DFQ));             % Calculates functional diversity (DF) for q = 1
        
    else
    
        for i=1:S
            DFQ(1,i) = propvolmean(1,i)^q; % Raises proportional abundances to the power of q
        end;
        
        DFmean = (sum(DFQ))^(1/(1-q));       % Calculates functional diversity (DF) of Scheiner 2012
        DFmean(isinf(DFmean))=1;
    end;
             
        
    
    %Calculation of D(AF) of Scheiner usind D(F) based on minimum distances
    
    DAFmindenom = [];
    
    for i=1:S
        DAFminbase(1,i) = community(1,i)*funcvolmin(1,i);  %calculates elements (nivi) for D(AF) of Scheiner
    end;
    
    DAFmindenom = sum(DAFminbase);                           %sums elements to get denominator for D(AF) of scheiner
    DAFminbase = DAFminbase/DAFmindenom;
    
    for i=1:S
        DAFminbase(1,i)=DAFminbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DAFminbase(1,i) = DAFminbase(1,i)*log(DAFminbase(1,i));   %Calculates D(AF) of Scheiner for q=1
        end;
       
        DAFminbase(isnan(DAFminbase))=0;
        DAFmin = exp(-sum(DAFminbase));
        
    else
        DAFminbase = sum(DAFminbase);
        DAFmin = DAFminbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
    end;
    
    
       %Calculation of D(AF) of Scheiner usind D(F) based on mean distances
    
    DAFmeandenom = [];
    
    for i=1:S
        DAFmeanbase(1,i) = community(1,i)*funcvolmean(1,i);  %calculates elements (nivi) for D(AF) of Scheiner
    end;
    
    DAFmeandenom = sum(DAFmeanbase);                           %sums elements to get denominator for D(AF) of scheiner
    DAFmeanbase = DAFmeanbase/DAFmeandenom;
    
    for i=1:S
        DAFmeanbase(1,i)=DAFmeanbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DAFmeanbase(1,i) = DAFmeanbase(1,i)*log(DAFmeanbase(1,i));   %Calculates D(AF) of Scheiner for q=1
        end;
       
        DAFmeanbase(isnan(DAFmeanbase))=0;
        DAFmean = exp(-sum(DAFmeanbase));
        
    else
        DAFmeanbase = sum(DAFmeanbase);
        DAFmean = DAFmeanbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
    end;
    
     %Calculation of D(PF) of Scheiner based on D(F) using minimum distances
    
    DPFmindenom = [];
    
    for i=1:S
        DPFminbase(1,i) = speciesLD(1,i)*funcvolmin(1,i);  %calculates elements (Livi) for D(PF) of Scheiner
    end;
    
    DPFmindenom = sum(DPFminbase);                           %sums elements to get denominator for D(PF) of scheiner
    DPFminbase = DPFminbase/DPFmindenom;
    
    for i=1:S
        DPFminbase(1,i)=DPFminbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DPFminbase(1,i) = DPFminbase(1,i)*log(DPFminbase(1,i));   %Calculates D(AF) of Scheiner for q=1
        end;
       
        DPFminbase(isnan(DPFminbase))=0;
        DPFmin = exp(-sum(DPFminbase));
        
    else
        DPFminbase = sum(DPFminbase);
        DPFmin = DPFminbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
        %DPFmin(isnan(DPF))=1;
    end;
    
    
    %Calculation of D(PF) of Scheiner based on D(F) using mean distances
    
    DPFmeandenom = [];
    
    for i=1:S
        DPFmeanbase(1,i) = speciesLD(1,i)*funcvolmean(1,i);  %calculates elements (Livi) for D(PF) of Scheiner
    end;
    
    DPFmeandenom = sum(DPFmeanbase);                           %sums elements to get denominator for D(PF) of scheiner
    DPFmeanbase = DPFmeanbase/DPFmeandenom;
    
    for i=1:S
        DPFmeanbase(1,i)=DPFmeanbase(1,i)^q;
    end;
    
       
    if (q==1)
        for i=1:S
           DPFmeanbase(1,i) = DPFmeanbase(1,i)*log(DPFmeanbase(1,i));   %Calculates D(AF) of Scheiner for q=1
        end;
       
        DPFmeanbase(isnan(DPFmeanbase))=0;
        DPFmean = exp(-sum(DPFmeanbase));
        
    else
        DPFmeanbase = sum(DPFmeanbase);
        DPFmean = DPFmeanbase^(1/(1-q));                        %Calculates D(AP) of Scheiner for q <> 1
        %DPFmean(isnan(DPF))=1;
    end;
    
    
    %Calculation of D(APF) of Scheiner using D(F) based on minimum distances
    
    DAPFmindenom = [];
    
    for i=1:S
        DAPFminbase(1,i) = community(1,i)*speciesLD(1,i)*funcvolmin(1,i);  %calculates elements (niLivi) for D(APF) of Scheiner
    end;
    
    DAPFmindenom = sum(DAPFminbase);                           %sums elements to get denominator for D(APF) of scheiner
    DAPFminbase = DAPFminbase/DAPFmindenom;
    
    for i=1:S
        DAPFminbase(1,i)=DAPFminbase(1,i)^q;
    end;
          
    if (q==1)
        for i=1:S
           DAPFminbase(1,i) = DAPFminbase(1,i)*log(DAPFminbase(1,i));   %Calculates D(APF) of Scheiner for q=1
        end;
       
        DAPFminbase(isnan(DAPFminbase))=0;
        DAPFmin = exp(-sum(DAPFminbase));
        
    else
        DAPFminbase = sum(DAPFminbase);
        DAPFmin = DAPFminbase^(1/(1-q));                        %Calculates D(APF) of Scheiner for q <> 1
    end;
    
    
        %Calculation of D(APF) of Scheiner using D(F) based on mean distances
    
    DAPFmeandenom = [];
    
    for i=1:S
        DAPFmeanbase(1,i) = community(1,i)*speciesLD(1,i)*funcvolmean(1,i);  %calculates elements (niLivi) for D(APF) of Scheiner
    end;
    
    DAPFmeandenom = sum(DAPFmeanbase);                           %sums elements to get denominator for D(APF) of scheiner
    DAPFmeanbase = DAPFmeanbase/DAPFmeandenom;
    
    for i=1:S
        DAPFmeanbase(1,i)=DAPFmeanbase(1,i)^q;
    end;
          
    if (q==1)
        for i=1:S
           DAPFmeanbase(1,i) = DAPFmeanbase(1,i)*log(DAPFmeanbase(1,i));   %Calculates D(APF) of Scheiner for q=1
        end;
       
        DAPFmeanbase(isnan(DAPFmeanbase))=0;
        DAPFmean = exp(-sum(DAPFmeanbase));
        
    else
        DAPFmeanbase = sum(DAPFmeanbase);
        DAPFmean = DAPFmeanbase^(1/(1-q));                        %Calculates D(APF) of Scheiner for q <> 1
    end;
    
    
return;


