% TURNOVER: This function asseses species turnover from  a community and range perspective.
%
%   USAGE: [Re,Rpr,MR,SR] = turnover(X,ord,Comm,iter,ax)
%
%       X = [sites x species] incidence matrix
%       ord = optional, specifies how to ordinate the data matrix
%           0 = no ordination is performed 
%           1 = reciprical averaging {default}
%           2 = maximally packed as in nestedness calcualtor
%           3 = species richness and occurence totals
%       Comm = optional, specifies whether to look at the Community (1) or Range (0) perspective {default = 0}
%       iter = optional, specifies the number of iterations to perform {default = 200}
%       ax = optional, specifies which axis of correspondance to use in ordination {default = 1)
%       ----------------------------------------------------------------------------------------------------------------------
%       Re = number of replacemnets (checkerboard)
%       Rpr = significance value for a two-tailed test --- less turnover (nested) or more turnover than expected by chance
%       MR = mean number of replacements base on null models
%       SR = standard deviation of number of repalcements based on null models
%
%   Chris L. Higgins 3/14/2005
%
%   Leibold,M.A. and Mikkelson,G.M.  2002.  Coherence, species turnover, and boundary clumping: elements 
%       of meta-community structure.  Oikos 97: 237-250.
%
% REQUIRES: makebinary, allzeros, ordinate, boundfill, checkstat, replacenull
%

function [Re,Rpr,MR,SR] = turnover(X,ord,Comm,iter,ax)

    if(nargin < 2)                                      % Set default input arguements
        ord = [];
    end;
    if(nargin < 3)
        Comm = [];
    end;
    if(nargin < 4)
        iter = [];
    end;
    if(nargin < 5)
        ax = [];
    end;
    
    if(isempty(ord))
        ord = 1;
    end;
    if(isempty(Comm))
        Comm = 0;
    end;
    if(isempty(iter))
        iter = 200;
    end;
    if(isempty(ax))
        ax = 1;
    end;

    X = makebinary(X);                                  % Make sure the data matrix is a presence-absence matrix
    
    switch(ord)                                         % Allow various kinds of ordination of data matrix
        case 0
            Y = X;                                      % Do not ordinate the data matrix
        case 1
            X = allzeros(X);                            % Remove any rows or columns that contain all zeros
            Y = ordinate(X,ax);                         % Ordinate the data matrix by reciprical averaging
        case 2
            Y = maxpack(X);                             % Ordinate the data matrix by maximumally packing it (nested calculator)
        case 3
            Y = nestsort(X);                            % Ordinate data matrix by species richness and occurence values
    end;

    Y = boundfill(Y,Comm);                              % Find boundaries for rows and columns

    Re = checkstat(Y);                                  % Count the number of checkerboards - my algorithm -- much faster
    
    if(iter > 0)
        for i = 1:iter                                  % Cycle through iterations
            Null = replacenull(Y,Comm);                 % Create null model
            
% They do not ordinate null matrix before calculating number of replacements!!!!!            
%             switch(ord)                                 % Allow various kinds of ordination of data matrix
%                 case 0
%                     EY = Null;                          % Do not ordinate the data matrix
%                 case 1
%                     EX = allzeros(Null);                % Remove any rows or columns that contain all zeros
%                     EY = ordinate(EX);                  % Ordinate the data matrix by reciprical averaging
%                 case 2
%                     EY = maxpack(Null);                 % Ordinate the data matrix by maximumally packing it (nested calculator)
%                 case 3
%                     EY = nestsort(Null);                % Ordinate data matrix by species richness and occurence values
%             end;
            
            ERe(i) = checkstat(Null);
        end;

        MR = mean(ERe);                         
        SR = std(ERe);
        [H,Rpr] = ztest(Re,MR,SR,.05,0);                % Perform two-tailed z-test comparing null distrbution to observed
    end;
    
return;